/*
 * The program is refered from iptables.c, iptables-standalone.c and iptabled-restore.c
 * The main purpose is to prevent from calling the iptc_init() and iptc_commit() while every one iptables command is issued.
 * It can reduce the timing a lot.
 *
 * NOTE:
 * It is not allowed to have more than two different cur-processing. It will cause the 2nd iptc_commit() fail.
 *
*/
                                                                                                                                               

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <iptables.h>
#include <getopt.h>
#include <sys/errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <syslog.h>
#include <stdarg.h>
#include <signal.h>

#define READ_CONF 0

#ifdef _DPRINTF_
FILE* TTYS0;
#define DPRINTF(args...) \
        TTYS0=fopen("/dev/ttyS0","w"); \
        fprintf(TTYS0,args); \
        fclose(TTYS0)
                                                                                                                                               
#define JP(args...) \
        TTYS0=fopen("/dev/ttyS0","w"); \
        fprintf(TTYS0,"\n--\ntest:%d\n--\n",args); \
        fclose(TTYS0)
#else
#define DPRINTF(args...)
#endif /* _DPRINTF_ */

static char *newargv[64]={0};
static int newargc;
static char buffer[256];

/* function adding one argument to newargv, updating newargc
 * returns true if argument added, false otherwise */
static int add_argv(char *what) {
        if (what && ((newargc + 1) < sizeof(newargv)/sizeof(char *))) {
                newargv[newargc] = strdup(what);
                newargc++;
                return 1;
        } else
                return 0;
}
                                                                                                                                               
static void free_argv(void) {
        int i;
                                                                                                                                               
        for (i = 0; i < newargc; i++)
                free(newargv[i]);
}

static char filter_table[] = "filter";
static char nat_table[] = "nat";
static char mangle_table[] = "mangle";

void iptableScriptInstall(char *iptableScriptFile)
{
        char *cur_table;
        char *pre_table ;
        iptc_handle_t handle=NULL;
        char /*buffer[256],*/ *buf = NULL;
        int ret;
        FILE *in=NULL;
        char *param_start=NULL, *curchar=NULL;
        int quote_open, table_id;

        memset(buffer,0,sizeof(buffer));

        in = fopen(iptableScriptFile, "r");
        if (!in) {
             fprintf(stderr, "Can't open %s: %s", iptableScriptFile, strerror(errno));
             return;
        }
        /* default table */
        cur_table = pre_table = filter_table;

        DPRINTF("######## start  %s  #########\n",iptableScriptFile);
        while (fgets(buffer, sizeof(buffer), in)) {
                if (buffer[0] == '\n')
                        continue;
                /* skip the first space character first */
                for (buf = buffer; (*buf) && (*buf == ' ' || *buf == '\t' || *buf == '\n'); buf++);
                if (!(*buf) || *buf == '\n')
                        continue; /* empty line */
                                                                                                                                               
                if (*buf == '#')  /* comment; skip */
                        continue;
                                                                                                                                               
                if (strncmp(buf, "iptables ", 9))
                        continue; /* does not start with iptables */
                                                                                                                                               
                // reset the counter
                newargc = quote_open = table_id = 0;
                param_start = buf;
               for (curchar = buf; *curchar; curchar++) {
                        if (*curchar == '"') {
                                /* quote_open cannot be true if there
                                 * was no previous character.  Thus,
                                 * curchar-1 has to be within bounds */
                                if (quote_open &&
                                    *(curchar-1) != '\\') {
                                        quote_open = 0;
                                        *curchar = ' ';
                                } else {
                                        quote_open = 1;
                                        param_start++;
                                }
                        }
                        if (*curchar == ' '
                            || *curchar == '\t'
                            || * curchar == '\n') {
                                char param_buffer[256];
                                int param_len = curchar-param_start;
                                  char valueBuf[60];
                                                                                                                                               
                                if (quote_open)
                                        continue;
                                                                                                                                               
                                if (!param_len) {
                                        /* two spaces? */
                                        param_start++;
                                        continue;
                                }
                                                                                                                                               
                                /* end of one parameter */
                                strncpy(param_buffer, param_start,
                                        param_len);
                                *(param_buffer+param_len) = '\0';
                                param_start += param_len + 1;

                                /* check if table name specified */
                                if (!strncmp(param_buffer, "-t", 3)
                                   || !strncmp(param_buffer, "--table", 8)) {
                                        /* table specified */
                                        table_id = 1;
                                        add_argv(param_buffer);
                                        continue;
                                }
                                                                                                                                               
                                if (table_id==1) {
                                        if (!strcmp(param_buffer, "filter")) {
                                                cur_table = filter_table;
                                        } else if (!strcmp(param_buffer, "nat")) {
                                                cur_table = nat_table;
                                        } else if (!strcmp(param_buffer, "mangle")) {
                                                cur_table = mangle_table;
                                        }
                                        /* reset the id */
                                        table_id = 0;
                                }

#if READ_CONF
                                /* check whether it is the variable, if yes, update it to the correct value */
                                if((param_buffer[0]=='$') &&// start with a $ sign
                                     (findParmVal(param_buffer+1,valueBuf))) //skip "$"
                                {
                                        add_argv(valueBuf);
                                }
                                else
#endif
                                        add_argv(param_buffer);
                        } /* endof of para */
                } /* end of for */

		if (handle == NULL) // the first time
		{
			handle = iptc_init(cur_table);
		}
		else if(pre_table != cur_table) // not the first time
		{
//			if (!iptc_commit(&handle))
//			    DPRINTF("Can't commit %s", iptc_strerror(errno));
			iptc_commit(&handle);
//			DPRINTF("after commit\n");
			handle = iptc_init(cur_table);
		}
                ret = do_command(newargc, newargv, &cur_table, &handle);
                if (!ret){
                        DPRINTF("%s : failed=> %s\n", buffer, iptc_strerror(errno));
                }
                                                                                                                                               
                                                                                                                                               
                free_argv();
                /* restore to the default table */
		pre_table = cur_table;
                cur_table = filter_table;
        }
        fclose(in);
                                                                                                                                               
        /* write to the tables */
        if (handle) {
//                if (!iptc_commit(&handle))
//                        printf("Can't commit %s", iptc_strerror(errno));
		iptc_commit(&handle);
        }
}

#if READ_CONF
#define VAR_ETC_STATUS_FILE "/var/etc/status"
#define MAX_CONFIG_ITEM         60
#define MAX_STRING_LEN          20
                                                                                                                                               
char configName [MAX_CONFIG_ITEM][MAX_STRING_LEN+1]={{0}};
char configValue[MAX_CONFIG_ITEM][MAX_STRING_LEN+1]={{0}};
int configItem=0;

int GetValue(char *name, char *value)
{
        int i;
        for (i=0; i<configItem; i++) {
                if (!strcmp(configName[i],name))
                        break;
        }
        if (i>=configItem) {
printf("can not found :%s \n", name);
                *value = '\0';
                return -1;
        }
        else {
                strncpy(value, configValue[i], MAX_STRING_LEN);
                return i;
        }
                                                                                                                                               
}
                                                                                                                                               
void SetValue(char *name, char *value)
{
        int i;
                                                                                                                                               
        for (i=0; i<configItem; i++) {
                if (!strcmp(configName[i],name))
                        break;
        }
        if (i>=configItem) { /* add it */
                strncpy(configName[configItem], name, MAX_STRING_LEN);
                strncpy(configValue[configItem], value, MAX_STRING_LEN);
                configItem++;
        }
        else {
                strncpy(configValue[i], value, MAX_STRING_LEN);
        }
 }
                                                                                                                                               


int findParmVal(char *param_buffer,char *valueBuffer)
{
        // what if $LAN_IP/$LAN_MASK
        char *p2=NULL;
                                                                                                                                               
        if((p2=strstr(param_buffer,"/$")))
        {
                p2[0]='\0';// null string for p1;
                // there are 2 parameters in the buffer
                p2+=2;//skip "/$"
        }
        if(GetValue(param_buffer,valueBuffer)>=0)
        {
                if(p2)
                {
                        char value2[30];
                                                                                                                                               
                        if(GetValue(p2,value2)<0)
                        {
                                return 0;
                        }
                        strcat(valueBuffer,"/");
                        strcat(valueBuffer,value2);
                }
                return 1;
        }
        return 0;// not find
}
static int ParseNameValue(char *buf, char **name, char **value)
{
        char *tmp;
        char in_quote=0;
                                                                                                                                               
        /* skip the leading space */
        for (; *buf && (*buf == '\t' || *buf == ' '); buf++) ;
                                                                                                                                               
        if (!(*buf) || *buf == '\n' || // empty line
            *buf == '#')  // comment
                return -1;
                                                                                                                                               
        /* find name */
        *name = buf;
        for (; *buf && *buf != ' ' && *buf != '\t' &&
                   *buf != '=' && *buf != '\n'; buf++);
        tmp = buf;
                                                                                                                                               
        /* find the operator */
        /* skip the leading space */
        for (; *buf && (*buf == '\t' || *buf == ' '); buf++);
        if (*buf != '=')
                return -1;
        *tmp = '\0';
                                                                                                                                               
        /* find the value */
        /* skip the leading space */
        for (buf++; *buf && (*buf == '\t' || *buf == ' '); buf++);
        if (*buf == '"') {
                in_quote = 1;
        }
        if (in_quote) {
                for (*value=++buf; *buf && *buf != '"' && *buf != '\n'; buf++);
        }
        else {
                for (*value=buf; *buf && *buf != ' ' && *buf != '\t' && *buf != '\n'; buf++);
                                                                                                                                               
        }
        *buf = '\0';
        return 0;
}

static void ReadSysStatus()
{
        FILE    *in;
        char    buffer[80];
        char    *name, *value;
                                                                                                                                               
        in = fopen(VAR_ETC_STATUS_FILE, "r");
        if (!in) { printf("Read SysStatus file fail ..\n"); exit(-1); }
        while (fgets(buffer, sizeof(buffer), in)) {
                if (!ParseNameValue(buffer, &name, &value))
                        SetValue(name,value);
        }
                                                                                                                                               
        fclose(in);
}

#endif

int
iptables_script_main(int argc, char *argv[])
{
        program_name = "iptables_script";
        program_version = IPTABLES_VERSION;
	if (argc != 2) {
                fprintf(stderr,"iptables_script <script files>\n");
                _exit (-1);
        }

#ifdef NO_SHARED_LIBS
        init_extensions();
#endif

#if READ_CONF
	ReadSysStatus();
#endif
	iptableScriptInstall(argv[1]);
	exit(0);
}

extern int iptables_main(int argc, char *argv[]);

int
main(int argc, char *argv[])
{
	char *app_name, *p;

	app_name = ((p = strrchr (argv[0], '/')) ? ++p : argv[0]);
	if (!strcmp(app_name, "iptables_script"))
		iptables_script_main(argc, argv);
	else if (!strcmp(app_name, "iptables"))
		iptables_main(argc, argv);
	else
	     fprintf(stderr, "wrong appication name : %s ..\n", app_name);

	exit(0);
}
