#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/proc_fs.h>
#include <linux/sched.h>
#include <asm/uaccess.h>
#include <asm/tc3162/gpio.h>
//#include <asm/arch/platform.h>
//#include <asm/leds.h>

#define MODULE_VERSION "1.0"
#define MODULE_NAME "GPIOUSER"

#define DATA_LEN 8
#define GPIO_LED_ON   0
#define GPIO_LED_OFF  1

#define LED_NO_BLICK	0
#define LED_FAST_BLICK	1
#define LED_MED_BLICK   2
#define LED_SLOW_BLICK  3

#define MAX_GPIO_ENTRY 	15
static struct gpioStr {
	int gpio;
	char name[10];
	int direction; 
	int mode; //0, does not need to create the proc entry
	struct proc_dir_entry *procEntry;
} gpioTbl[MAX_GPIO_ENTRY] = GPIO_VALUE;
/**
{ // refer to tc3162/gpio.h
	{ GPIO_LAN_RST, "lanreset", GPIO_MODE_OUTPUT, 0222, NULL},
	{ GPIO_ADSL_LNK, "adslled", GPIO_MODE_OUTPUT, 0222, NULL},
	{ GPIO_IP_OK, "ipokled", GPIO_MODE_OUTPUT, 0222, NULL},
	{ GPIO_IP_NG, "ipngled", GPIO_MODE_OUTPUT, 0222, NULL},
	{ GPIO_SYS_OK, "sysokled", GPIO_MODE_OUTPUT, 0222, NULL},
	{ GPIO_SYS_NG, "sysngled", GPIO_MODE_OUTPUT, 0222, NULL},
	{ GPIO_RESET, "reset", GPIO_MODE_INPUT, 0444, NULL},
	{ GPIO_3G_LNK, "3gled", GPIO_MODE_OUTPUT, 0222, NULL},
	{ GPIO_WIRELESS_LED, "wlanled", GPIO_MODE_OUTPUT, 0222, NULL},
	{ GPIO_WIRELESS_CONFIG, "wps", GPIO_MODE_INPUT, 0444, NULL},
	{ NULL, NULL, NULL, NULL, NULL}, 
};
**/

static void set_led_flick(int gpio_pin, int freq);
static void set_gpio_output(int gpio_pin,int value);
// export led control function
void ledTurnOff(int gpio)
{
    set_gpio_output(gpio,GPIO_LED_OFF);
    set_led_flick(gpio, LED_NO_BLICK);
}

void ledTurnOn(int gpio)
{
    set_gpio_output(gpio,GPIO_LED_ON);
    set_led_flick(gpio, LED_NO_BLICK);
}

void ledTurnBlick(int gpio)
{
     set_gpio_output(gpio,GPIO_LED_OFF);
     set_led_flick(gpio, LED_FAST_BLICK);
}

void system_fail()
{
#ifdef GPIO_SYS_NG
    ledTurnBlick(GPIO_SYS_NG);
#endif
}
// ----


static struct proc_dir_entry *proc_dir = NULL;

static int get_gpio_input(int gpio_pin)
{
	int j;
	j = *(volatile unsigned short*)TC3162_GPDATA_REG;
	if ((j & (1<<(gpio_pin/2))))
	{
		return 1;
	}
	else
		return 0;
}

static void set_gpio_output(int gpio_pin,int value)
{
	unsigned short j;
	j = *(volatile unsigned short*)TC3162_GPDATA_REG;
	if (value)
	{
		j |= (1<<(gpio_pin/2));
		
	}
	else
	{
		j &= ~(1<<(gpio_pin/2));
	}
	*(volatile unsigned short*)TC3162_GPDATA_REG = j;
}

void set_gpio_mode(int gpio_pin, int gpio_mode)
{
	unsigned int j;
	unsigned short k;
	j = *(volatile unsigned int*)(TC3162_GPCTL_REG);
 	k = *(volatile unsigned short*)TC3162_GPOPENDRAIN_REG;
	if(gpio_mode == GPIO_MODE_INPUT)
	{
		j &= ~(1 << gpio_pin);
		k &= ~(1 << (gpio_pin/2));
	}
	else {
		j |= (1 << gpio_pin);
		k |= (1 << (gpio_pin/2));
		
 	}
	*(volatile unsigned int*)TC3162_GPCTL_REG = j;
	*(volatile unsigned short*)TC3162_GPOPENDRAIN_REG = k;
}

static void set_led_flick(int gpio_pin, int freq)
{
	unsigned int j;
	j = *(volatile unsigned int*)(TC3162_GPOUTPUT_LED_REG);
	j &= ~(1 << gpio_pin);
	if (freq)
	    j |= (freq << gpio_pin);
	*(volatile unsigned int*)TC3162_GPOUTPUT_LED_REG=j;
}

static int proc_read_data(char *page, char **start,
                            off_t off, int count, 
                            int *eof, void *data)
{
	struct gpioStr *entry = (struct gpioStr *) data;
	int index;
	
        MOD_INC_USE_COUNT;
        
        index = sprintf(page, "%d\n", get_gpio_input(entry->gpio));

        MOD_DEC_USE_COUNT;

        return index;
}


static int proc_write_data(struct file *file,
                             const char *buffer,
                             unsigned long count, 
                             void *data)
{
        int len,i,j;
	struct gpioStr *entry = (struct gpioStr *) data;
	char tmpbuf[DATA_LEN];
	int value;

        MOD_INC_USE_COUNT;

        if(count > DATA_LEN)
                len = DATA_LEN;
        else
                len = count;

        if(copy_from_user(tmpbuf, buffer, len)) {
                MOD_DEC_USE_COUNT;
                return -EFAULT;
        }

        tmpbuf[len] = '\0';
	sscanf(tmpbuf,"%d",&value);

        MOD_DEC_USE_COUNT;
 	
	switch(value)
	{
		case 1://led on
		ledTurnOn(entry->gpio);
		break;
		case 2:// flash
		ledTurnBlick(entry->gpio);
		break;
		case 3:
		set_gpio_mode(entry->gpio, GPIO_MODE_OUTPUT);
		break;
		case 4:
		set_gpio_mode(entry->gpio, GPIO_MODE_INPUT);
		break;
		default://led off
		ledTurnOff(entry->gpio);
		break;
	}
	
        return len;
}


static int __init init_gpioUser(void)
{

 	int i;

        /* create directory */
        proc_dir = proc_mkdir(MODULE_NAME, NULL);
        if(proc_dir == NULL) {
                return -ENOMEM;
        }
	proc_dir->owner = THIS_MODULE;

	for (i=0; i<MAX_GPIO_ENTRY; ++i)
	{
	    if (gpioTbl[i].mode== NULL) break;
//	    if (gpioTbl[i].gpio == 0xff) break;

	    set_gpio_mode(gpioTbl[i].gpio,gpioTbl[i].direction);
            if (gpioTbl[i].mode)
	    {
                gpioTbl[i].procEntry = create_proc_entry(gpioTbl[i].name, gpioTbl[i].mode, proc_dir);	    
	  	if (!gpioTbl[i].procEntry) continue;
	 	gpioTbl[i].procEntry->data = &gpioTbl[i];
		gpioTbl[i].procEntry->read_proc = proc_read_data;
		gpioTbl[i].procEntry->write_proc = proc_write_data;
		gpioTbl[i].procEntry->owner = THIS_MODULE;
	    }
	}

	// turn on the sysok; turn off sysng
#ifdef GPIO_SYS_OK
	ledTurnBlick(GPIO_SYS_OK);
#endif 
#ifdef GPIO_SYS_NG 
	ledTurnOff(GPIO_SYS_NG);
#endif
#ifdef GPIO_ADSL_LNK
	ledTurnOff(GPIO_ADSL_LNK);
#endif
#ifdef GPIO_IP_OK
	ledTurnOff(GPIO_IP_OK);
#endif
#ifdef GPIO_IP_NG
	ledTurnOff(GPIO_IP_NG);
#endif

	// set the LED frequency
	*(volatile unsigned short*)TC3162_GPLED_DP_REG=0xff;

        return 0;
}


static void __exit cleanup_gpioUser(void)
{
	int i;

	for (i=0; i<MAX_GPIO_ENTRY; ++i)
	{
	    if (gpioTbl[i].name==NULL) break;

	    if (gpioTbl[i].procEntry)
	        remove_proc_entry(gpioTbl[i].name, proc_dir);
	}
	
        remove_proc_entry(MODULE_NAME, NULL);
}

module_init(init_gpioUser);
module_exit(cleanup_gpioUser);

MODULE_AUTHOR("Robert Wu, Billion Co.");
MODULE_DESCRIPTION("Provide User Led & Resetpin Control");

EXPORT_SYMBOL(ledTurnBlick);
EXPORT_SYMBOL(ledTurnOff);
EXPORT_SYMBOL(ledTurnOn);
