
/*  
 * Definitions used to for UART module.
 *
 */
 
#ifndef __TC3162_SIO_H__ 
#define __TC3162_SIO_H__ 

#include <asm/addrspace.h>

#define		TC3162_UART_BASE    (KSEG1 + 0x1FBF0000)
#define		TC3162_UART_RBR     (0x00+TC3162_UART_BASE+0x03)
#define		TC3162_UART_THR     (0x00+TC3162_UART_BASE+0x03)
#define		TC3162_UART_IER     (0x04+TC3162_UART_BASE+0x03)
#define		TC3162_UART_IIR     (0x08+TC3162_UART_BASE+0x03)
#define		TC3162_UART_FCR     (0x08+TC3162_UART_BASE+0x03)
#define		TC3162_UART_LCR     (0x0c+TC3162_UART_BASE+0x03)
#define		TC3162_UART_MCR     (0x10+TC3162_UART_BASE+0x03)
#define		TC3162_UART_LSR     (0x14+TC3162_UART_BASE+0x03)
#define		TC3162_UART_MSR     (0x18+TC3162_UART_BASE+0x03)
#define		TC3162_UART_SCR     (0x1c+TC3162_UART_BASE+0x03)
#define		TC3162_UART_BRDL    (0x00+TC3162_UART_BASE+0x03)
#define		TC3162_UART_BRDH    (0x04+TC3162_UART_BASE+0x03)
#define		TC3162_UART_WORDA	(0x20+TC3162_UART_BASE+0x03)
#define		TC3162_UART_HWORDA	(0x28+TC3162_UART_BASE+0x03)
#define		TC3162_UART_MISCC	(0x24+TC3162_UART_BASE+0x03)
#define		TC3162_UART_XYD     (0x2c+TC3162_UART_BASE)

#define		UART_BRD_ACCESS		0x80
#define		UART_XYD_Y      	65000
#define		UART_UCLK_115200	0
#define		UART_UCLK_57600 	1
#define		UART_UCLK_38400		2
#define		UART_UCLK_28800		3
#define		UART_UCLK_19200		4
#define		UART_UCLK_14400		5
#define		UART_UCLK_9600 		6
#define		UART_UCLK_4800 		7
#define		UART_UCLK_2400 		8
#define		UART_UCLK_1200 		9
#define		UART_UCLK_600 		10
#define		UART_UCLK_300 		11
#define		UART_UCLK_110 		12
#define		UART_BRDL		0x03
#define		UART_BRDH		0x00
#define		UART_LCR		0x03
#define		UART_FCR		0x0f
#define		UART_WATERMARK		0x0<<6
#define		UART_MCR		0x0
#define		UART_MISCC		0x0
#define		UART_IER		0x05

#define		IER_RECEIVED_DATA_INTERRUPT_ENABLE	0x01
#define		IER_THRE_INTERRUPT_ENABLE		0x02
#define		IER_LINE_STATUS_INTERRUPT_ENABLE	0x04

#define		IIR_INDICATOR				*(volatile unsigned char*)(TC3162_UART_IIR)
#define		IIR_NO_INTERRUPT			0x01
#define		IIR_RECEIVED_LINE_STATUS		0x06
#define		IIR_RECEIVED_DATA_AVAILABLE		0x04
#define 	IIR_RECEIVER_IDLE_TRIGGER		0x0C
#define		IIR_TRANSMITTED_REGISTER_EMPTY		0x02
#define		IIR_MODEM_STATUS			0x00
#define		LSR_INDICATOR				*(volatile unsigned char*)(TC3162_UART_LSR)
#define		LSR_RECEIVED_DATA_READY			0x01
#define		LSR_OVERRUN				0x02
#define		LSR_PARITY_ERROR			0x04
#define		LSR_FRAME_ERROR				0x08
#define		LSR_BREAK				0x10
#define		LSR_THRE				0x20
#define		LSR_THE					0x40
#define		LSR_RFIFO_FLAG				0x80

#define 	uartTxIntOn()	*(volatile unsigned char*)(TC3162_UART_IER)|=IER_THRE_INTERRUPT_ENABLE
#define 	uartTxIntOff()	*(volatile unsigned char*)(TC3162_UART_IER)&=~IER_THRE_INTERRUPT_ENABLE
#define 	uartRxIntOn()	*(volatile unsigned char*)(TC3162_UART_IER)|= IER_RECEIVED_DATA_INTERRUPT_ENABLE
#define		uartRxIntOff()	*(volatile unsigned char*)(TC3162_UART_IER)&= ~IER_RECEIVED_DATA_INTERRUPT_ENABLE

#if 1
#define TC3160_UART_BASE	(KSEG1 + 0x12000200)
#define TC3160_UARTREG1_REG	(TC3160_UART_BASE + 0x0)
#define TC3160_UARTREG2_REG	(TC3160_UART_BASE + 0x4)
#define TC3160_INTENB_REG	(TC3160_UART_BASE + 0x8)
#define TC3160_BAUDREG_REG	(TC3160_UART_BASE + 0xc)
#define TC3160_TXREG_REG	(TC3160_UART_BASE + 0x10)
#define TC3160_RXREG_REG	(TC3160_UART_BASE + 0x14)
#define TC3160_TXBF_REG		(TC3160_UART_BASE + 0x18)
#define TC3160_RXBF_REG		(TC3160_UART_BASE + 0x1c)
#define TC3160_INTSTA_REG	(TC3160_UART_BASE + 0x20)
#define TC3160_MSREG_REG	(TC3160_UART_BASE + 0x24)
#define TC3160_MCREG_REG	(TC3160_UART_BASE + 0x28)

/*
 * UARTREG1
 */
#define SIO_UARTEN	0x80000000
#define SIO_TXEN	0x40000000
#define SIO_RXEN	0x20000000
#define SIO_UARTON	0x10000000
#define SIO_TXON	0x08000000
#define SIO_RXON	0x04000000
#define SIO_LOOKBK	0x02000000

/*
 * UARTREG2
 */
#define SIO_PTYEN	0x80000000 
#define SIO_ODDEVEN	0x40000000 /* 0/1: odd/even */
#define SIO_STOP	0x20000000 /* 0/1: 1/2 bit */
#define SIO_CHARLEN	0x10000000 /* 0/1: 8/7 bit */
#define SIO_TXPOL	0x08000000 /* 0/1: normal/invert */
#define SIO_RXPOL	0x04000000 /* 0/1: normal/invert */

/*
 * INTENB
 */
#define SIO_TXEMPTY	0x80000000
#define SIO_RXFULL	0x40000000
#define SIO_RXOVR	0x20000000
#define SIO_RXBRK	0x10000000
#define SIO_BRKEND	0x08000000
#define SIO_MSC		0x04000000

/*
 *
 */
#define SIO_ENB_TR	0xe0000000
#define SIO_ON_MASK	0x10000000
#define SIO_TXON_MASK	0x08000000
#define SIO_RXON_MASK	0x04000000
#define SIO_LB_TEST	0x02000000
#define SIO_ALL_ON	(SIO_ON_MASK | SIO_TXON_MASK | SIO_RXON_MASK)

#define SIO_MSC_RTSCNT	0x00000002
#define SIO_MSC_CTSCNT	0x00000001
#define SIO_HWFLOW	(SIO_MSC_RTSCNT | SIO_MSC_CTSCNT)

#define SIO_TR_RST	0x00008000

/*
 * Baudrate 
 */
#define SIO_BAUD_921600	0
#define SIO_BAUD_230400	2
#define SIO_BAUD_115200 (SIO_BAUD_230400 * 2)
#define SIO_BAUD_57600	(SIO_BAUD_115200 * 2)
#define SIO_BAUD_38400	(SIO_BAUD_115200 * 3)
#define SIO_BAUD_28800	(SIO_BAUD_57600 * 2)
#define SIO_BAUD_19200	(SIO_BAUD_38400 * 2)
#define SIO_BAUD_14400 	(SIO_BAUD_28800 * 2)
#define SIO_BAUD_9600	(SIO_BAUD_19200 * 2)
#define SIO_BAUD_7200	(SIO_BAUD_14400 * 2)
#define SIO_BAUD_4800	(SIO_BAUD_9600 * 2)
#define SIO_BAUD_3600	(SIO_BAUD_7200 * 2)
#define SIO_BAUD_2400	(SIO_BAUD_4800 * 2)

#ifndef __ASSEMBLY__

/*
 * SIO function prototipes
 */
int tc3160_sio_get_status (void);
int tc3160_sio_get_control (void);
int tc3160_sio_get_mode (void);
int tc3160_sio_get_baundrate (void);
int tc3160_sio_get_byte (void);

void tc3160_sio_set_control (int value);
void tc3160_sio_set_mode (int value);
void tc3160_sio_set_baundrate (int value);
void tc3160_sio_set_byte (int value);
#endif


#endif /* !__ASSEMBLY__ */
#endif /* __TC3162_SIO_H__  */

