/*
** $Id: domain.c 1.1 2004/08/12 12:58:36 cmchang Exp $
*/

/*************************************************************************
* Copyright (c) 2004 Inprocomm, Inc.
*
* All rights reserved. Copying, compilation, modification, distribution
* or any other use whatsoever of this material is strictly prohibited
* except in accordance with a Software License Agreement with
* Inprocomm, Inc.
**************************************************************************/
/*************************************************************************
GENERAL DESCRIPTION
********************
This file contains the regulatory domain related routines of Windows driver
for INPROCOMM 802.11 Wireless LAN Adapters.
*/

/************************************************************************
*                E X T E R N A L   R E F E R E N C E S
*************************************************************************
*/
#include "precomp.h"

/************************************************************************
*                          C O N S T A N T S
*************************************************************************
*/

/************************************************************************
*                         D A T A   T Y P E S
*************************************************************************
*/

/************************************************************************
*                        P U B L I C   D A T A
*************************************************************************
*/

/************************************************************************
*                       P R I V A T E   D A T A
*************************************************************************
*/

/* Supported regulatory domain information table */
static DOMAIN_INFO_ENTRY supportedDomainInfoTable[] = {
    { COUNTRY_CODE_AT,  MIB_REG_DOMAIN_ETSI,    { 1,  13, 20, 0 },  { { 36, 4, 15, MIB_U_NII_BAND_LOWER },    { 0,  0, 0,  0 },                       { 0,   0,  0,  0 },     { 0,   0, 0,  0 } } },
    { COUNTRY_CODE_AU,  MIB_REG_DOMAIN_OTHER,   { 1,  13, 20, 0 },  { { 36, 4, 16, MIB_U_NII_BAND_LOWER },    { 52, 4, 23, MIB_U_NII_BAND_MIDDLE },   { 0,   0,  0,  0 },     { 149, 4, 29, MIB_U_NII_BAND_UPPER } } },
    { COUNTRY_CODE_BE,  MIB_REG_DOMAIN_ETSI,    { 1,  13, 20, 0 },  { { 36, 4, 18, MIB_U_NII_BAND_LOWER },    { 52, 4, 18, MIB_U_NII_BAND_MIDDLE },   { 0,   0,  0,  0 },     { 0,   0, 0,  0 } } },
    { COUNTRY_CODE_CA,  MIB_REG_DOMAIN_IC,      { 1,  11, 30, 0 },  { { 36, 4, 16, MIB_U_NII_BAND_LOWER },    { 52, 4, 23, MIB_U_NII_BAND_MIDDLE },   { 0,   0,  0,  0 },     { 149, 4, 29, MIB_U_NII_BAND_UPPER } } },
    { COUNTRY_CODE_CH,  MIB_REG_DOMAIN_ETSI,    { 1,  13, 20, 0 },  { { 36, 4, 18, MIB_U_NII_BAND_LOWER },    { 52, 4, 18, MIB_U_NII_BAND_MIDDLE },   { 0,   0,  0,  0 },     { 0,   0, 0,  0 } } },
    { COUNTRY_CODE_DE,  MIB_REG_DOMAIN_ETSI,    { 1,  13, 20, 0 },  { { 36, 4, 15, MIB_U_NII_BAND_LOWER },    { 0,  0, 0,  0 },                       { 0,   0,  0,  0 },     { 0,   0, 0,  0 } } },
    { COUNTRY_CODE_DK,  MIB_REG_DOMAIN_ETSI,    { 1,  13, 20, 0 },  { { 36, 4, 17, MIB_U_NII_BAND_LOWER },    { 0,  0, 0,  0 },                       { 0,   0,  0,  0 },     { 0,   0, 0,  0 } } },
    { COUNTRY_CODE_ES,  MIB_REG_DOMAIN_SPAIN,   { 10, 2,  20, 0 },  { { 0,  0, 0,  0 },                       { 0,  0, 0,  0 },                       { 0,   0,  0,  0 },     { 0,   0, 0,  0 } } },
    { COUNTRY_CODE_EU,  MIB_REG_DOMAIN_ETSI,    { 1,  13, 20, 0 },  { { 0,  0, 0,  0 },                       { 0,  0, 0,  0 },                       { 0,   0,  0,  0 },     { 0,   0, 0,  0 } } },
    { COUNTRY_CODE_FI,  MIB_REG_DOMAIN_ETSI,    { 1,  13, 20, 0 },  { { 36, 4, 23, MIB_U_NII_BAND_LOWER },    { 0,  0, 0,  0 },                       { 0,   0,  0,  0 },     { 0,   0, 0,  0 } } },
    { COUNTRY_CODE_FR,  MIB_REG_DOMAIN_FRANCE,  { 10, 4,  20, 0 },  { { 36, 4, 23, MIB_U_NII_BAND_LOWER },    { 52, 4, 23, MIB_U_NII_BAND_MIDDLE },   { 0,   0,  0,  0 },     { 0,   0, 0,  0 } } },
    { COUNTRY_CODE_GB,  MIB_REG_DOMAIN_ETSI,    { 1,  13, 20, 0 },  { { 36, 4, 18, MIB_U_NII_BAND_LOWER },    { 52, 4, 18, MIB_U_NII_BAND_MIDDLE },   { 0,   0,  0,  0 },     { 0,   0, 0,  0 } } },
    { COUNTRY_CODE_GR,  MIB_REG_DOMAIN_ETSI,    { 1,  13, 20, 0 },  { { 0,  0, 0,  0 },                       { 0,  0, 0,  0 },                       { 0,   0,  0,  0 },     { 0,   0, 0,  0 } } },
    { COUNTRY_CODE_IE,  MIB_REG_DOMAIN_ETSI,    { 1,  13, 20, 0 },  { { 36, 4, 18, MIB_U_NII_BAND_LOWER },    { 52, 4, 18, MIB_U_NII_BAND_MIDDLE },   { 0,   0,  0,  0 },     { 0,   0, 0,  0 } } },
    { COUNTRY_CODE_IL,  MIB_REG_DOMAIN_OTHER,   { 5,  4,  20, 0 },  { { 0,  0, 0,  0 },                       { 0,  0, 0,  0 },                       { 0,   0,  0,  0 },     { 0,   0, 0,  0 } } },
    { COUNTRY_CODE_IS,  MIB_REG_DOMAIN_ETSI,    { 1,  13, 20, 0 },  { { 36, 4, 23, MIB_U_NII_BAND_LOWER },    { 52, 4, 23, MIB_U_NII_BAND_MIDDLE },   { 100, 11, 30, 0 },     { 0,   0, 0,  0 } } },
    { COUNTRY_CODE_IT,  MIB_REG_DOMAIN_ETSI,    { 1,  13, 20, 0 },  { { 36, 4, 23, MIB_U_NII_BAND_LOWER },    { 52, 4, 23, MIB_U_NII_BAND_MIDDLE },   { 100, 11, 30, 0 },     { 0,   0, 0,  0 } } },
#if SUPPORT_802_11G
    { COUNTRY_CODE_JP,  MIB_REG_DOMAIN_JAPAN,   { 1,  13, 23, 0 },  { { 34, 4, 23, MIB_U_NII_BAND_LOWER },    { 0,  0, 0,  0 },                       { 0,   0,  0,  0 },     { 0,   0, 0,  0 } } },
#else
    { COUNTRY_CODE_JP,  MIB_REG_DOMAIN_JAPAN,   { 1,  13, 23, 0 },  { { 0,  0, 0,  0 },                       { 0,  0, 0,  0 },                       { 0,   0,  0,  0 },     { 0,   0, 0,  0 } } },
#endif
    { COUNTRY_CODE_LI,  MIB_REG_DOMAIN_ETSI,    { 1,  13, 20, 0 },  { { 36, 4, 18, MIB_U_NII_BAND_LOWER },    { 52, 4, 18, MIB_U_NII_BAND_MIDDLE },   { 0,   0,  0 , 0 },     { 0,   0, 0,  0 } } },
    { COUNTRY_CODE_LU,  MIB_REG_DOMAIN_ETSI,    { 1,  13, 20, 0 },  { { 36, 4, 18, MIB_U_NII_BAND_LOWER },    { 52, 4, 15, MIB_U_NII_BAND_MIDDLE },   { 0,   0,  0 , 0 },     { 0,   0, 0,  0 } } },
    { COUNTRY_CODE_NL,  MIB_REG_DOMAIN_ETSI,    { 1,  13, 20, 0 },  { { 36, 4, 23, MIB_U_NII_BAND_LOWER },    { 52, 4, 23, MIB_U_NII_BAND_MIDDLE },   { 100, 11, 30, 0 },     { 0,   0, 0,  0 } } },
    { COUNTRY_CODE_NO,  MIB_REG_DOMAIN_ETSI,    { 1,  13, 20, 0 },  { { 36, 4, 23, MIB_U_NII_BAND_LOWER },    { 52, 4, 23, MIB_U_NII_BAND_MIDDLE },   { 100, 11, 30, 0 },     { 0,   0, 0,  0 } } },
    { COUNTRY_CODE_PT,  MIB_REG_DOMAIN_ETSI,    { 1,  13, 20, 0 },  { { 36, 4, 23, MIB_U_NII_BAND_LOWER },    { 52, 4, 23, MIB_U_NII_BAND_MIDDLE },   { 100, 11, 30, 0 },     { 0,   0, 0,  0 } } },
    { COUNTRY_CODE_SE,  MIB_REG_DOMAIN_ETSI,    { 1,  13, 20, 0 },  { { 36, 4, 23, MIB_U_NII_BAND_LOWER },    { 0,  0, 0,  0 },                       { 0,   0,  0 , 0 },     { 0,   0, 0,  0 } } },
    { COUNTRY_CODE_TW,  MIB_REG_DOMAIN_OTHER,   { 1,  13, 30, 0 },  { { 0,  0, 0,  0 },                       { 52, 4, 23, MIB_U_NII_BAND_MIDDLE },   { 0,   0,  0 , 0 },     { 149, 4, 29, MIB_U_NII_BAND_UPPER } } },
    { COUNTRY_CODE_US,  MIB_REG_DOMAIN_FCC,     { 1,  11, 30, 0 },  { { 36, 4, 16, MIB_U_NII_BAND_LOWER },    { 52, 4, 23, MIB_U_NII_BAND_MIDDLE },   { 0,   0,  0 , 0 },     { 149, 4, 29, MIB_U_NII_BAND_UPPER } } }
};

/* Default regulatory domain information */
static DOMAIN_INFO_ENTRY defaultDomainInfo = {
    0, MIB_REG_DOMAIN_OTHER,
    { 1, 14, 20, 0 },
    {
#if SUPPORT_802_11A
        { 36, 8, 23, MIB_U_NII_BAND_LOWER | MIB_U_NII_BAND_MIDDLE },
        { 100, 11, 30, 0 },
        { 149, 4, 29, MIB_U_NII_BAND_UPPER },
        { 34, 4, 23, MIB_U_NII_BAND_LOWER }
#else
        { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }
#endif
    }
};

/************************************************************************
*                             M A C R O S
*************************************************************************
*/

/************************************************************************
*              F U N C T I O N   D E C L A R A T I O N S
*************************************************************************
*/

/************************************************************************
*                          F U N C T I O N S
*************************************************************************
*/

/*_________________________________________________________________________
**  domainCheckForSupportedDomain
**
**  descriptions: This routine is called to search the supported domain
**                information table to check if the given domain is
**                specified in the table.
**  parameters:
**      countryCode - The country code of the given domain to be checked
**  result:
**      domainInfo_p - Pointer to the information entry of the given domain
**  return:
**      TRUE - The given domain is supported.
**      FALSE - The given domain is not supported.
**  note:
**__________________________________________________________________________*/
BOOLEAN
domainCheckForSupportedDomain(IN UINT_16 countryCode,
                              OUT PDOMAIN_INFO_ENTRY *domainInfo_p)
{
    INT i, j, k;

    i = 0;
    j = sizeof(supportedDomainInfoTable) / sizeof(DOMAIN_INFO_ENTRY) - 1;

    while (i <= j) {
        k = (i + j) / 2;

        if (countryCode == supportedDomainInfoTable[k].countryCode) {
            *domainInfo_p = &supportedDomainInfoTable[k];
            return TRUE;
        }

        if (countryCode < supportedDomainInfoTable[k].countryCode) {
            j = k - 1;
        } else {
            i = k + 1;
        }
    }

    *domainInfo_p = NULL;

    return FALSE;
}   /* domainCheckForSupportedDomain */

/*_________________________________________________________________________
**  domainSetupChannelInfo
**
**  descriptions: This routine is called to set up the channel information
**                according to the given regulatory domain information.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**      domainInfo_p - Pointer to the information entry of the given
**                     domain. If the value of NULL is given, the default
**                     channel information is used.
**  return:         (none)
**  note:
**__________________________________________________________________________*/
VOID
domainSetupChannelInfo(IN PMP_ADAPTER adapter_p,
                       IN PDOMAIN_INFO_ENTRY domainInfo_p)
{
    UINT i;
    UINT_8 firstChannelNum;
    UINT numChannels;
#if SUPPORT_802_11A
    UINT_8 channelNum;
    PVOID entry_p;
    UINT j;
#endif

    DEBUGFUNC("domainSetupChannelInfo");

    /* If there is not any regulatory domain information provided, we use the
       default regulatory information instead. */
    if (domainInfo_p == NULL) {
        domainInfo_p = &defaultDomainInfo;
    }
    adapter_p->domainInfo_p = domainInfo_p; /* CMC add 2004/7/30 */

    /* Clear the operating channel list for the 2.4 GHz band. */
    adapter_p->numOperatingChannels24 = 0;

    for (i = 0; i < MAX_NUM_2G_RF_CHANNELS; i++) {
        adapter_p->operatingChannels24[i] = 0;
    }

    /* Set up the operating channel list for the 2.4 GHz band. */
    firstChannelNum = domainInfo_p->band24.firstChannelNum;
    numChannels = domainInfo_p->band24.numChannels;

    ASSERT(numChannels <= MAX_NUM_2G_RF_CHANNELS);

    adapter_p->numOperatingChannels24 = numChannels;

    for (i = 0; i < numChannels; i++) {
        adapter_p->operatingChannels24[i] = firstChannelNum + (UINT_8) i;
    }

    INITLOG(("Operating channels 2.4 GHz (%d): %d %d %d %d %d %d %d %d %d %d %d %d %d %d\n",
        adapter_p->numOperatingChannels24, adapter_p->operatingChannels24[0],
        adapter_p->operatingChannels24[1], adapter_p->operatingChannels24[2],
        adapter_p->operatingChannels24[3], adapter_p->operatingChannels24[4],
        adapter_p->operatingChannels24[5], adapter_p->operatingChannels24[6],
        adapter_p->operatingChannels24[7], adapter_p->operatingChannels24[8],
        adapter_p->operatingChannels24[9], adapter_p->operatingChannels24[10],
        adapter_p->operatingChannels24[11], adapter_p->operatingChannels24[12],
        adapter_p->operatingChannels24[13]));

#if SUPPORT_802_11A
    /* Clear the operating channel list for the 5 GHz band. */
    adapter_p->numOperatingChannels5 = 0;

    for (i = 0; i < MAX_NUM_5G_RF_CHANNELS; i++) {
        adapter_p->operatingChannels5[i] = 0;
    }

    /* Clear the MIB attribute dot11FrequencyBandsSupported. */
    adapter_p->mib.dot11FrequencyBandsSupported = 0;

    /* Set up the operating channel list for the 5 GHz band. */
    ASSERT(domainInfo_p->band5[0].numChannels +
        domainInfo_p->band5[1].numChannels +
        domainInfo_p->band5[2].numChannels +
        domainInfo_p->band5[3].numChannels <= MAX_NUM_5G_RF_CHANNELS);

    for (i = 0; i < 4; i++) {
        firstChannelNum = domainInfo_p->band5[i].firstChannelNum;
        numChannels = domainInfo_p->band5[i].numChannels;

        if (firstChannelNum == 0) {
            continue;
        }

        for (j = 0; j < numChannels; j++) {
            channelNum = firstChannelNum + (UINT_8) j * 4;

            /* Add this channel to the 5-GHz operating channel list if it is
               valid on the NIC. */
            if (phyRFCheckForValidChannel(adapter_p, channelNum,
                CHANNEL_BAND_5, &entry_p)) {
                adapter_p->operatingChannels5[adapter_p->numOperatingChannels5++] =
                    channelNum;
            }
        }

        /* Set the MIB attribute dot11FrequencyBandsSupported. */
        adapter_p->mib.dot11FrequencyBandsSupported |=
            domainInfo_p->band5[i].mibBandsSupportedValue;
    }

    INITLOG(("Operating channels 5 GHz (%2d): %d %d %d %d %d %d %d %d\n",
        adapter_p->numOperatingChannels5, adapter_p->operatingChannels5[0],
        adapter_p->operatingChannels5[1], adapter_p->operatingChannels5[2],
        adapter_p->operatingChannels5[3], adapter_p->operatingChannels5[4],
        adapter_p->operatingChannels5[5], adapter_p->operatingChannels5[6],
        adapter_p->operatingChannels5[7]));
    INITLOG(("                               %d %d %d %d %d %d %d %d\n",
        adapter_p->operatingChannels5[8], adapter_p->operatingChannels5[9],
        adapter_p->operatingChannels5[10], adapter_p->operatingChannels5[11],
        adapter_p->operatingChannels5[12], adapter_p->operatingChannels5[13],
        adapter_p->operatingChannels5[14], adapter_p->operatingChannels5[15]));
    INITLOG(("                               %d %d %d %d %d %d %d %d\n",
        adapter_p->operatingChannels5[16], adapter_p->operatingChannels5[17],
        adapter_p->operatingChannels5[18], adapter_p->operatingChannels5[19],
        adapter_p->operatingChannels5[20], adapter_p->operatingChannels5[21],
        adapter_p->operatingChannels5[22], adapter_p->operatingChannels5[23]));
#endif
}   /* domainSetupChannelInfo */

/*_________________________________________________________________________
**  domainInitCurrentRegDomain
**
**  descriptions: This routine is called to initialize the current
**                regulatory domain.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**      eepromDomainSetting - The regulatory domain setting read from the
**                            EEPROM
**  return:         (none)
**  note:
**__________________________________________________________________________*/
VOID
domainInitCurrentRegDomain(IN PMP_ADAPTER adapter_p,
                           IN UINT_16 eepromDomainSetting)
{
    PDOMAIN_INFO_ENTRY domainInfo_p;

    DEBUGFUNC("domainInitCurrentRegDomain");

    /* If the current country parameter exists in registry, we use it as the
       current regulatory domain. Otherwise, we use the regulatory domain
       settting in the EEPROM. */
    if (adapter_p->regCountryString[0] && adapter_p->regCountryString[1]) {
        INITLOG(("Use the current country in registry\n"));

        adapter_p->nicConfiguredCountry =
            ((UINT_16) adapter_p->regCountryString[0] << 8) |
            (UINT_16) adapter_p->regCountryString[1];
    } else {
        INITLOG(("Use the current country in EEPROM\n"));

        /* The following lines except the default case are added for the old
           format of the regulatory domain field in the EEPROM. */
        switch (eepromDomainSetting) {
            case MIB_REG_DOMAIN_FCC:
                 adapter_p->nicConfiguredCountry = COUNTRY_CODE_US;
                 break;

            case MIB_REG_DOMAIN_IC:
                 adapter_p->nicConfiguredCountry = COUNTRY_CODE_CA;
                 break;

            case MIB_REG_DOMAIN_ETSI:
                 adapter_p->nicConfiguredCountry = COUNTRY_CODE_EU;
                 break;

            case MIB_REG_DOMAIN_SPAIN:
                 adapter_p->nicConfiguredCountry = COUNTRY_CODE_ES;
                 break;

            case MIB_REG_DOMAIN_FRANCE:
                 adapter_p->nicConfiguredCountry = COUNTRY_CODE_FR;
                 break;

            case MIB_REG_DOMAIN_JAPAN:
                 adapter_p->nicConfiguredCountry = COUNTRY_CODE_JP;
                 break;

            default:
                 adapter_p->nicConfiguredCountry = eepromDomainSetting;
        }
    }

#if DBG
    if ((adapter_p->nicConfiguredCountry & 0xFF00) &&
        (adapter_p->nicConfiguredCountry & 0x00FF)) {
        INITLOG(("NIC configured country: 0x%04x (%c%c)\n",
            adapter_p->nicConfiguredCountry,
            (UINT_8) (adapter_p->nicConfiguredCountry >> 8),
            (UINT_8) (adapter_p->nicConfiguredCountry & 0x00FF)));
    } else {
        INITLOG(("NIC configured country: 0x%04x\n",
            adapter_p->nicConfiguredCountry));
    }
#endif

#if SUPPORT_802_11D && 0 /* CMC 2004/7/30 */
    if (adapter_p->mib.dot11MultiDomainCapabilityEnabled) {
        adapter_p->mib.dot11CurrentRegDomain = MIB_REG_DOMAIN_OTHER;
        adapter_p->currentCountry = 0;

        domainInfo_p = NULL;

        INITLOG(("Multi-domain operation is enabled\n"));
    } else
#endif
    {
        if (domainCheckForSupportedDomain(adapter_p->nicConfiguredCountry,
            &domainInfo_p)) {
            adapter_p->mib.dot11CurrentRegDomain =
                domainInfo_p->mibRegDomainValue;
            adapter_p->currentCountry = adapter_p->nicConfiguredCountry;
        } else {
            adapter_p->mib.dot11CurrentRegDomain = MIB_REG_DOMAIN_OTHER;
            adapter_p->currentCountry = 0;
        }

        INITLOG(("Use NIC configured country as current reg. domain: 0x%04x\n",
            adapter_p->currentCountry));
    }

    /* Set up the channel information according to the current regulatory
       domain. */
    domainSetupChannelInfo(adapter_p, domainInfo_p);
}   /* domainInitCurrentRegDomain */

/*_________________________________________________________________________
**  domainCheckForValidChannel
**
**  descriptions: This routine is called to check if the given channel
**                number is valid for the current regulatory domain.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**      channelNum - The channel number to be checked
**      channelBand - The channel band to which the given channel number
**                    belongs
**  return:
**      TRUE - All the operating channels in a band are supported.
**      FALSE - The given band is not supported.
**  note:
**__________________________________________________________________________*/
BOOLEAN
domainCheckForValidChannel(IN PMP_ADAPTER adapter_p,
#if SUPPORT_802_11A
                           IN UINT_8 channelNum,
                           IN UINT_8 channelBand)
#else
                           IN UINT_8 channelNum)
#endif
{
    BOOLEAN status;

#if SUPPORT_802_11A
    if (channelBand == CHANNEL_BAND_5) {
        UINT i;

        status = FALSE;

        for (i = 0; i < adapter_p->numOperatingChannels5; i++) {
            if (adapter_p->operatingChannels5[i] == channelNum) {
                status = TRUE;
                break;
            }
        }
    } else
#endif
    {
        if (channelNum >
            adapter_p->operatingChannels24[adapter_p->numOperatingChannels24 - 1] ||
            channelNum < adapter_p->operatingChannels24[0]) {
            status = FALSE;
        } else {
            status = TRUE;
        }
    }

    return status;
}   /* domainCheckForValidChannel */


