/*
** $Id: proc.c 1.52 2004/09/23 12:01:21 chyeh Exp $
*/

/*! \file proc.c
    \brief This file contains the Proc file system routines.

    We will create a AP2220 fs in proc/net directory
*/

/*************************************************************************
* Copyright (c) 2002-2003 Integrated Programmable Communications, Inc.
*
* All rights reserved. Copying, compilation, modification, distribution
* or any other use whatsoever of this material is strictly prohibited
* except in accordance with a Software License Agreement with
* Integrated Programmable Communications, Inc.
**************************************************************************/

/************************************************************************
*                     C O M P I L E R   F L A G S
*************************************************************************
*/

/************************************************************************
*                E X T E R N A L   R E F E R E N C E S
*************************************************************************
*/

#include "wlan_os.h"
#include "precomp.h"

/************************************************************************
*                          C O N S T A N T S
*************************************************************************
*/

/************************************************************************
*                         D A T A   T Y P E S
*************************************************************************
*/
typedef struct proc_dir_entry proc_dir_entry_t;

/************************************************************************
*                        P U B L I C   D A T A
*************************************************************************
*/

/************************************************************************
*                       P R I V A T E   D A T A
*************************************************************************
*/

/************************************************************************
*                             M A C R O S
*************************************************************************
*/

#define create_proc_d(fn, level, type) ({\
type *____ret; \
____ret = proc_mkdir(fn, level);\
if (____ret==NULL)\
        KDBG_LOG_MLME(KDBG_INFO,("creat proc hostap fail"));\
____ret;\
})

/************************************************************************
*              F U N C T I O N   D E C L A R A T I O N S
*************************************************************************
*/

/************************************************************************
*                          F U N C T I O N S
*************************************************************************
*/
static int skip_atoi(const char **s)
{
        int i=0;

        while (isdigit(**s))
                i = i*10 + *((*s)++) - '0';
        return i;
}

/**
 * vsscanf - Unformat a buffer into a list of arguments
 * @buf:        input buffer
 * @fmt:        format of buffer
 * @args:       arguments
 */
int vsscanf_(const char * buf, const char * fmt, va_list args)
{
        const char *str = buf;
        char *next;
        int num = 0;
        int qualifier;
        int base;
        int field_width = -1;
        int is_sign = 0;

        while(*fmt && *str) {
                /* skip any white space in format */
                /* white space in format matchs any amount of
                 * white space, including none, in the input.
                 */
                if (isspace(*fmt)) {
                        while (isspace(*fmt))
                                ++fmt;
                        while (isspace(*str))
                                ++str;
                }

                /* anything that is not a conversion must match exactly */
                if (*fmt != '%' && *fmt) {
                        if (*fmt++ != *str++)
                                break;
                        continue;
                }

                if (!*fmt)
                        break;
                ++fmt;

                /* skip this conversion.
                 * advance both strings to next white space
                 */
                if (*fmt == '*') {
                        while (!isspace(*fmt) && *fmt)
                                fmt++;
                        while (!isspace(*str) && *str)
                                str++;
                        continue;
                }

                /* get field width */
                if (isdigit(*fmt))
                        field_width = skip_atoi(&fmt);

                /* get conversion qualifier */
                qualifier = -1;
                if (*fmt == 'h' || *fmt == 'l' || *fmt == 'L' || *fmt == 'Z') {
                        qualifier = *fmt;
                        fmt++;
                }
                base = 10;
                is_sign = 0;

                if (!*fmt || !*str)
                        break;

                switch(*fmt++) {
                case 's':
                {
                        char *s = (char *) va_arg(args, char *);
                        if(field_width == -1)
                                field_width = INT_MAX;
                        /* first, skip leading white space in buffer */
                        while (isspace(*str))
                                str++;

                        /* now copy until next white space */
                        while (*str && !isspace(*str) && field_width--) {
                                *s++ = *str++;
                        }
                        *s = '\0';
                        num++;
                }
                continue;
                case 'x':
                case 'X':
                        base = 16;
                        break;
                case 'd':
                case 'i':
                        is_sign = 1;
                case 'u':
                        break;
                case '%':
                        /* looking for '%' in str */
                        if (*str++ != '%')
                                return num;
                        continue;
                default:
                        /* invalid format; stop here */
                        return num;
                }

                /* have some sort of integer conversion.
                 * first, skip white space in buffer.
                 */
                while (isspace(*str))
                        str++;
#if 0
                if (!*str || !isdigit(*str)){
                        printk("break isdigit\n");
                        break;
                        }
#endif
                if (!*str)
                        break;

                switch(qualifier) {
                case 'l':
                        if (is_sign) {
                                long *l = (long *) va_arg(args,long *);
                                *l = simple_strtol(str,&next,base);
                        } else {
                                unsigned long *l = (unsigned long*) va_arg(args,unsigned long*);
                                *l = simple_strtoul(str,&next,base);
                        }
                        break;
                default:
                        if (is_sign) {
                                int *i = (int *) va_arg(args, int*);
                                *i = (int) simple_strtol(str,&next,base);
                        } else {
                                unsigned int *i = (unsigned int*) va_arg(args, unsigned int*);
                                *i = (unsigned int) simple_strtoul(str,&next,base);
                        }
                        break;
                }
                num++;

                if (!next)
                        break;
                str = next;
        }
        return num;
}

/**
 * sscanf - Unformat a buffer into a list of arguments
 * @buf:        input buffer
 * @fmt:        formatting of buffer
 * @...:        resulting arguments
 */
int mysscanf(const char * buf, const char * fmt, ...)
{
        va_list args;
        int i;

        va_start(args,fmt);
        i = vsscanf_(buf,fmt,args);
        va_end(args);
        return i;
}

//#if WLAN_INCLUDE_DEBUG /* for hw sub-directory */
/*----------------------------------------------------------------*/
/*!
* \brief The PROC for read Access Point PCI registers.
*
* This service handles user read the Accees Point PCI registers.
* Not support right now.
* This procedure only present when WLAN_INCLUDE_PROC=1
*
* \param   the PROC CONST defined parameters
* \return  no use..
*/
/*----------------------------------------------------------------*/
static int proc_pci_read(char *page, char **start, off_t off,
                        int count, int *eof, void *data)
{
        char            *p = page;
        i2220mgmt_t     *priv_p = (i2220mgmt_t *)data;
        kal_pci_dev_t   *pdev = priv_p->pcidev;
        UINT32          offset, tmpUlong;

        for (offset = 0; offset < 0x40; offset += 4) {
                if ((offset % 16) == 0) {
                        p += sprintf(p, "\n0x%02lx:", offset);
                }
                pci_read_config_dword(pdev, (int)offset, (UINT*)&tmpUlong);
                p += sprintf(p, " %08lx", tmpUlong);
        }
        p += sprintf(p, "\n");

        return (p - page);
}

/*----------------------------------------------------------------*/
/*!
* \brief The PROC function for write PCI config registers
*
* This service handles user write PCI config registers.
*
* This procedure only present when WLAN_INCLUDE_PROC=1
*
* \param   the PROC CONST defined parameters
* \return  not use.
*/
/*----------------------------------------------------------------*/
static int proc_pci_write(struct file *file, const char *buffer,
                        unsigned long count, void *data)
{
        i2220mgmt_t     *priv_p = (i2220mgmt_t *)data;
        kal_pci_dev_t   *pdev = priv_p->pcidev;
        char            kbuf[81];
        int             copy_size;
        UINT32          offset, value;

        copy_size = (count < 80) ? count : 80;
        copy_from_user(kbuf, buffer, copy_size);
        kbuf[copy_size] = '\0';
        if ( mysscanf(kbuf, "0x%lx 0x%lx", &offset, &value) == 2
            && (offset % 4) == 0 ){
                pci_write_config_dword(pdev, (int)offset, value);
                KDBG_LOG_TEMP(KDBG_INFO,"Write 0x%08lx to pci offset=0x%08lx\n",
                        value, offset);
        }
        else if ( kbuf[0] != '\n' ){
                KDBG_LOG_TEMP(KDBG_INFO,"Usage:   echo \"offset(4) value\" > pci\n");
                KDBG_LOG_TEMP(KDBG_INFO,"Example: echo \"0x0004 0xffffffff\" > pci\n");
        }

        return count;
}

/*----------------------------------------------------------------*/
/*!
* \brief The PROC for read Access Point mmi parameters.
*
* This service handles user read the Accees Point configuration.
* Not support right now.
* This procedure only present when WLAN_INCLUDE_PROC=1
*
* \param   the PROC CONST defined parameters
* \return  no use.
*/
/*----------------------------------------------------------------*/
static int proc_bcr_read(char *page, char **start, off_t off,
                        int count, int *eof, void *data)
{
        i2220mgmt_t     *priv_p = (i2220mgmt_t *) data;

        return wlan_read_bcr(priv_p->netdev, page);
}

/*----------------------------------------------------------------*/
/*!
* \brief The PROC function for write MMI registers
*
* This service handles user write MMI registers.
*
* This procedure only present when WLAN_INCLUDE_PROC=1
*
* \param   the PROC CONST defined parameters
* \return  not use.
*/
/*----------------------------------------------------------------*/
static int proc_bcr_write(struct file *file, const char *buffer,
                        unsigned long count, void *data)
{
        i2220mgmt_t     *priv_p = (i2220mgmt_t *)data;
        char            kbuf[81];
        int             copy_size;
        UINT32          index, value;

        copy_size = (count < 80) ? count : 80;
        copy_from_user(kbuf, buffer, copy_size);
        kbuf[copy_size] = '\0';
        if ( mysscanf(kbuf, "0x%lx 0x%lx", &index, &value) == 2){
                BCIWrite8(priv_p, index, value, TRUE, 0xff);
                KDBG_LOG_TEMP(KDBG_INFO,"Wirte 0x%02x to CR 0x%02x\n",
                        (UINT8)value, (UINT8)index);
        }
        else if ( kbuf[0] != '\n' ){
                KDBG_LOG_TEMP(KDBG_INFO,"Usage:   echo \"index value\" > bcr\n");
                KDBG_LOG_TEMP(KDBG_INFO,"Example: echo \"0x0c 0xff\" > bcr\n");
        }

        return count;
}

/*----------------------------------------------------------------*/
/*!
* \brief The PROC for read Access Point MCR registers.
*
* This service handles user read the Accees Point MCR registers.
* Not support right now.
* This procedure only present when WLAN_INCLUDE_PROC=1
*
* \param   the PROC CONST defined parameters
* \return  no use.
*/
/*----------------------------------------------------------------*/
static int proc_mcr_read(char *page, char **start, off_t off,
                        int count, int *eof, void *data)
{
        i2220mgmt_t     *priv_p = (i2220mgmt_t *) data;

        return wlan_read_mcr(priv_p->netdev, page);
}

/*----------------------------------------------------------------*/
/*!
* \brief The PROC function for write MCR registers
*
* This service handles user write MCR registers.
*
* This procedure only present when WLAN_INCLUDE_PROC=1
*
* \param   the PROC CONST defined parameters
* \return  not use.
*/
/*----------------------------------------------------------------*/
static int proc_mcr_write(struct file *file, const char *buffer,
                        unsigned long count, void *data)
{
        i2220mgmt_t     *priv_p = (i2220mgmt_t *)data;
        char            kbuf[81];
        int             copy_size;
        UINT32          offset, value;

        copy_size = (count < 80) ? count : 80;
        copy_from_user(kbuf, buffer, copy_size);
        kbuf[copy_size] = '\0';
        if ( mysscanf(kbuf, "0x%lx 0x%lx", &offset, &value) == 2
            && (offset % 4) == 0 ){
                NIC_MCR_WRITE(priv_p, offset, value);
                KDBG_LOG_TEMP(KDBG_INFO,"Write 0x%08lx to MCR 0x%04lx",
                        value, offset);
        }
        else if ( kbuf[0] != '\n' ){
                KDBG_LOG_TEMP(KDBG_INFO,"Usage:   echo \"offset(4) value\" > mcr\n");
                KDBG_LOG_TEMP(KDBG_INFO,"Example: echo \"0x000c 0xffffffff\" > mcr\n");
        }

        return count;
}
//#endif /* end of WLAN_INCLUDE_DEBUG */

/*----------------------------------------------------------------*/
/*!
* \brief The PROC driver version read function .
*
* This service provides AP driver's version information.
* This procedure only present when WLAN_INCLUDE_PROC=1
*
* \param   the PROC CONST defined parameters
* \return  no use.
*/
/*----------------------------------------------------------------*/
static int proc_version_read(char *page, char **start,
                        off_t off, int count, int *eof, void *data)
{
        char                  *p = page;
        i2220mgmt_t     *priv_p = (i2220mgmt_t *) data;
        char                 *version = param_get_version(priv_p->netdev);

        p+=sprintf(p, "version = %s\n\n", version);

       return (p - page);
}

/*----------------------------------------------------------------*/
/*!
* \brief The PROC read function for Access Point mib parameters.
*
* This service handles user read the Accees Point MIB Setting.
* including SSID, baacon period, DTIM period,... etc.
* This procedure only present when WLAN_INCLUDE_PROC=1
*
* \param   the PROC CONST defined parameters
* \return  no use.
*/
/*----------------------------------------------------------------*/
static int proc_config_read(char *page,char **start,
                        off_t off, int count, int *eof, void *data)
{
        i2220mgmt_t     *priv_p = (i2220mgmt_t *) data;

        return wlan_read_config(priv_p->netdev, page);
}

/*----------------------------------------------------------------*/
/*!
* \brief The PROC AP infomation read function .
*
* This service handles user read the Accees Point mib information.
* including Mac Address, RTSThreadhold, ShortRetryLimit, LongRetryLimit
* MaxTransmitMSDULifetime, MaxReceiveLifetime, ManufacturerID,
* and ProductID
* This procedure only present when WLAN_INCLUDE_PROC=1
*
* \param   the PROC CONST defined parameters
* \return  no use.
*/
/*----------------------------------------------------------------*/
static int proc_info_read(char *page,char **start,
                        off_t off,int count, int *eof,void *data)
{
        i2220mgmt_t     *priv_p = (i2220mgmt_t *) data;

        return wlan_read_info(priv_p->netdev, page);
}

/*----------------------------------------------------------------*/
/*!
* \brief The Access Point PROC for general Statistic read function .
*
* This service handles user read the Accees Point general information.
* including RfbRxOkCounter, RFCrcErr, TransmittedFragmentCount
* MulticastTransmittedFrameCount, FailedCount, RetryCount, MultipleRetryCount
* FrameDuplicateCount ... etc.
* This procedure only present when WLAN_INCLUDE_PROC=1
*
* \param   the PROC CONST defined parameters
* \return  no use.
*/
/*----------------------------------------------------------------*/
static int proc_stat_read(char *page,char **start,
                        off_t off,int count, int *eof,void *data)
{
        i2220mgmt_t     *priv_p = (i2220mgmt_t *) data;

        return wlan_read_stat(priv_p->netdev, page);
}

/*----------------------------------------------------------------*/
/*!
* \brief The PROC for read Access Point Physical registers.
*
* This service handles user read the Accees Point Physical registers.
* Not support right now.
* This procedure only present when WLAN_INCLUDE_PROC=1
*
* \param   the PROC CONST defined parameters
* \return  no use.
*/
/*----------------------------------------------------------------*/
static int proc_eeprom_read(char *page, char **start, off_t off,
                        int count, int *eof, void *data)
{
        i2220mgmt_t     *priv_p = (i2220mgmt_t *) data;

        return wlan_read_eeprom(priv_p->netdev, page);
}

/*----------------------------------------------------------------*/
/*!
* \brief The PROC function for write PHY registers
*
* This service handles user write PHY registers.
*
* This procedure only present when WLAN_INCLUDE_PROC=1
*
* \param   the PROC CONST defined parameters
* \return  not use.
*/
/*----------------------------------------------------------------*/
static int proc_eeprom_write(struct file *file, const char *buffer,
                        unsigned long count, void *data)
{
        i2220mgmt_t     *priv_p = (i2220mgmt_t *)data;
        char            kbuf[81];
        int             copy_size;
        UINT32          offset, value;

        copy_size = (count < 80) ? count : 80;
        copy_from_user(kbuf, buffer, copy_size);
        kbuf[copy_size] = '\0';
        if ( mysscanf(kbuf, "0x%lx 0x%lx", &offset, &value) == 2
            && (offset % 2) == 0 ){
                WriteEEProm16(priv_p, (UINT8)(offset>>1), (UINT16)value);
                KDBG_LOG_TEMP(KDBG_INFO,"Write 0x%04lx to eeprom offset=0x%04lx",
                        value, offset);
        }
        else if ( kbuf[0] != '\n' ){
                KDBG_LOG_TEMP(KDBG_INFO,"Usage:   echo \"offset(2) value\" > eeprom\n");
                KDBG_LOG_TEMP(KDBG_INFO,"Example: echo \"0x0c 0xffff\" > eeprom\n");
        }

        return count;
}

/*----------------------------------------------------------------*/
/*!
* \brief The PROC initial function for Access Point.
*
* This function create a PROC fs in linux /proc/net subdirectory
* First create a AP2220 directory, then create some sub-directory.
* This procedure only present when WLAN_INCLUDE_PROC=1
*
* \param  priv_p the management entry
* \return  not use.
*/
/*----------------------------------------------------------------*/
void init_2220_proc(struct net_device *dev, char *dev_name)
{
        i2220mgmt_t             *priv_p = kal_dev_priv(dev);
        struct proc_dir_entry   *entry;
        char                    str[6+IFNAMSIZ];

        if (priv_p && proc_net != NULL) {

        /* setup root directory, /proc/net/wlan0, consists of
         * files (version, info, statistics, config) and
         * sub-directories (debug, hw, STA)
         */
        sprintf(str, "net/%s", dev_name);
        priv_p->proc_root_dir = create_proc_d(str, 0, proc_dir_entry_t);
        if (priv_p->proc_root_dir == NULL) return;

        /* setup proc/net/wlan0/version file (read only) */
        entry = create_proc_read_entry("version", 0, priv_p->proc_root_dir,
                               proc_version_read, priv_p);

        /* setup proc/net/wlan0/config file (read only) */
        entry = create_proc_read_entry("config", 0, priv_p->proc_root_dir,
                               proc_config_read, priv_p);

        /* setup proc/net/wlan0/info file (read only) */
        entry = create_proc_read_entry("info", 0, priv_p->proc_root_dir,
                               proc_info_read, priv_p);

        /* setup proc/net/wlan0/statistics file (read only) */
        entry = create_proc_read_entry("statistics", 0, priv_p->proc_root_dir,
                               proc_stat_read, priv_p);

        /* setup proc/net/wlan0/eeprom file (read/write) */
        entry = create_proc_read_entry("eeprom", 0, priv_p->proc_root_dir,
                                proc_eeprom_read, priv_p);
        if (entry) entry->write_proc = proc_eeprom_write;

        /* setup proc/net/wlan0/hw sub-directory, which consists of
         * pci, mcr and bcr files.
         */
        priv_p->proc_hw_dir = create_proc_d("hw", priv_p->proc_root_dir, proc_dir_entry_t);
        if (priv_p->proc_hw_dir == NULL) return;

        entry = create_proc_read_entry("pci", 0, priv_p->proc_hw_dir,
                                proc_pci_read, priv_p);
        if (entry) entry->write_proc = proc_pci_write;

        entry = create_proc_read_entry("mcr", 0, priv_p->proc_hw_dir,
                                proc_mcr_read, priv_p);
        if (entry) entry->write_proc = proc_mcr_write;

        entry = create_proc_read_entry("bcr", 0, priv_p->proc_hw_dir,
                                proc_bcr_read, priv_p);
        if (entry) entry->write_proc = proc_bcr_write;

#if WLAN_INCLUDE_DEBUG
        /* setup proc/net/wlan0/STA sub-directory, which consists of all
         * associated STA infomation.
         */
        priv_p->proc_STA_dir = create_proc_d("STA", priv_p->proc_root_dir, proc_dir_entry_t);

#endif /* WLAN_INCLUDE_DEBUG */

        } /* end of proc_net!=NULL */
}

/*----------------------------------------------------------------*/
/*!
* \brief The remove function for Access Point PROC.
*
* This function remove the PROC fs in linux /proc/net/ap2220
* and all sub-directory.
* This procedure only present when WLAN_INCLUDE_PROC=1
*
* \param   none
* \return  not use.
*/
/*----------------------------------------------------------------*/
void remove_2220_proc(struct net_device *dev, char *dev_name)
{
        i2220mgmt_t             *priv_p = kal_dev_priv(dev);

        if (!priv_p || priv_p->proc_root_dir == NULL) {
                return;
        }

#if WLAN_INCLUDE_DEBUG
        /* remove STA sub-directory. Its files should be removed
         * in free_sta function before invoking this function.
         */
        remove_proc_entry("STA", priv_p->proc_root_dir);
#endif

        /* remove hw sub-directory */
        if (priv_p->proc_hw_dir) {
                remove_proc_entry("bcr", priv_p->proc_hw_dir);
                remove_proc_entry("mcr",priv_p->proc_hw_dir);
                remove_proc_entry("pci", priv_p->proc_hw_dir);
        }
        remove_proc_entry("hw", priv_p->proc_root_dir);

        /* remove eeprom file */
        remove_proc_entry("eeprom", priv_p->proc_root_dir);

        /* remove statistics file */
        remove_proc_entry("statistics", priv_p->proc_root_dir);

        /* remove info file */
        remove_proc_entry("info", priv_p->proc_root_dir);

        /* remove config file */
        remove_proc_entry("config", priv_p->proc_root_dir);

        /* remove version file */
        remove_proc_entry("version", priv_p->proc_root_dir);

        /* remove root directory (proc/net/wlan0) */
        remove_proc_entry(dev_name, proc_net);

#if SUPPORT_RSN
        //remove_proc_entry("authorsm", proc_net);
#endif
}

#if WLAN_INCLUDE_DEBUG
/*----------------------------------------------------------------*/
/*!
* \brief The read function for station Statistic.
*
* This function read the Statistic for each station
* including tx_packet, rx_packet, ...etc.
* This procedure only present when WLAN_INCLUDE_PROC=1
*
* \param   the PROC CONST defined parameters
* \return  not use.
*/
/*----------------------------------------------------------------*/
int proc_sta_read(char *page, char **start, off_t off,
                  int count, int *eof, void *data)
{
        char            *p = page;
        sta_info_t      *sta_p = (sta_info_t *) data;
        UINT32          idleTime = kal_jiffies - sta_p->actTime;

        ASSERT(sta_p);
        p+=sprintf(p,"Record Type     = ");

        if (sta_p->aid > 0) {
                p+=sprintf(p,"Associated STA (aid=%d)\n", sta_p->aid);
                p+=sprintf(p,"NonERP          = %d\n",(sta_p->incNonERP>0) ? 1 : 0);
                p+=sprintf(p,"LngPreamble     = %d\n",(sta_p->incLngPreamble>0) ? 1 : 0);
                p+=sprintf(p,"LngSlotTime     = %d\n",(sta_p->incLngSlotTime>0) ? 1 : 0);
                p+=sprintf(p,"Power Saving    = %d\n", sta_p->asoc.adPsm);
                p+=sprintf(p,"Supported Rates = 0x%08lx\n", sta_p->asoc.adSuptRates);
                p+=sprintf(p,"CurTxRate       = %d Mbps\n", sta_p->curTxRate_p->txRate>>1);
                p+=sprintf(p,"Association Time= %ld minutes\n",
                        kal_jiffies >= sta_p->asoc.adAsocTickTime ?
                        (kal_jiffies - sta_p->asoc.adAsocTickTime) / HZ / 60 :
                        ((UINT32)0xffffffff - sta_p->asoc.adAsocTickTime + kal_jiffies) / HZ / 60);
        }
        else if (sta_p->aid < 0) {
                p+=sprintf(p,"Remote WDS AP\n");
        }
        else if (sta_p->incNonERP == NEIGHBOR_NONERP) {
                p+=sprintf(p,"Neighbor NonERP BSS\n");
                idleTime -= (STA_MAX_INACTIVITY - 6*HZ);
        }
        else {
                p+=sprintf(p,"Not Associated STA\n");
        }
        p+=sprintf(p,"Average RSSI    = 0x%02x\n", sta_p->bbCr29);
        p+=sprintf(p,"Idle Time       = %ld ms\n", idleTime * 1000 / HZ);
        p+=sprintf(p,"\n");

        return (p - page);
}


/* Called from scheduled AP frame handlers and hostapd ioctl() */
/*----------------------------------------------------------------*/
/*!
* \brief The function for create a proc for each station.
*
* The function for create a proc for each station.
* This procedure only present when WLAN_INCLUDE_PROC=1
*
* \param   sta the station info structure sta_info_t
* \return  not use.
*/
/*----------------------------------------------------------------*/
void add_sta_proc(i2220mgmt_t *priv_p, sta_info_t *sta_p)
{
        char name[20];

        KDBG_ENTER;
        if (priv_p->proc_STA_dir == NULL) return;

        sprintf(name, MACSTR, MAC2STR(sta_p->asoc.adAddr));
        sta_p->proc = create_proc_read_entry(name, 0, priv_p->proc_STA_dir,
                                           proc_sta_read, sta_p);
        KDBG_EXIT;
}
#endif /* WLAN_INCLUDE_DEBUG */


