/*
** $Id: sdl_MLME_AP.c 2.12 2004/10/06 10:45:01 cmchang Exp $
*/

/*! \file sdl_MLME_AP.c
    \brief IEEE 802.11 standard MAC SDL, BLOCK MLME_AP

    In this block are the handlers for MLME operation requests, the responders
    for incoming management frames, and the time synchronization function for
    the AP, as well as contention free period timing if this AP includes a PCF.
    This block also contains the process which maintains record of power save mode
    and station state for access by other processes.
*/

/******************************************************************************
Copyright (c) 2002-2003 Integrated Programmable Communications, Inc.

All rights reserved. Copying, compilation, modification, distribution
or any other use whatsoever of this material is strictly prohibited
except in accordance with a Software License Agreement with
Integrated Programmable Communications, Inc.
******************************************************************************/

/************************************************************************
*                     C O M P I L E R   F L A G S
*************************************************************************
*/

/************************************************************************
*                E X T E R N A L   R E F E R E N C E S
*************************************************************************
*/
#include "precomp.h"

/************************************************************************
*                          C O N S T A N T S
*************************************************************************
*/

/************************************************************************
*                         D A T A   T Y P E S
*************************************************************************
*/


/************************************************************************
*                        P U B L I C   D A T A
*************************************************************************
*/
extern int      x_MAX_AID;

/************************************************************************
*                       P R I V A T E   D A T A
*************************************************************************
*/

/************************************************************************
*                             M A C R O S
*************************************************************************
*/

/************************************************************************
*              F U N C T I O N   D E C L A R A T I O N S
*************************************************************************
*/

/************************************************************************
*                          F U N C T I O N S
*************************************************************************
*/

/*----------------------------------------------------------------*/
/*!
* \brief  SDL utility function, to set the Supported Rate Information Element.
*
* Since the Supported Rate Information Element is carried in many type of frames,
* we use a seperate function to set the field.
*
* \param  cp  the byte position for this Supported Rate Information Element
*
* \return the byte position immediately after this Information Element
*/
/*----------------------------------------------------------------*/
UINT8* __set_SupRate_InfoElem(i2220mgmt_t *priv_p, UINT8 *cp, bool extended)
{
        int             i, num, startIndex;
        eSupRates_t     *eSupRates_p;
        eExtSupRates_t  *eExtSupRates_p;

        startIndex = priv_p->currentPhyType == PHY_TYPE_OFDM ? 4 : 0;

        if (!extended) {
                /* Supported rates */
                num = (mib_import(dot11OperationalRateSet.rateSetSize) < startIndex) ?
                      0 : mib_import(dot11OperationalRateSet.rateSetSize) - startIndex;
                if (num > 8) num = 8;

                sdl_mkElem( cp, eSupRates, num );
                for (i = 0; i < num; i++) {
                        eSupRates_p->supRates[i] = mib_import(
                                dot11OperationalRateSet.rateSet[i+startIndex]);
                }
        }
        else if (mib_import(dot11OperationalRateSet.rateSetSize) >
                 (8 + startIndex)) {
                /* Extended supported rates */
                num = mib_import(dot11OperationalRateSet.rateSetSize) -
                      8 - startIndex;

                sdl_mkElem( cp, eExtSupRates, num );
                for (i = 0; i < num; i++) {
                        eExtSupRates_p->supRates[i] = mib_import(
                                dot11OperationalRateSet.rateSet[i+8+startIndex]);
                }
        }

        return cp;
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return
*/
/*----------------------------------------------------------------*/
void* __search_infoElem(UINT8 eid, UINT8 *infoElem, int infoLen)
{
        while (infoLen > 0) {
                if (infoElem[0] == eid) {
                        return (void*)infoElem;
                }
                /* select next information element */
                infoLen -= infoElem[1] + 2;
                infoElem += infoElem[1] + 2;
        }

        return NULL;
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param  tx rate
* \return pointer to corresponding position in txDataRateList[]
*/
/*----------------------------------------------------------------*/
txRate_info_t* __get_txRate_info(i2220mgmt_t *priv_p, UINT8 txRate)
{
        txRate_info_t   *txRate_p;

        if (!priv_p->txRates_p) {
                KDBG_LOG_MLME(KDBG_WARN,"NULL txRates_p\n");
                return NULL;
        }

        txRate_p = priv_p->txRates_p;
        while (txRate_p->txRate) {
                if (txRate == txRate_p->txRate){
                        break;
                }
                txRate_p++;
        }

        return txRate_p->txRate ? txRate_p : NULL;
}

/*----------------------------------------------------------------*/
/*!
* \brief  SDL utility function, do rate conversion.
*
* This function will convert the \a RateString_t, a string of octets as
* defined by the standard, to the bitmapped \a RateSet_t type value. The
* maximum rate value is also calculated.
*
* \param  rate_p      pointer of the RateString_t object to be converted
* \param  rateSet_p   pointer of the rateSet_p object for the converted result
* \param  max_rate    pointer for the returned maximum rate value
* \return  return the StatusCode_t value as defined in the standard.
*/
/*----------------------------------------------------------------*/
StatusCode_t __convert_rates(RateString_t rate_p, RateSet_t *suptRateSet_p,
                        RateSet_t *basicRateSet_p, int numRates)
{
        int             i;
        DATA_RATE_INFO  *dataRateInfo_p;
        StatusCode_t    statusCode = SYN_successful;

        *suptRateSet_p = 0;
        *basicRateSet_p = 0;
        for (i = 0; i < numRates; i++, rate_p++) {
                if (*rate_p == 0) continue;
                if (rateGetDataRateInfo(sdl_rateVal(*rate_p), &dataRateInfo_p)) {
                        *suptRateSet_p |= dataRateInfo_p->rateSetBit;
                        if (sdl_isBasic(*rate_p)) {
                                *basicRateSet_p |= dataRateInfo_p->rateSetBit;
                        }
                }
                else {
                        KDBG_LOG_MLME(KDBG_NOTICE, "Unknown supported rate(0x%02x)\n", *rate_p);
                        statusCode = SYN_unsup_rate;
                }
        }

        return statusCode;
}

#if SUPPORT_MSSID
/*----------------------------------------------------------------*/
/*!
* \brief  SDL utility function, do rate conversion.
*
*
* \param  rate_p      pointer of the RateString_t object to be converted
* \param  rateSet_p   pointer of the rateSet_p object for the converted result
* \param  max_rate    pointer for the returned maximum rate value
* \return  return -1 no match, 0 primary, 1 ~ MAX_SSID -1 other SSID
*/
/*----------------------------------------------------------------*/
static int match_ssid(i2220mgmt_t *priv_p, eSsId_t *eSsId_p)
{
        ssid_info_t     *ssid;
        int             i;

        if (!eSsId_p) return -1;

        /*We reservce 0 for primary interface
        ** Please refer to sdlu_init_ap. priv_p->ap.ssid_list[0].enabled = true
        */
        for (i = 0; i < MAX_SSID; i++) {
                ssid = &priv_p->ap.ssid_list[i];
                if ((ssid->enabled) && (eSsId_p->length == ssid->ssid_len) &&
                    ipn_memcmp(eSsId_p->ssid, ssid->ssid,eSsId_p->length)== 0){
                        return i;
                }
        }

        return -1;
}
#endif

/*----------------------------------------------------------------*/
/*!
* \brief  utility function, update association information for both association
*         and reassociation frame types.
*
* The STA record of \a sta_p will be updated with the information from the
* received association or reassociation frame.
* SSID will be checked first, only matched one will get procecced.
* Then the supported rate information is updated.
*
* \param  sta_p   pointer of the STA record
* \param  cp      pointer of the Information Element field for the frame.
* \return return the StatusCode_t value as defined in the standard
*/
/*----------------------------------------------------------------*/
StatusCode_t __update_common_assoc(i2220mgmt_t *priv_p, sta_info_t *sta_p,
                                UINT8 *cp, UINT16 infoLen)
{

        eSsId_t         *eSsId_p;
        eSupRates_t     *eSupRates_p;
        eExtSupRates_t  *eExtSupRates_p;
        UINT8           txRate;
        UINT8           rate_num, supRates[24];
        UINT32          phyMode;
#if SUPPORT_MSSID
        int             whichSsid;
#endif

        /* Check SSID info element */
        eSsId_p = __search_infoElem(SYN_eSsId, cp, infoLen);
        if (!eSsId_p || eSsId_p->length > MAX_SSID_LEN) {
                return SYN_unspec_fail;
        }

#if SUPPORT_MSSID
        if ((whichSsid = match_ssid(priv_p, eSsId_p)) < 0) {
                return SYN_fail_other;
        }
        sta_p->whichSsid = whichSsid;
#else
        if (eSsId_p->length != mib_import(dot11DesiredSSID.ssidLen) ||
            ipn_memcmp(eSsId_p->ssid, mib_import(dot11DesiredSSID.ssid),
                  eSsId_p->length)) {
                return SYN_fail_other;
        }
#endif
        /* Check (extended) supported rate info element */
        eSupRates_p = __search_infoElem(SYN_eSupRates, cp, infoLen);
        if (!eSupRates_p ) {
                return SYN_unspec_fail;
        }
        rate_num = min((int)eSupRates_p->length, 24);
        ipn_memcpy(supRates, eSupRates_p->supRates, rate_num);

        eExtSupRates_p = __search_infoElem(SYN_eExtSupRates, cp, infoLen);
        if (eExtSupRates_p ) {
                ipn_memcpy(supRates + rate_num, eExtSupRates_p->supRates,
                        min((int)eExtSupRates_p->length, 24 - rate_num));
                rate_num += min((int)eExtSupRates_p->length, 24 - rate_num);
        }

        /* Check supported rates of STA */
        __convert_rates(supRates, &sta_p->asoc.adSuptRates,
                        &sta_p->asoc.adBasicRates, rate_num);
        if ((sta_p->asoc.adSuptRates & priv_p->basicRates) != priv_p->basicRates) {
                /* Not all basic rates are supported by the STA */
                return SYN_unsup_rate;
        }

        if (sta_p->asoc.adSuptRates & RATE_SET_MASK_ERP_OFDM) {
                /* It is an ERP STA*/
                sta_p->incNonERP = 0;
        }
        else {
                sta_p->incNonERP = 1;
        }

        /* Choose highest rate as initial tx rate for Wi-Fi */
        if (rateGetHighestRate(sta_p->asoc.adSuptRates & priv_p->supportRates,
                        RATE_54M, &txRate, &phyMode)) {
                sta_p->curTxRate_p = __get_txRate_info(priv_p, txRate);
        }
        else {
                sta_p->curTxRate_p = priv_p->lowestBRate_p;
        }
        sta_p->weightedRetryCount = RATE_SWITCH_INITIAL_WEIGHTED_COUNT;

        KDBG_LOG_MLME(KDBG_DEBUG,"SuptRate=0x%08lx,BasicRate=0x%08lx, "
                "txRate=%d Mbps\n", sta_p->asoc.adSuptRates, sta_p->asoc.adBasicRates,
                sta_p->curTxRate_p->txRate / 2);

        return SYN_sts_successful;
}

/*----------------------------------------------------------------*/
/*!
* \brief  utility function, update association information reassociation frame
*
* The STA record of \a sta_p will be updated with the information from the
* received reassociation frame.
* Here only the capability and listen-interval are updated, others are updated
* in the call of \a __update_common_assoc.
*
* \param  sta_p   pointer of the STA record
* \param  asRdu   pointer of the received association frame
* \return return the StatusCode_t value as defined in the standard
*/
/*----------------------------------------------------------------*/
StatusCode_t __update_reassoc_info(i2220mgmt_t *priv_p, reassoc_req_frame_t *asRdu,
                                sta_info_t *sta_p, bool reassoc)
{
        StatusCode_t    statusCode;
        UINT16          cap = ieee2host16(asRdu->capInfo);

        KDBG_ENTER;

        KDBG_LOG_SDL(KDBG_DEBUG,"STA " MACSTR " currAP=" MACSTR " cap=0x%04x listen=%d\n"
                , MAC2STR(asRdu->addr2)
                , MAC2STR(asRdu->currentApAddr)
                , ieee2host16(asRdu->capInfo)
                , ieee2host16(asRdu->listenInterval));

        if ((cap & (SYN_cPollable|SYN_cPollReq)) == SYN_cPollable ||
            (cap & (SYN_cChAgility|SYN_cPBCC)) ||
        #if 0 /* Symbol-B can not pass when WEP is enabled */
            ((cap & SYN_cPrivacy) && !priv_p->BCcipherSuite) ||
            (!(cap & SYN_cPrivacy) && priv_p->BCcipherSuite) ||
        #endif
            (!(cap & SYN_cShPreamble) &&
              priv_p->preambleMode == PREAMBLE_MODE_SHORT &&
              priv_p->currentPhyType != PHY_TYPE_OFDM)) {
                /* Not support CF-Poll, Channel Agility and PBCC
                 * Reject wrong privacy setting.
                 */
                KDBG_EXIT;
                return SYN_unsup_cap;
        }
        if (cap & SYN_cDsssOFDM) {
                KDBG_EXIT;
                return SYN_unsupt_DSOFDM;
        }

        /* If this record is nonERP AP before, we reset it */
        if (sta_p->incNonERP == NEIGHBOR_NONERP) {
                sta_p->incNonERP = 0;
                priv_p->bss_nonERP--;
        }

        /* Update preamble, slot time info by capability */
        if (ieee2host16(asRdu->capInfo) & SYN_cShPreamble ||
            priv_p->currentPhyType == PHY_TYPE_OFDM) {
                /* support short preamble */
                sta_p->incLngPreamble = 0;
        }
        else {
                sta_p->incLngPreamble = 1;
        }
        if (ieee2host16(asRdu->capInfo) & SYN_cShortSlot ||
            priv_p->currentPhyType == PHY_TYPE_OFDM) {
                /* support short slot time */
                sta_p->incLngSlotTime = 0;
        }
        else {
                /* avoid multiple association, so not true/false */
                sta_p->incLngSlotTime = 1;
        }

        /* Check information elements */
        statusCode = reassoc ?
                __update_common_assoc(priv_p, sta_p, asRdu->infoElem,
                        (UINT16) max((int)asRdu->LLClength - 10, 0)) :
                __update_common_assoc(priv_p, sta_p, asRdu->currentApAddr,
                        (UINT16) max((int)asRdu->LLClength - 4, 0));

        KDBG_EXIT;
        return statusCode;
}

/*----------------------------------------------------------------*/
/*!
* \brief  SDL utility function, to fill the beacon/probe-response frame fields.
*
* The beacon frame pointer \a yTdu, will be set with the proper value. This
* routine will set only the common part of beacon/probe-response frame, with
* the returned \a cp pointer for further Information Element treatment.
*
* \param  yTdu   pointer to the beacon or probe-response frame
* \param  yCfpm  the CF parameters from MLME-START request.
* \return the \a cp pointer for further Information Element treatment
*/
/*----------------------------------------------------------------*/
UINT8* __set_beacon_probrsp_frame(i2220mgmt_t *priv_p, beacon_frame_t *yTdu,
                                bool hide_ssid, int which_ssid)
{
        UINT8                   *cp;
        eSsId_t                 *eSsId_p;
        eDsParms_t              *eDsParms_p;
        eErpInfo_t              *eErpInfo_p;
#if SUPPORT_802_11D
        eCountryInfo_t          *eCountryInfo_p;
        DOMAIN_INFO_ENTRY       *domainInfo_p;
        int                     numSubBands;
#if SUPPORT_802_11A
        int                     i;
#endif
#endif /* SUPPORT_802_11D */

        KDBG_ENTER;
        yTdu->beaconInterval = host2ieee16(mib_import(dot11BeaconPeriod));

        /* Need more care, (ESS and support short preamble) */
        yTdu->capInfo = priv_p->phySlotType == phyLongSlot ?
                host2ieee16(0x0021) : host2ieee16(0x0021 | SYN_cShortSlot);

        if (priv_p->preambleMode == PREAMBLE_MODE_LONG)
                yTdu->capInfo &= ~host2ieee16(SYN_cShPreamble);
        if (priv_p->BCcipherSuite != CipherMode_NONE)
                yTdu->capInfo |= host2ieee16(SYN_cPrivacy);
        if (priv_p->currentPhyType == PHY_TYPE_OFDM)
                yTdu->capInfo &= ~host2ieee16(SYN_cShPreamble|SYN_cShortSlot);

        /* Information elements */
        cp = (UINT8*)&yTdu->infoElem[0];

        if (hide_ssid) {
                sdl_mkElem( cp, eSsId, 1 );
                eSsId_p->ssid[0] = 0;
        }
        else {
        #if SUPPORT_MSSID
                sdl_mkElem( cp, eSsId, priv_p->ap.ssid_list[which_ssid].ssid_len);
                ipn_memcpy( eSsId_p->ssid, priv_p->ap.ssid_list[which_ssid].ssid,
                        priv_p->ap.ssid_list[which_ssid].ssid_len);
        #else
                sdl_mkElem( cp, eSsId, mib_import(dot11DesiredSSID.ssidLen) );
                ipn_memcpy( eSsId_p->ssid, mib_import(dot11DesiredSSID.ssid), mib_import(dot11DesiredSSID.ssidLen));
        #endif
        }

        /* Setup supported rates */
        cp = __set_SupRate_InfoElem(priv_p, cp, FALSE);

        /* Setup RF channel */
        sdl_mkElem( cp, eDsParms, 1 );
        eDsParms_p->curChnl = (UINT8)priv_p->curChannel;

        /* Setup TIM if tx pkt is Beacon */
        if (sdl_ftype(yTdu) == SYN_beacon) {
                if (priv_p->dtimCount == 0 && sdlQ_Length(&priv_p->bcQ) != 0) {
                        cp = __mkTim(priv_p, cp, TRUE);
                }
                else {
                        cp = __mkTim(priv_p, cp, FALSE);
                }
        }

#if SUPPORT_802_11D
        if (priv_p->mib.dot11MultiDomainCapabilityEnabled &&
            priv_p->domainInfo_p && priv_p->domainInfo_p->countryCode) {
                domainInfo_p = priv_p->domainInfo_p;
                eCountryInfo_p = (eCountryInfo_t*) cp;

                eCountryInfo_p->countryString[0] =
                        (UINT8)(domainInfo_p->countryCode >> 8);
                eCountryInfo_p->countryString[1] =
                        (UINT8)(domainInfo_p->countryCode);
                eCountryInfo_p->countryString[2] = ' ';/*  Indoor and outdoor */

                numSubBands = 0;
                if (domainInfo_p->band24.numChannels > 0) {
                        eCountryInfo_p->subBands[0].firstChannelNum =
                                domainInfo_p->band24.firstChannelNum;
                        eCountryInfo_p->subBands[0].numChannels =
                                domainInfo_p->band24.numChannels;
                        eCountryInfo_p->subBands[0].maxTxPowerLevel =
                                domainInfo_p->band24.maxTxPowerLevel;
                        numSubBands++;
                }

        #if SUPPORT_802_11A
                for (i = 0; i < 4; i++) {
                        if (domainInfo_p->band5[i].numChannels > 0) {
                                eCountryInfo_p->subBands[numSubBands].firstChannelNum =
                                        domainInfo_p->band5[i].firstChannelNum;
                                eCountryInfo_p->subBands[numSubBands].numChannels =
                                        domainInfo_p->band5[i].numChannels;
                                eCountryInfo_p->subBands[numSubBands].maxTxPowerLevel =
                                        domainInfo_p->band5[i].maxTxPowerLevel;
                                numSubBands++;
                        }
                }
        #endif
                sdl_mkElem( cp, eCountryInfo, (3 + 3*numSubBands + 1) & ~1);
                if (!(numSubBands & 1)) {
                        *(UINT8*)(cp - 1) = 0; /* Pad with 0 content */
                }
        }
#endif /* end of SUPPORT_802_11D */

        /* Setup ERP information element */
        if ((priv_p->supportRates & RATE_SET_MASK_ERP_OFDM) &&
            priv_p->currentPhyType != PHY_TYPE_OFDM) {
                sdl_mkElem( cp, eErpInfo, 1 );
                eErpInfo_p->info = priv_p->sta_nonERP ? BITS2(0,1) : 0;
                if (priv_p->protectMode == CTS_PROTECT_NONE) {
                        eErpInfo_p->info &= ~BIT(1); /* not use protection */
                }
                else if (priv_p->bss_nonERP > 0 ||
                         priv_p->protectMode == CTS_PROTECT_MUST) {
                        eErpInfo_p->info |= BIT(1); /* use protection */
                }

                if (priv_p->phyPreambleType == phyLongPreamble) {
                        eErpInfo_p->info |= BIT(2); /* set Barker preamble mode */
                }
        }

        /* Setup extended supported rates */
        cp = __set_SupRate_InfoElem(priv_p, cp, TRUE);

        /* Setup WPA information element */
        cp = __mkWpaIE(priv_p, cp);

        KDBG_EXIT;
        return cp;
}

/*----------------------------------------------------------------*/
/*!
* \brief  utility function to make the TIM.
*
* This procedural operator is part of sort TrafficMap.
* __mkTim builds the info field for a TIM element from the DTIM
* count and DTIM period values and the contents of the (xlo:xhi)
* range of bits in the TrafficMap. The resulting Octetstring can
* be used as an operand of mkElem (by an AP generating a Beacon
* frame).
*
* \param  cp      pointer to the current Information Element
* \param  bcast   whether there are any buffered broadcast packets
*                 NOTE: it will only happen when mDtimCount==0
* \return next _cp_ position
*/
/*----------------------------------------------------------------*/
UINT8* __mkTim(i2220mgmt_t *priv_p, UINT8* cp, bool bcast)
{
        eTim_t  *eTim_p;
        int     i, k, bmCtl;

        KDBG_ENTER;

        /* Normally these cover the range of AId values
         * in use, but subsets are permitted.
         */
        i = priv_p->ap.tmap_xlo;
        k = priv_p->ap.tmap_xhi;
        KDBG_LOG_MLME(KDBG_FREQ,"Send TIM info elem [%d,%d]\n",i,k);

        i = (i/8) & 0xFE;               /* even */
        k = 3 + (k - i*8) / 8 + 1;      /* length */
        ASSERT(k>=4);

        /* Add starting index to bc/mc indicator to get bitmap
         * control field value for TIM.
         */
        bmCtl = (bcast) ? (i + 1) : i;

        sdl_mkElem( cp, eTim, k );
        eTim_p->dtimCount = priv_p->dtimCount;
        eTim_p->dtimPeriod = mib_import(dot11DTIMPeriod);
        eTim_p->bitmapCtrl = (UINT8)bmCtl;
        ipn_memcpy( &eTim_p->bitmap[0], &priv_p->ap.tmap[i], k - 3 );

        KDBG_LOG_MLME(KDBG_FREQ,"TIM(%d,%d,0x%x) -- %02x %02x %02x %02x\n",
                        priv_p->dtimCount, mib_import(dot11DTIMPeriod), bmCtl,
                        eTim_p->bitmap[0], eTim_p->bitmap[1],
                        eTim_p->bitmap[2], eTim_p->bitmap[2]);

        KDBG_EXIT;
        return cp;
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return
*/
/*----------------------------------------------------------------*/
void __process_sta_roaming(i2220mgmt_t *priv_p, UINT8 *macAddr)
{
        ethhdr_t        *eth_p;
        kal_skb_t       *skb;
        UINT8           *llc;

        if (priv_p->relay_mode == RelayMode_SWITCH ||
            priv_p->relay_mode == RelayMode_SWITCH_NO_FORWARD) {

                /* Send a bcast frame to DS for roaming */
                if ((skb = kal_alloc_skb(60)) != NULL) {
                        kal_skb_put(skb, 60);
                        ipn_memset(kal_skb_data(skb), 0, 60);

                        eth_p = (ethhdr_t *)kal_skb_data(skb);
                        ipn_memset(eth_p->h_dest, 0xff, MAC_ADDR_LEN);
                        ipn_memcpy(eth_p->h_source, macAddr, MAC_ADDR_LEN);
                        eth_p->h_proto = host2ieee16(46); /* length */

                        /* Fill Inprocomm OUI (00 08 22) */
                        llc = (UINT8*)&eth_p->h_proto + 2;
                        llc[0] = 0xaa; llc[1] = 0xaa; llc[2] = 0x03;
                        llc[3] = 0x00; llc[4] = 0x08; llc[5] = 0x22;

                        kal_netdev_rx(skb, priv_p->netdev);
                }
		else printk("__process_sta_roaming: kal_alloc_skb fail\n");
        }
}

/*-----------------------------------------------------*/
/* The following functions are added by CMC venus 6/12 */
/*-----------------------------------------------------*/

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return
*/
/*----------------------------------------------------------------*/
void rx_process_ps_poll(i2220mgmt_t *priv_p, Frame_t *pdu, sta_info_t *sta_p)
{
        FragSdu_t       *fsdu;

        KDBG_ENTER;

        KDBG_LOG_MLME(KDBG_DEBUG,"Rx ps-poll frame from "
                MACSTR "\n", MAC2STR(pdu->addr2));
        if (sta_p == NULL || sta_p->aid == 0) {
                sdlf_send_deauth(priv_p, pdu->addr2, SYN_class3_err);
                return;
        }

        KDBG_LOG_MLME(KDBG_DEBUG,"qLen=%d\n", sdlQ_Length(&sta_p->psQ));

        do {
                sdlQ_dequeue_first(&sta_p->psQ, fsdu);
                if (fsdu && check_timeout(fsdu->eol))
                        sdlu_release_fsdu(fsdu);
                else    break;
        } while (1);
        if (fsdu == NULL) {
                if (sta_p->aid > 0)
                        sdl_reset_tmap(priv_p,sta_p->aid);
                return;
        }

        if (sdlQ_Length(&sta_p->psQ) != 0){
                sdl_setMoreData(fsdu->sw.sdu, 1);
        }
        else {
                sdl_reset_tmap(priv_p,sta_p->aid);
        }
        /* Start to tx out a series of fragments
         * Now we don't check its eol because it's in PS mode ??
         */
        sdlu_tx_FragSdu(priv_p, fsdu, TXQ_AC0, TRUE);

        KDBG_EXIT;
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return
*/
/*----------------------------------------------------------------*/
void rx_process_beacon(i2220mgmt_t *priv_p, Frame_t *pdu, sta_info_t *sta_p)
{
        beacon_frame_t          *rpdu = (beacon_frame_t*)pdu;
        eSupRates_t             *eSupRates_p;
        eDsParms_t              *eDsParms_p;
        eErpInfo_t              *eErpInfo_p;
        eExtSupRates_t          *eExtSupRates_p;
        sta_info_t              *bss_sta_p;
        UINT8                   tmp, ErpInfo;
        int                     infoLen = rpdu->LLClength - 12;
        UINT32                  suptRates, basicRates;
        bool                    isNonERP;

        KDBG_ENTER;
        /* judge if nonERP BSS is our neighbor */

        eSupRates_p = __search_infoElem(SYN_eSupRates, rpdu->infoElem, infoLen);
        if (!eSupRates_p || eSupRates_p->length > 24) return;

        eDsParms_p = __search_infoElem(SYN_eDsParms, rpdu->infoElem, infoLen);
        if (!eDsParms_p || (eDsParms_p->curChnl != priv_p->curChannel
              && priv_p->hw_state == I2220HW_STATE_RUNNING))
                return;

        eErpInfo_p = __search_infoElem(SYN_eErpInfo, rpdu->infoElem, infoLen);
        ErpInfo = eErpInfo_p ? eErpInfo_p->info : 0;

        eExtSupRates_p = __search_infoElem(SYN_eExtSupRates, rpdu->infoElem, infoLen);
        if (eExtSupRates_p && eExtSupRates_p->length <= 24) {
                /* Note: original pkt content is damaged here */
                tmp = eExtSupRates_p->length;
                ipn_memcpy(&eSupRates_p->supRates[eSupRates_p->length],
                       &eExtSupRates_p->supRates[0], tmp);
                eSupRates_p->length += tmp;
        }
        __convert_rates(eSupRates_p->supRates, &suptRates,
                        &basicRates, eSupRates_p->length);

        isNonERP = (!(suptRates & RATE_SET_MASK_ERP_OFDM) || (ErpInfo & BIT(0))) ?
                        TRUE : FALSE;

        if (priv_p->hw_state == I2220HW_STATE_RUNNING) {
                /* NonERP BSS neighbor or ERP BSS neighbor associated with
                 * nonERP STA, apply nonERP protection. If this AP is also a
                 * STA (aid != 0), we ignore it now.
                 */
                if (isNonERP && (bss_sta_p = sdlu_query_sta(priv_p, pdu->addr3)) != NULL
                    && bss_sta_p->aid == 0) {
                        /* 6sec aging time */
                        bss_sta_p->actTime = kal_jiffies - STA_MAX_INACTIVITY + 6*HZ;
                        if (bss_sta_p->incNonERP == 0) {
                                bss_sta_p->incNonERP = NEIGHBOR_NONERP;
                                priv_p->bss_nonERP++;

                                KDBG_LOG_MLME(KDBG_DEBUG,"NonERP BSS " MACSTR
                                        " is detected. bss_nonERP=%d\n",
                                        MAC2STR(pdu->addr3), priv_p->bss_nonERP);
                        }
                }
                /* If a BSS is changed to ERP from nonERP, we will detect
                 * this case when time out (aging, ap_timer_func())
                 */
        }
        else if (priv_p->chnl_info_p) {
                chnl_info_t     *chnl_info_p;
                int             i, index;

                /* record all listened BSS in scanning state */
                chnl_info_p = priv_p->chnl_info_p + priv_p->scan_index;
                bss_sta_p = sdlu_query_sta(priv_p, pdu->addr3);
                for (i = 0; i < BSS_MAX_NUM_PER_CHNL; i++) {
                        if (bss_sta_p == chnl_info_p->bss[i])
                                break;
                }

                index = chnl_info_p->bss_nonERP + chnl_info_p->bss_ERP;
                if (i >= BSS_MAX_NUM_PER_CHNL && index < BSS_MAX_NUM_PER_CHNL) {
                        /* New detected neighbor BSS */
                        if (isNonERP) {
                                chnl_info_p->bss_nonERP++;
                        }
                        else {
                                chnl_info_p->bss_ERP++;
                        }
                        chnl_info_p->bss[index] = bss_sta_p;
                }
                if (bss_sta_p) bss_sta_p->actTime = kal_jiffies;
        }

        KDBG_EXIT;
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return
*/
/*----------------------------------------------------------------*/
void rx_process_probe_req(i2220mgmt_t *priv_p, Frame_t *pdu, sta_info_t *sta_p)
{
        probe_req_frame_t       *rpdu = (probe_req_frame_t*)pdu;
        probe_rsp_frame_t       *yTdu;
        UINT8                   *cp;
        eSsId_t                 *eSsId_p;
        bool                    hide_ssid;
        int                     whichSsid = 0; /* Default primary */

        KDBG_ENTER;

        KDBG_LOG_MLME(KDBG_DEBUG,"Rx probe_req frame from "
                MACSTR "\n", MAC2STR(pdu->addr2));

        if (priv_p->hw_state != I2220HW_STATE_RUNNING) return;

        /* check SSID and capacity */
        eSsId_p = __search_infoElem(SYN_eSsId, rpdu->infoElem, rpdu->LLClength);

#if SUPPORT_MSSID
        if (!eSsId_p || (eSsId_p->length != 0 &&
            (whichSsid = match_ssid(priv_p, eSsId_p)) < 0)) {
                KDBG_LOG_MLME(KDBG_DEBUG,"SSID not matched. Ignore.\n");
                return;
        }
#else
        if (!eSsId_p || (eSsId_p->length != 0 &&
            eSsId_p->length != mib_import(dot11DesiredSSID.ssidLen))
            || ipn_memcmp(eSsId_p->ssid, mib_import(dot11DesiredSSID.ssid),
               eSsId_p->length)) {
                KDBG_LOG_MLME(KDBG_DEBUG,"SSID not matched. Ignore.\n");
                return;
        }
#endif
        yTdu = (probe_rsp_frame_t*) sdl_mkFrame(priv_p, sizeof(probe_rsp_frame_t),
                        SYN_probe_rsp, pdu->addr2, mib_import(dot11MACAddress));
        if (yTdu == NULL) return;

        hide_ssid = (priv_p->x_hide_ssid && eSsId_p->length == 0) ? TRUE : FALSE;

        cp = __set_beacon_probrsp_frame(priv_p, yTdu, hide_ssid, whichSsid);

        yTdu->LLClength = (UINT16)(cp - yTdu->LLCdata);
        yTdu->mac_ctl |= MACCTL_NEED_TIMESTAMP;
        sdu_send_request(priv_p, (Frame_t *)yTdu, NULL, SYN_contention);

        KDBG_EXIT;
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return
*/
/*----------------------------------------------------------------*/
void rx_process_reasoc_req(i2220mgmt_t *priv_p, Frame_t *pdu, sta_info_t *sta_p)
{
        reassoc_req_frame_t     *rpdu = (reassoc_req_frame_t*)pdu;
        reassoc_rsp_frame_t     *yTdu;
        UINT8                   *cp;
        UINT16                  statusCode;
        bool                    isReassoc = TRUE;

        KDBG_ENTER;

        KDBG_LOG_MLME(KDBG_DEBUG,"Rx (re)assoc frame from "
                MACSTR "\n", MAC2STR(pdu->addr2));

        if (!sta_p || !(sta_p->flags & (STA_INFO_AUTH_OS|STA_INFO_AUTH_KEY))) {
                /* not authenticated, send class2_err deauth frame */
                sdlf_send_deauth(priv_p, pdu->addr2, SYN_class2_err);
                KDBG_EXIT;
                return;
        }

        yTdu = (reassoc_rsp_frame_t*) sdl_mkFrame(priv_p, sizeof(reassoc_rsp_frame_t),
                        SYN_reasoc_rsp, pdu->addr2, mib_import(dot11MACAddress));
        if (yTdu == NULL) return;

        if (sdl_ftype(pdu) == SYN_asoc_req) {
                yTdu->frameCtrl = host2ieee16(SYN_asoc_rsp);
                isReassoc = FALSE;
        }
        /* Fill supported rates */
        cp = __set_SupRate_InfoElem(priv_p, yTdu->supRates, FALSE);
        cp = __set_SupRate_InfoElem(priv_p, cp, TRUE);
        yTdu->LLClength -= sizeof(yTdu->supRates) - (cp - yTdu->supRates);

        /* Here: check SSID, support_rates, capability */
        statusCode = __update_reassoc_info(priv_p, rpdu, sta_p, isReassoc);

        if (statusCode == SYN_sts_successful) {
                yTdu->assocId = host2ieee16(task_assign_AId(priv_p, sta_p) | MASK_DI_AID_MSB);
                if (yTdu->assocId == host2ieee16(MASK_DI_AID_MSB)) {
                        statusCode = SYN_ap_full;
                }
        }
        sta_p->KeyMapOn = FALSE; /* Disable old security key */

        if (statusCode == SYN_sts_successful) {
                /* Incidate upper layer for an successful (re)association */
                if (isReassoc) {
                        statusCode = (UINT16)kal_assoc_request(priv_p->netdev, rpdu->addr2,
                                rpdu->infoElem, max((int)rpdu->LLClength - 10, 0));
                }
                else {
                        statusCode = (UINT16)kal_assoc_request(priv_p->netdev, rpdu->addr2,
                                rpdu->currentApAddr, max((int)rpdu->LLClength - 4, 0));
                }

                if (statusCode == SYN_sts_successful) {
                        sta_p->asoc.adAsocTickTime = kal_jiffies; /* association time */

                        /* Update nonERP, preamble and slot time info */
                        update_bss_operating(priv_p);

                        KDBG_LOG_MLME(KDBG_DEBUG,"sta_nonERP=%d\n", priv_p->sta_nonERP);
                        KDBG_LOG_MLME(KDBG_DEBUG,"sta_LngPreamble=%d\n", priv_p->sta_LngPreamble);
                        KDBG_LOG_MLME(KDBG_DEBUG,"sta_LngSlotTime=%d\n", priv_p->sta_LngSlotTime);

                } /* end of check NonERP protection and preamble type */

                /* Notify the Distribution System by send a dummy packet */
                if (statusCode == SYN_sts_successful) {
                        __process_sta_roaming(priv_p, pdu->addr2);
                }
        }

        if (statusCode != SYN_sts_successful) {
                /* Cut the old communication if this STA was associated
                 * and avoid misunderstanding of preamble, slot time and nonERP
                 */
                free_sta(priv_p, sta_p);
        }

        KDBG_LOG_MLME(KDBG_DEBUG,"Response (re)assoc frame "
                "(status=%d)(aid=%ld)\n", statusCode, yTdu->assocId & MASK_DI_AID);

        /* Fill capability */
        yTdu->capInfo = priv_p->phySlotType == phyLongSlot ?
                host2ieee16(0x0021) : host2ieee16(0x0021 | SYN_cShortSlot);
        if (priv_p->BCcipherSuite != CipherMode_NONE)
                yTdu->capInfo |= host2ieee16(SYN_cPrivacy);

        yTdu->statusCode = host2ieee16(statusCode);
        sdu_send_request(priv_p, (Frame_t *)yTdu, NULL, SYN_contention);

        /* whsu for waveplus */
#if WP_3210_WORKAROUND /* WP ecos for associate response indicate */
        if (statusCode == SYN_sts_successful)
                kal_assoc_resp_indicate(priv_p->netdev,pdu->addr2,1);
        else
                kal_assoc_resp_indicate(priv_p->netdev,pdu->addr2,0);
#endif
        KDBG_EXIT;
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return
*/
/*----------------------------------------------------------------*/
void rx_process_disasoc(i2220mgmt_t *priv_p, Frame_t *pdu, sta_info_t *sta_p)
{
#if WLAN_INCLUDE_DEBUG
        disassoc_frame_t        *rpdu = (disassoc_frame_t*)pdu;
#endif
        KDBG_ENTER;
#if WLAN_INCLUDE_DEBUG
        KDBG_LOG_MLME(KDBG_DEBUG,"Rx disassoc frame (reason=%d) from "
                MACSTR "\n", ieee2host16(rpdu->reasonCode), MAC2STR(pdu->addr2));
#endif
        rx_process_deauth(priv_p, pdu, sta_p);
        KDBG_EXIT;
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return
*/
/*----------------------------------------------------------------*/
void rx_process_auth(i2220mgmt_t *priv_p, Frame_t *pdu, sta_info_t *sta_p)
{
        auth_frame_t    *yTdu, *rpdu = (auth_frame_t*)pdu;
        UINT16          seq = ieee2host16(rpdu->authTransSeqNo);
        UINT16          alg = ieee2host16(rpdu->authAlgNum);
        UINT16          statusCode = SYN_sts_successful;
        int             i, j;

        KDBG_ENTER;
        if (sta_p == NULL) {
                sta_p = create_sta(priv_p, pdu->addr2);
                if (sta_p == NULL) {
                        KDBG_EXIT;
                        return;
                }
        }

        yTdu = (auth_frame_t*) sdl_mkFrame(priv_p, sizeof(auth_frame_t),
                        SYN_auth, pdu->addr2, mib_import(dot11MACAddress));
        if (yTdu == NULL) return;

        /* general length except challenge's frame */
        yTdu->LLClength -= sChallengeLng + 2;

        KDBG_LOG_MLME(KDBG_DEBUG,"Rx auth frame (seq=%d, alg=%d) from "
                MACSTR "\n", seq, alg, MAC2STR(pdu->addr2));

        if (seq == 1) {
                sta_p->flags &= ~(STA_INFO_AUTH_OS |STA_INFO_AUTH_KEY);

                if (task_find_authAlg(priv_p, alg) < 0){
                        statusCode = SYN_unsupt_alg;
                }
                /* Indicate auth_req to data base of upper layer */
                else if (kal_auth_request(priv_p->netdev, pdu->addr2) ||
                    sta_p->flags & STA_INFO_ACL) {
                        /* Rejected by ACL */
                        statusCode = SYN_fail_other;
                        sta_p->auth_state = ST_Auth_Rsp_Idle;
                }
                else if (alg == SYN_shared_key){
                        sta_p->challenge_xor = kal_jiffies;

                        yTdu->ElementID = SYN_eCtext;
                        yTdu->Length = sChallengeLng;

                        /* generate challenge text */
                        for (i = 0; i < sChallengeLng/8; i++) {
                                for (j = 0; j < 8; j++) {
                                        yTdu->challengeText[8*i+j] =
                                          (UINT8)((sta_p->challenge_xor>>i)
                                                ^ ((UINT32)sta_p>>j));
                                }
                        }
                        yTdu->LLClength += sChallengeLng + 2;
                        sta_p->auth_state = ST_Wait_Chal_Rsp;
                }
                else {
                        /* open system, successful */
                        sta_p->flags |= STA_INFO_AUTH_OS;
                        sta_p->auth_state = ST_Auth_Rsp_Idle;
                        KDBG_LOG_MLME(KDBG_DEBUG,"Open System Succeed!\n");
                        rpdu->authAlgNum = 0;
                }
        }
        else if (sta_p->auth_state == ST_Wait_Chal_Rsp && sdl_wepBit(pdu)) {
                sta_p->auth_state = ST_Auth_Rsp_Idle;

                if (seq != 3 || alg != SYN_shared_key) {
                        /* decryption error message */
                        statusCode = SYN_chlng_fail;
                        seq = 3;
                        alg = SYN_shared_key;
                }
                /* check if challenge text is correct */
                for (i = 0; i < sChallengeLng/8; i++) {
                        for (j = 0; j < 8; j++) {
                                if (rpdu->challengeText[8*i+j] !=
                                      (UINT8)((sta_p->challenge_xor>>i)
                                               ^ ((UINT32)sta_p>>j)) ){
                                        statusCode = SYN_chlng_fail;
                                        i = sChallengeLng/8;
                                        break;
                                }
                        }
                }
                if (statusCode == SYN_sts_successful) {
                        sta_p->flags |= STA_INFO_AUTH_KEY;
                        KDBG_LOG_MLME(KDBG_DEBUG,"Shared Key Succeed!\n");
                }
        }
        else {
                /* seq==3 or others */
                statusCode = SYN_auth_seq_fail;
        }

        KDBG_LOG_MLME(KDBG_DEBUG,"Response auth frame "
                "(seq=%d,alg=%d,status=%d)\n", seq+1, alg, statusCode);

        yTdu->authAlgNum = host2ieee16(alg);
        yTdu->authTransSeqNo = host2ieee16(seq+1);
        yTdu->statusCode = host2ieee16(statusCode);
        sdu_send_request(priv_p, (Frame_t *)yTdu, NULL, SYN_contention);

        KDBG_EXIT;
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return
*/
/*----------------------------------------------------------------*/
void rx_process_deauth(i2220mgmt_t *priv_p, Frame_t *pdu, sta_info_t *sta_p)
{
#if WLAN_INCLUDE_DEBUG
        deauth_frame_t  *rpdu = (deauth_frame_t*)pdu;
#endif

        KDBG_ENTER;

#if WLAN_INCLUDE_DEBUG
        KDBG_LOG_MLME(KDBG_DEBUG,"Rx deauth frame (reason=%d) from "
                MACSTR "\n", ieee2host16(rpdu->reasonCode), MAC2STR(pdu->addr2));
#endif
        if (sta_p == NULL) {
                KDBG_EXIT;
                return;
        }

        /* Not kill the wds sta_p record, otherwise the sta_p for other wds
         * will be free the same time, but we should keep the sta_p record
         * for the wds usage
         */
        if (sta_p->aid >= 0)
                free_sta(priv_p, sta_p);

        KDBG_EXIT;
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return
*/
/*----------------------------------------------------------------*/
Frame_t *sdl_mkFrame(i2220mgmt_t *priv_p, int max_size, TypeSubtype_t ft,
                        UINT8 *da, UINT8 *bssid)
{
        Frame_t         *yTdu;
        kal_skb_t       *skb;

        skb = kal_alloc_skb(max_size);
        if(skb == NULL) { printk("sdl_mkFrame: alloc skb fail\n"); return NULL;}

        yTdu = (Frame_t*) kal_skb_data(skb);
        yTdu->frameCtrl = host2ieee16(ft);
        ipn_memcpy( yTdu->addr1, da, MAC_ADDR_LEN );
        ipn_memcpy( yTdu->addr2, mib_import(dot11MACAddress), MAC_ADDR_LEN );
        ipn_memcpy( yTdu->addr3, bssid, MAC_ADDR_LEN );
        yTdu->mac_ctl = MACCTL_SDU_FRAME_IN_SKB; /* Not check PS and Security */
        yTdu->skb = skb;
        yTdu->LLCdata = (UINT8*)yTdu + wlanbody_offset;
        yTdu->LLClength = max_size - wlanbody_offset;

        return yTdu;
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return
*/
/*----------------------------------------------------------------*/
void sdlf_send_deauth(i2220mgmt_t *priv_p, MacAddr_t da, ReasonCode_t reason)
{
        deauth_frame_t          *deauth;

        KDBG_ENTER;

        deauth = (deauth_frame_t*) sdl_mkFrame(priv_p, sizeof(deauth_frame_t),
                        SYN_deauth, da, mib_import(dot11MACAddress));

        if (deauth != NULL) {
                deauth->reasonCode = host2ieee16(reason);
                sdu_send_request(priv_p, (Frame_t *)deauth,
                        NULL, SYN_contention);
        }
        KDBG_LOG_MLME(KDBG_DEBUG,"Send deauth (reason=%d) to "
                MACSTR "\n", reason, MAC2STR(da));

        KDBG_EXIT;
        return;
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return
*/
/*----------------------------------------------------------------*/
void sdlf_send_disassoc(i2220mgmt_t *priv_p, MacAddr_t da, ReasonCode_t reason)
{
        disassoc_frame_t          *disassoc;

        KDBG_ENTER;

        disassoc = (disassoc_frame_t*) sdl_mkFrame(priv_p, sizeof(disassoc_frame_t),
                        SYN_disasoc, da, mib_import(dot11MACAddress));

        if (disassoc != NULL) {
                disassoc->reasonCode = host2ieee16(reason);
                sdu_send_request(priv_p, (Frame_t *)disassoc,
                        NULL, SYN_contention);
        }
        KDBG_LOG_MLME(KDBG_DEBUG,"Send disassoc (reason=%d) to "
                MACSTR "\n", reason, MAC2STR(da));

        KDBG_EXIT;
        return;
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return
*/
/*----------------------------------------------------------------*/
void set_scan_timer(i2220mgmt_t *priv_p)
{
        mgmt_timer_t    *tmr = &priv_p->scan_timer;

        mgmt_stop_timer(tmr);
        mgmt_start_timer(tmr, MLME_SCAN_INTERVAL,
                     (mgmt_timeout_func) MlmeScan_request,
                     (unsigned long) priv_p);
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return
*/
/*----------------------------------------------------------------*/
void MlmeStart_request(i2220mgmt_t *priv_p)
{
        UINT32                  temp32;
        struct list_head        *lst, *next_lst;
        sta_info_t              *sta_p;

        if (priv_p->hw_state == I2220HW_STATE_PREINIT)
                return;

        /* Terminate the previous scanning loop if we are scanning */
        if (priv_p->hw_state == I2220HW_STATE_SCANNING) {
                mgmt_stop_timer(&priv_p->scan_timer);
                if (priv_p->chnl_info_p) {
                        kal_free(priv_p->chnl_info_p);
                        priv_p->chnl_info_p = NULL;
                }
        }
        else if (priv_p->hw_state == I2220HW_STATE_RUNNING) {
                /* Send a deauth frame with bcast address because some
                 * settings are changed. (Deauth all STAs.)
                 */
                sdlf_send_deauth(priv_p, bcstAddr, SYN_deauth_lv_ss);
                kal_mdelay(2); /* let the deauth frame sent out */
        }

        /* Clear up all station records except WDS */
        for (lst = priv_p->ap.sta_chain.head.next;
	     lst != &priv_p->ap.sta_chain.head; lst = next_lst) {
                next_lst = lst->next;
                sta_p = list_entry(lst, sta_info_t, sw.link);
                if (sta_p->aid >= 0) {
                        free_sta(priv_p, sta_p);
                }
        }

        if (mib_import(dot11CurrentChannel) == 0) {
                /* auto channel selection mode.
                 * Avoid TBTT event to send out Beacon during scanning
                 */
                tsftimerDisableTbttTimer(priv_p);
                tsftimerDisablePreTbttTimer(priv_p);
                tsftimerDisableDtimTimer(priv_p);

                /* allocate memory to store scan result */
                priv_p->chnl_info_p = (chnl_info_t*) kal_malloc(
                        sizeof(chnl_info_t)*MAX_NUM_RF_CHANNELS);
                if (priv_p->chnl_info_p == NULL) {
                        /* Note: the default channel should depend on
                         * regulation domain.
                         */
                #if SUPPORT_802_11A
                        priv_p->curChannel =
                                priv_p->channelBand == CHANNEL_BAND_5 ?
                                priv_p->operatingChannels5[0] :
                                priv_p->operatingChannels24[0];
                #else
                        priv_p->curChannel =
                                priv_p->operatingChannels24[0];
                #endif
                        printk(/*KDBG_LOG_MLME(KDBG_WARN,*/
                                "No memory for scan result. Use channel %d\n",
                                priv_p->curChannel);
                        i2220hw_drvr_start(priv_p);
                        return;
                }
                ipn_memset(priv_p->chnl_info_p, 0,
                        sizeof(chnl_info_t)*MAX_NUM_RF_CHANNELS);

                /* Clear CCA active time */
                NIC_MCR_READ(priv_p, MCR_CATR, &temp32);

                /* start passive scan for all available channels,
                 * are adjacent by one channel.
                 */
                priv_p->hw_state = I2220HW_STATE_SCANNING;
                priv_p->scan_index = 0;
        #if SUPPORT_802_11A
                priv_p->curChannel = priv_p->channelBand == CHANNEL_BAND_5 ?
                        priv_p->operatingChannels5[0] :
                        priv_p->operatingChannels24[0];
        #else
                priv_p->curChannel =
                        priv_p->operatingChannels24[0];
        #endif
                hw_set_channel(priv_p, 0xff);
                set_scan_timer(priv_p);

                pauEnable(priv_p);      /* Enable PAU */
                nicEnableInterrupt(priv_p);
        }
        else {
                /* Fixed channel setting */
                priv_p->curChannel = mib_import(dot11CurrentChannel);
                i2220hw_drvr_start(priv_p);

                /* If we change channel RF channel and does not stop RX DMA,
                 * it's very likely that RX DMA will stop work. So we reset_mac
                 * to prevent this. ???
                 */
                i2220hw_reset_mac(priv_p);
        }
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return
*/
/*----------------------------------------------------------------*/
UINT32 channel_noisyLevel(chnl_info_t *chnl_info_p)
{
#define NONERP_WEIGHT (MLME_SCAN_INTERVAL*1000*1000/HZ)

        return (chnl_info_p->bss_nonERP*NONERP_WEIGHT +
                chnl_info_p->bss_ERP*NONERP_WEIGHT/2 +
                chnl_info_p->CCA_activeTime*1);
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return
*/
/*----------------------------------------------------------------*/
static void MlmeScan_bestChannel(i2220mgmt_t *priv_p)
{
        int             i, j, bestChannel = 0;
        chnl_info_t     *chnl_info_p;
        UINT32          noisyLevel, lowestLevel=0xffffffff;
        UINT32          lowestRxPower=0xffffffff;
        int             numOperationChannels;
        UINT8           *chnlNum_p;

        if (mib_import(dot11CurrentChannel) > 0) {
                priv_p->curChannel = mib_import(dot11CurrentChannel);
                return;
        }
        /* Make sure our operation band */
#if SUPPORT_802_11A
        if (priv_p->channelBand == CHANNEL_BAND_5) {
                numOperationChannels = priv_p->numOperatingChannels5;
                chnlNum_p = &priv_p->operatingChannels5[0];
        }
        else
#endif
        {
                numOperationChannels = priv_p->numOperatingChannels24;
                chnlNum_p = &priv_p->operatingChannels24[0];
        }

        if (priv_p->chnl_info_p == NULL) {
                priv_p->curChannel = *chnlNum_p;
                return;
        }

        chnl_info_p = priv_p->chnl_info_p;
        for (i = 0; i < numOperationChannels; i++, chnl_info_p++) {
                /* To free scanned BSS records */
                for (j = 0; j < BSS_MAX_NUM_PER_CHNL; j++) {
                        if (chnl_info_p->bss[j] == NULL)
                                break;
                        if (chnl_info_p->bss[j]->magicID == STA_MAGIC_ID) {
                                /* Note: don't call free_sta() here, otherwise
                                 * repeated freeing will occur --> crash
                                 */
                                chnl_info_p->bss[j]->actTime =
                                        kal_jiffies - STA_MAX_INACTIVITY;
                        }
                }

                /* In Japan, channel 14 only allow b-only mode */
                if ((mib_import(dot11CurrentRegDomain) == MIB_REG_DOMAIN_JAPAN_1 ||
                     mib_import(dot11CurrentRegDomain) == MIB_REG_DOMAIN_JAPAN_2)
                    && *(chnlNum_p + i) == 14) {
                        continue;
                }

                /* Higher priority for boundary and middle channels.
                 * the offset 3000us may be changed in the future.
                 */
                if (priv_p->channelBand == CHANNEL_BAND_24 &&
                    (i == 0 || i == numOperationChannels - 1 ||
                     i == numOperationChannels / 2)) {
                        chnl_info_p->CCA_activeTime =
                                chnl_info_p->CCA_activeTime < 3000 ?
                                0 : chnl_info_p->CCA_activeTime - 3000;
                }

                noisyLevel = channel_noisyLevel(chnl_info_p);
                KDBG_LOG_MLME(KDBG_DEBUG,"NoisyLevel=%ld (nonERP=%ld,ERP=%ld,CCA=%ld)"
                        " in channel %d\n", noisyLevel, chnl_info_p->bss_nonERP,
                        chnl_info_p->bss_ERP, chnl_info_p->CCA_activeTime,
                        *(chnlNum_p + i));

                if (noisyLevel < lowestLevel ||
                    (noisyLevel == lowestLevel &&
                     chnl_info_p->rxChannelPower < lowestRxPower)) {
                        lowestLevel = noisyLevel;
                        lowestRxPower = chnl_info_p->rxChannelPower;
                        bestChannel = i;
                }
        }

        /* to free scanned BSS records */
        ap_timer_func((unsigned long)priv_p);

        priv_p->curChannel = *(chnlNum_p + bestChannel);
        KDBG_LOG_MLME(KDBG_DEBUG,"Select channel %d\n", priv_p->curChannel);
}

/*----------------------------------------------------------------*/
/*!
* \brief
*
* \param
* \return
*/
/*----------------------------------------------------------------*/
void MlmeScan_request(unsigned long data)
{
        i2220mgmt_t             *priv_p = (i2220mgmt_t *)data;
        UINT32                  tmp32;
        UINT8                   tmp8;
        int                     scan_over = TRUE;

        /* Here: AC0, TQ1 and Beacon should be stopped
         * if state is RUNNING. Now only startup is considered.
         */

        /* Record rxChannelPower and rxMdRdyCount(traffic) */
        if (priv_p->chnl_info_p) {
                BCIRead8(priv_p, 116, &tmp8, FALSE, 0); /* bbCr 116 */
                priv_p->chnl_info_p[priv_p->scan_index].rxChannelPower = tmp8;
                NIC_MCR_READ(priv_p, MCR_CATR, &tmp32);
                priv_p->chnl_info_p[priv_p->scan_index].CCA_activeTime = tmp32;
        }

        /* Check the next channel to scan */
        priv_p->scan_index++;
#if SUPPORT_802_11A
        if (priv_p->channelBand == CHANNEL_BAND_5) {
                if (priv_p->scan_index < priv_p->numOperatingChannels5) {
                        priv_p->curChannel =
                                priv_p->operatingChannels5[priv_p->scan_index];
                        scan_over = FALSE;
                }
        }
        else
#endif
        {
                if (priv_p->scan_index < priv_p->numOperatingChannels24) {
                        priv_p->curChannel =
                                priv_p->operatingChannels24[priv_p->scan_index];
                        scan_over = FALSE;
                }
        }

        if (scan_over) {
                /* scan over, select best channel to stay.
                 * the curChannel will be updated.
                 */
                MlmeScan_bestChannel(priv_p);

                if (priv_p->chnl_info_p) {
                        kal_free(priv_p->chnl_info_p);
                        priv_p->chnl_info_p = NULL;
                }
                i2220hw_drvr_start(priv_p);

                /* If we change channel RF channel and does not stop RX DMA,
                 * it's very likely that RX DMA will stop work. So we reset_mac
                 * to prevent this. ???
                 */
                i2220hw_reset_mac(priv_p);
                return;
        }

        hw_set_channel(priv_p, 0xff);
        set_scan_timer(priv_p);
}


