/*
** $Id: mgmt_timer.h 1.3 2003/12/19 15:37:24 chyeh Exp $
*/

/*! \file mgmt_timer.h
    \brief

        This file is similar with linux/timer.h in Linux OS.
*/

/******************************************************************************
Copyright (c) 2002-2003 Integrated Programmable Communications, Inc.

All rights reserved. Copying, compilation, modification, distribution
or any other use whatsoever of this material is strictly prohibited
except in accordance with a Software License Agreement with
Integrated Programmable Communications, Inc.
******************************************************************************/

#ifndef _MGMT_TIMER_H
#define _MGMT_TIMER_H
#include <linux/timer.h>
#include <linux/list.h>
/************************************************************************
*                     C O M P I L E R   F L A G S
*************************************************************************
*/

/************************************************************************
*                E X T E R N A L   R E F E R E N C E S
*************************************************************************
*/

/************************************************************************
*                          C O N S T A N T S
*************************************************************************
*/

/************************************************************************
*                         D A T A   T Y P E S
*************************************************************************
*/
typedef struct mgmt_timer_list {
	struct list_head list;
	unsigned long expires;
	unsigned long data;
	void (*function)(unsigned long);
} mgmt_timer_t;

typedef void (*mgmt_timeout_func)(unsigned long);

/************************************************************************
*                        P U B L I C   D A T A
*************************************************************************
*/

/************************************************************************
*                             M A C R O S
*************************************************************************
*/
#define mgmt_stop_timer(tmr)            mgmt_del_timer(tmr)
#define mgmt_renew_timer(tmr,interval)  (tmr)->expires = kal_jiffies + (interval)

/************************************************************************
*              F U N C T I O N   D E C L A R A T I O N S
*************************************************************************
*/
void mgmt_add_timer(struct mgmt_timer_list * timer);
int mgmt_del_timer(struct mgmt_timer_list * timer);
void mgmt_start_timer(mgmt_timer_t *tmr, unsigned long interval,
                        mgmt_timeout_func func, unsigned long data);

/************************************************************************
*           I N L I N E   F U N C T I O N   D E F I N I T I O N S
*************************************************************************
*/
__KAL_INLINE__ void mgmt_init_timer(struct mgmt_timer_list * timer)
{
	timer->list.next = timer->list.prev = NULL;
}

__KAL_INLINE__ int mgmt_timer_pending (const struct mgmt_timer_list * timer)
{
	return timer->list.next != NULL;
}

#endif /* _MGMT_TIMER_H */
