/*
** $Id: wlanbox.h 1.80 2004/08/24 10:32:59 whsu Exp $
*/

/*! \file wlanbox.h
    \brief This header file consists of the definitions for AP / STA wlan-boxes.
*/

/******************************************************************************
Copyright (c) 2002-2003 Integrated Programmable Communications, Inc.

All rights reserved. Copying, compilation, modification, distribution
or any other use whatsoever of this material is strictly prohibited
except in accordance with a Software License Agreement with
Integrated Programmable Communications, Inc.
******************************************************************************/

#ifndef _WLANBOX_H
#define _WLANBOX_H

/************************************************************************
*                     C O M P I L E R   F L A G S
*************************************************************************
*/

/************************************************************************
*                E X T E R N A L   R E F E R E N C E S
*************************************************************************
*/

/************************************************************************
*                          C O N S T A N T S
*************************************************************************
*/
#define SDL_TIMER_INTERVAL      (1*HZ)          /* in unit of HZ (jiffies) */ 
#define STA_MAX_INACTIVITY      (90*HZ)         /* in unit of HZ (jiffies) */
#define BRIDGE_PROCESSING_DELAY (1*HZ)          /* in unit of HZ (jiffies) */
#define COUNTER_READ_INTERVAL   (10*60*HZ)      /* in unit of HZ (jiffies) */
#define MLME_SCAN_INTERVAL      (HZ/5)          /* in unit of HZ (jiffies) */
#define WPA_CtrMsur_INTERVAL    (60*HZ)         /* in unit of HZ (jiffies) */
#define TX_TIMEOUT_INTERVAL     (1*HZ)          /* in unit of HZ (jiffies) */

#define STA_HASH_SZ     256
#define sta_hashfn(x)   ((x[4] ^ x[5]) & (STA_HASH_SZ - 1))
#define STA_MAGIC_ID    0x1234abcd

/* CMC venus: define return value of sdlu_defragment() 6/16 */
#define FRAG_STATUS_COMPLETE            0
#define FRAG_STATUS_BEGIN               1
#define FRAG_STATUS_DISCARD             2

/* Authentication state for a sta_p->auth_state */
#define ST_Auth_Rsp_Idle                0
#define ST_Wait_Chal_Rsp                1

////////////////////////////////////////////////////////////////////////
/* sta_info_t::flags, definitions from SDL PROCESS 'Power_Save_Monitor'
 * They are originally of 'MacAddrSet' sort.
 * -------------------------------------------------------------------
 * Each of these sets holds MAC addresses of stations with a particular
 * operational state. Stations are added to and removed from sets due to
 * MLME requests, received management frames, and bits in received MAC headers.
 * Sets are not aged, as there is no requirement for periodic activity,
 * but aging to expunge addresses of inactive stations is permitted.
 * -------------------------------------------------------------------
 */
#define STA_INFO_AWAKE          BIT(0)  /* detected in sta_active mode */
#define STA_INFO_ASLEEP         BIT(1)  /* detected in power_save mode */
#define STA_INFO_AUTH_OS        BIT(2)  /* authenticated by open system */
#define STA_INFO_AUTH_KEY       BIT(3)  /* authenticated by any other alg. */
#define STA_INFO_ASSOC          BIT(4)  /* associated (0|1 member, non-AP) */

/* whsu : add for ACL control, 1 for for accept, 0 for deny */
#define STA_INFO_ACL            BIT(6)  /* permanent; do not remove entry on expiration */

#define NEIGHBOR_NONERP         -100    /* incNonERP in sta_p indicates nonERP BSS */
                                        /* Note: this def must be negative */

/************************************************************************
*                         D A T A   T Y P E S
*************************************************************************
*/
typedef enum _NIC_OP_MODE {
        NIC_AP, NIC_STATION, TOTAL_NIC_OP_MODE
} nic_op_mode_t;

typedef enum _txRate_index {
        rate_index_1M, rate_index_2M, rate_index_5_5M, rate_index_11M,
        rate_index_6M, rate_index_9M, rate_index_12M, rate_index_18M,
        rate_index_24M, rate_index_36M, rate_index_48M, rate_index_54M
} txRate_index_t;

/* added by CMC venus 6/23 */
typedef struct txRate_info_s {
        UINT8           txRate;
        UINT32          txRateCode;
        UINT8           rspRate;
        UINT32          rspRateCode;
        UINT16          rspDuration;    /* CTS/ACK duration */
                                        /* long/short preamble is included */
        bool            isERP;          /* indicate it's ERP rate */
} txRate_info_t;


typedef struct ssid_info_s{
        UINT8           ssid[MAX_SSID_LEN];
        UINT8           ssid_len;

        UINT8           priority;
        UINT16          vlanid;
        UINT8           vlan_tag_enabled;
        UINT8           enabled;
        UINT8           directPskValue;
        /*We might need to add key information after here*/
        //UINT8           auth_mode;// refer to param_authMode_t
        //UINT8           cipher_mode;//refer to param_cipherMode_t
        //UINT8           WPA_PSK;
        //UINT8           dot11RSNEnabled;
        //UINT8           dot11Pure1XEnabled;
        UINT8           dot11RSNConfigPSKPassPhrase[64];
        UINT8           dot11RSNConfigPSKValue[32];
} ssid_info_t;

struct ipn_net_device_stats
{
        UINT32          rx_bytes;
        UINT32          tx_bytes;
};

/*------------------------------------------------------------------*/
struct ap_info_ {
        chain_list_t            sta_chain;       /* list of all STAs, assoc or not */
        sta_info_t              *sta_hash[STA_HASH_SZ];

        /* pointers to STA info; based on allocated AID or NULL if AID free
         * AID is in the range 1-2007, so sta_aid[0] corresponders to AID 1
         * and so on
         */
        sta_info_t              **sta_aid;

        UINT8                   tmap[(sMaxAId/8)+1];
        int                     tmap_xlo, tmap_xhi;

        PartialSdu_t            defrag_buf[SYN_defragSize];

        mgmt_timer_t            sdl_timer;
#if SUPPORT_MSSID
        ssid_info_t             ssid_list[MAX_SSID];
#endif
};

struct sta_info_ {
        /*-----------------2002/10/1 22:45------------------
         * administration properties
         * --------------------------------------------------*/
        sta_info_t              *hash_next;
        struct {
            struct list_head    link; /* Link to the next STA in the list */
        } sw;
#if WLAN_INCLUDE_PROC
        struct proc_dir_entry   *proc;
#endif
        /*--------------------------------------------------
         * Station status/capability under different situations
         * --------------------------------------------------*/
        UINT32                  magicID;        /* to judge if this record still exists */
        AsocData_t              asoc;           /* association information, ref. SDL */
        UINT32                  flags;          /* STA_INFO_xxx flags */
        INT16                   aid;            /* STA's unique AID (1 .. 2007) or 0 if not yet assigned */
                                                /* if (AID < 0) ==> remote WDS AP */
        UINT16                  seqCtrl;        /* last rx seq control */
        chain_list_t            psQ;
        UINT32                  challenge_xor;  /* XOR mask of challenge text */
        txRate_info_t           *curTxRate_p;
        UINT32                  weightedRetryCount;
        UINT32                  rateUpdateTime;
        UINT32                  actTime;        /* final tx/rx(activity) time */
        UINT8                   authMode;       /* Current auth mode */
        UINT8                   keydescripter;
        UINT8                   KeyMapOn;
        UINT8                   keyId;
        UINT8                   cipherSuite;    /* STA's cipher suite: */
        UINT8                   encryptKey[32]; /* i.e., the key, max 32 bytes */
        INT8                    incLngPreamble; /* (0,1): only long preamble */
        INT8                    incLngSlotTime; /* (0,1): only long slot time */
        int                     incNonERP;      /* (-100,0,1): NonERP BSS/STA */

#if RX_MIC_PARTIAL_CHECK
        UINT32                  micTick;
#endif
#if SUPPORT_MSSID
        UINT8                   whichSsid;
#endif
        /*--------------------------------------------------
         * Statistical properties
         * --------------------------------------------------*/
        UCHAR                   bbCr29;         /* average BB CR29 */

        /*--------------------------------------------------
         * Authentication FSM properties
         * --------------------------------------------------*/
        UINT8                   auth_state;     /* FSM for AuthRspService */
        UINT8                   rsnBlockingTraffic;

        UINT8                   MIC_error_count;
        UINT32                  MIC_error_time;

        DOT11_COUNTERS_ENTRY    sta_statistics; /* Station traffic information */

        UINT8                   rsnMode; /* 0=wpa, 1= rsn */
#if SUPPORT_IPNRSN
        eREQ_WPA_t              wpaIE_p[1];
        /* IEEE 802.1X related data */
        struct Author_state_machine     author_sm[1];
#endif

#if WLAN_INCLUDE_WDS
        UINT8                   role;           /*used to distinguish this station is a Station or Master. Refer to IW_MODE*/
        wdslink_info_t*         wds_link;       /*used to associate with WDS info object. one to one */
#endif
};

/************************************************************************
*                        P U B L I C   D A T A
*************************************************************************
*/

/************************************************************************
*                             M A C R O S
*************************************************************************
*/
#define TU2JIFFIES(t)           (((t)<<10) / (1000000 / HZ))

#define check_timeout(eol)  ((UINT32)kal_jiffies - (UINT32)(eol) <= 0x7fffffff)

/* Check the 60 sec timer for WPA countermeasure block active */
#define check_60_sec(time)      check_timeout((time)+60*HZ)

/************************************************************************
*       S D L    E N V I R O N M E N T A L    D E F I N I T I O N S
*************************************************************************
*/
/* Modified by CMChang on 6/11 */
#define sdl_tmap(priv,aid)    (priv->ap.tmap[aid/8] & BIT(aid%8))

#define sdl_set_tmap(priv,aid) do { \
        priv->ap.tmap[aid/8] |= BIT(aid%8); \
        if (priv->ap.tmap_xlo > aid || priv->ap.tmap_xlo == 0) \
                priv->ap.tmap_xlo = aid; \
        if (priv->ap.tmap_xhi < aid || priv->ap.tmap_xhi == 0) \
                priv->ap.tmap_xhi = aid; \
} while (0)

#define sdl_reset_tmap(priv,aid) do { \
        priv->ap.tmap[aid/8] &= ~BIT(aid%8); \
        if (priv->ap.tmap_xlo == priv->ap.tmap_xhi){ \
                if (priv->ap.tmap_xlo == aid) \
                        priv->ap.tmap_xlo = priv->ap.tmap_xhi = 0; \
        } \
        else if (priv->ap.tmap_xlo == aid) { int i=aid;\
                while (++i <= priv->ap.tmap_xhi) \
                        if (sdl_tmap(priv,i)) { priv->ap.tmap_xlo = i; break; } \
        } \
        else if (priv->ap.tmap_xhi == aid) { int i=aid; \
                while (--i >= priv->ap.tmap_xlo) \
                        if (sdl_tmap(priv,i)) { priv->ap.tmap_xhi = i; break; } \
        } \
} while (0)

/************************************************************************
*              F U N C T I O N   D E C L A R A T I O N S
*************************************************************************
*/

/************************************************************************
*           I N L I N E   F U N C T I O N   D E F I N I T I O N S
*************************************************************************
*/

#endif /* _WLANBOX_H */

