/*
** $Id: 11i-rsn.c 1.12 2004/10/04 17:49:36 cmchang Exp $
*/

/*!     \file 11i-rsn.c
        \brief IEEE 802.11i rsn standard SDL, Authenticator key management state machine
*/

/******************************************************************************
Copyright (c) 2000-2002 Integrated Programmable Communications, Inc.

All rights reserved. Copying, compilation, modification, distribution
or any other use whatsoever of this material is strictly prohibited
except in accordance with a Software License Agreement with
Integrated Programmable Communications, Inc.
******************************************************************************/

/************************************************************************
*                                E X T E R N A L   R E F E R E N C E S
*************************************************************************
*/

/* System Includes */
#include "wlan_os.h"
#include "precomp.h"
/************************************************************************
*                                                  C O N S T A N T S
*************************************************************************
*/

/************************************************************************
*                    D A T A    T Y P E S
*************************************************************************
*/

/************************************************************************
*                    P U B L I C   D A T A
*************************************************************************
*/


/************************************************************************
*                    P R I V A T E  D A T A
*************************************************************************
*/

/************************************************************************
*                    M A C R O S
*************************************************************************
*/

/************************************************************************
*                    F U N C T I O N       D E C L A R A T I O N S
*************************************************************************
*/
/************************************************************************
*                    F U N C T I O N S
*************************************************************************
*/

/************************************************************************
*                    F U N C T I O N       D E C L A R A T I O N S
*************************************************************************
*/

/************************************************************************
*                          F U N C T I O N S
*************************************************************************
*/

/*_________________________________________________________________________
* Function       : rsnGenWPAIE
* Input          : priv_p - Pointer to the adapter object data area.
*                  ieStartAddr_p, pointer to put the generated WPA IE
* Output:          expendedLen_p, pointer to put the length of WPA IE
* Return         : TRUE, if the RSN IE generation is uccessful
*                  FALSE, if the RSN IE is generation is failure
* Description:
**__________________________________________________________________________*/
//whsu say, this function orginal call when send beacon or probe response,
//          but it seems spend lots of time, and in 11i, when EAPOL key frame 3
//          we should send the same IE for every sta, so keep a copy in priv_p,
//          and do memcpy to beacon or probe response when need.
BOOLEAN
rsnGenWPAIE(i2220mgmt_t *priv_p)
{
        int             i;
        UINT16          uniSuiteCount = 0, authSuiteCount = 0;
        UINT8           *pairwiseKeyCipherSuite, *authKeyMgtSuite;
        UINT32          suite;
        eWPA_t          wpaIE_buf, *wpaIE_p = &wpaIE_buf;

        /* No WPA element ID */
        if (!mib_import(dot11RSNEnabled)) {
                cmd_set_RsnIE(priv_p->netdev, (UINT8*)wpaIE_p, 0);
                return FALSE;
        }

        for (i = 1; i < UINCAST_CIPHER_SUITE_COUNT; i++) {
                if (priv_p->mib.dot11RSNConfigUnicastCiphersTable[i].
                    dot11RSNConfigUnicastCipherEnabled) {
                        uniSuiteCount++;
                }
        }
        for (i = 1; i < AUTHENTICATION_SUITE_COUNT; i++) {
                if (priv_p->mib.dot11RSNConfigAuthenticationSuitesTable[i].
                    dot11RSNConfigAuthenticationSuiteEnabled) {
                        authSuiteCount++;
                }
        }

        /* Start to construct WPA IE */
        /* Element Length */
        wpaIE_p->length = (UCHAR)(16 + (uniSuiteCount + authSuiteCount) * 4);
        if (wpaIE_p->length > sizeof(eWPA_t) - 2) {
                KDBG_LOG_INIT(KDBG_WARN , "WPA buffer too small\n");
                cmd_set_RsnIE(priv_p->netdev, (UINT8*)wpaIE_p, 0);
                return FALSE;
        }

        /* Element ID */
        wpaIE_p->elemId = SYN_eWPA;

        /* WPA OUI */
        ipn_memcpy(&wpaIE_p->oui[0], IE_WPA_OUI, sizeof(IE_WPA_OUI));

        /* Version (version 1)*/
        wpaIE_p->version = host2ieee16(1);

        /* Group key (Do not use union) */
        wpaIE_p->groupKeyCipherSuite[0] = 0x00;
        wpaIE_p->groupKeyCipherSuite[1] = 0x50;
        wpaIE_p->groupKeyCipherSuite[2] = 0xf2;
        wpaIE_p->groupKeyCipherSuite[3] = priv_p->BCcipherSuite;

        /* Pairwise key cipher suite count */
        wpaIE_p->pairwiseKeyCipherSuiteCount = host2ieee16(uniSuiteCount);
        pairwiseKeyCipherSuite = (UINT8*)&wpaIE_p->pairwiseKeyCipherSuiteList[0];

        /* Pairwise key cipher suite list */
        for (i = 1; i < UINCAST_CIPHER_SUITE_COUNT; i++) {
                if (priv_p->mib.dot11RSNConfigUnicastCiphersTable[i].
                    dot11RSNConfigUnicastCipherEnabled) {
                        /* Do not use union */
                        suite = priv_p->mib.dot11RSNConfigUnicastCiphersTable[i].
                                        dot11RSNConfigUnicastCipher.u.suiteSelector;

                        pairwiseKeyCipherSuite[0] = (suite & 0x000000FF);
                        pairwiseKeyCipherSuite[1] = (suite & 0x0000FF00) >> 8;
                        pairwiseKeyCipherSuite[2] = (suite & 0x00FF0000) >> 16;
                        pairwiseKeyCipherSuite[3] = (suite & 0xFF000000) >> 24;

                        pairwiseKeyCipherSuite += 4;
                }
        }

        /* Authenticated key management suite count */
        pairwiseKeyCipherSuite[0] = (UINT8)(authSuiteCount);
        pairwiseKeyCipherSuite[1] = (UINT8)(authSuiteCount >> 8);
        authKeyMgtSuite = pairwiseKeyCipherSuite + 2;

        /* Authenticated key management suite list */
        for (i = 1; i < AUTHENTICATION_SUITE_COUNT; i++) {
                if (priv_p->mib.dot11RSNConfigAuthenticationSuitesTable[i].
                    dot11RSNConfigAuthenticationSuiteEnabled) {
                        /* Do not use union */
                        suite = priv_p->mib.dot11RSNConfigAuthenticationSuitesTable[i].
                                        dot11RSNConfigAuthenticationSuite.u.suiteSelector;

                        authKeyMgtSuite[0] = (suite & 0x000000FF);
                        authKeyMgtSuite[1] = (suite & 0x0000FF00) >> 8;
                        authKeyMgtSuite[2] = (suite & 0x00FF0000) >> 16;
                        authKeyMgtSuite[3] = (suite & 0xFF000000) >> 24;

                        authKeyMgtSuite += 4;
                }
        }
        /* RSN capability */
        authKeyMgtSuite[0] = 0;
        authKeyMgtSuite[1] = 0;

        cmd_set_RsnIE(priv_p->netdev, (UINT8*)wpaIE_p, wpaIE_p->length + 2);

        return TRUE;
}

/*_________________________________________________________________________
**  rsnParseWPAIE
**
**  descriptions: This routine is called to parse WPA IE.
**  parameters:
**      priv_p - Pointer to the Adapter structure
**      infoElem_p - Pointer to the starting address of WPA IE
**      remainIeLen - Remain total IE length
**      beaconFrame_p - Pointer to starting address of beacon frame
**      bssDescription_p - Pointer to store the BSSDescription copied from
**                         the WPA IE
**  return:         (none)
**  note:
**__________________________________________________________________________*/
BOOLEAN
rsnParseWPAIE(i2220mgmt_t *priv_p, sta_info_t *sta_p, eREQ_WPA_t* wpaie_p)
{
        UINT8   grpSuite[4] = {0x00, 0x00, 0x00, 0x00}; /* TKIP */
        UINT8   pairSuite[4]= {0x00, 0x00, 0x00, 0x00}; /* TKIP */
        UINT8   authSuite[4]= {0x00, 0x00, 0x00, 0x00}; /* WPA  */
        UINT32  suite;
        int     i;

        if (sta_p == NULL) return FALSE; /* Reject */

        if (wpaie_p->length >= 6) {
                ipn_memcpy(&grpSuite[0], &wpaie_p->groupKeyCipherSuite[0], 4);
        }
        if (wpaie_p->length >= 12) {
                ipn_memcpy(&pairSuite[0], &wpaie_p->pairwiseKeyCipherSuite[0], 4);
        }
        if (wpaie_p->length >= 18) {
                ipn_memcpy(&authSuite[0], &wpaie_p->authKeyMgtSuite[0], 4);
        }

        /* Check group cipher suite */
        if (grpSuite[0] != 0x00 ||
            grpSuite[1] != 0x50 ||
            grpSuite[2] != 0xf2 ||
            grpSuite[3] != priv_p->BCcipherSuite) {
                KDBG_LOG_SDL(KDBG_DEBUG,"Group cipherSuite not passed\n");
                //printk(KERN_ALERT "Group cipherSuite not passed %02x %02x %02x %02x \n",
                //        grpSuite[0], grpSuite[1], grpSuite[2], grpSuite[3] );
                return FALSE; /* Rejcet */
        }

        /* Check pairwise cipher suite */
        for (i = 0; i < UINCAST_CIPHER_SUITE_COUNT; i++) {
                if (priv_p->mib.dot11RSNConfigUnicastCiphersTable[i].
                    dot11RSNConfigUnicastCipherEnabled) {
                        suite = priv_p->mib.dot11RSNConfigUnicastCiphersTable[i].
                                        dot11RSNConfigUnicastCipher.u.suiteSelector;
                        if (pairSuite[0] == (suite & 0x000000FF) &&
                            pairSuite[1] == ((suite & 0x0000FF00) >> 8) &&
                            pairSuite[2] == ((suite & 0x00FF0000) >> 16) &&
                            pairSuite[3] == ((suite & 0xFF000000) >> 24)) {
                                break;
                        }
                }
        }
        if (i >= UINCAST_CIPHER_SUITE_COUNT) {
                KDBG_LOG_SDL(KDBG_DEBUG,"Unicast cipherSuite not passed\n");
                //printk(KERN_ALERT "Unicast cipherSuite not passed\n");
                return FALSE; /* Rejcet */
        }

        /* Check authentication suite */
        for (i = 0; i < AUTHENTICATION_SUITE_COUNT; i++) {
                if (priv_p->mib.dot11RSNConfigAuthenticationSuitesTable[i].\
                    dot11RSNConfigAuthenticationSuiteEnabled) {
                        suite = priv_p->mib.dot11RSNConfigAuthenticationSuitesTable[i].
                                        dot11RSNConfigAuthenticationSuite.u.suiteSelector;
                        if (authSuite[0] == (suite & 0x000000FF) &&
                            authSuite[1] == ((suite & 0x0000FF00) >> 8) &&
                            authSuite[2] == ((suite & 0x00FF0000) >> 16) &&
                            authSuite[3] == ((suite & 0xFF000000) >> 24)) {
                                break;
                        }
                }
        }
        if (i >= AUTHENTICATION_SUITE_COUNT) {
                KDBG_LOG_SDL(KDBG_DEBUG,"Unicast cipherSuite not passed\n");
                //printk(KERN_ALERT "Unicast cipherSuite not passed\n");
                return FALSE; /* Rejcet */
        }

        /* Update station cipher information. If the pairSuite[3] = 0 means
         * using the same cipher with group key, thus force it to BCcipherSuite
         */
        if ((sta_p->cipherSuite = pairSuite[3]) == 0) {
                sta_p->cipherSuite = priv_p->BCcipherSuite;
        }

        sta_p->keydescripter = (sta_p->cipherSuite == CIPHER_SUITE_CCMP) ? 2 : 1;

        return TRUE; /* Accept */
}


