/*
** $Id: wlan_kal.h 1.8 2004/09/29 11:54:14 cmchang Exp $
*/

/******************************************************************************
Copyright (c) 2002-2003 Inprocomm, Inc.

All rights reserved. Copying, compilation, modification, distribution
or any other use whatsoever of this material is strictly prohibited
except in accordance with a Software License Agreement with Inprocomm, Inc.
******************************************************************************/

#ifndef __WLAN_KAL_H
#define __WLAN_KAL_H

/************************************************************************
*                     C O M P I L E R   F L A G S
*************************************************************************
*/
/* New KAL interface must define IPN_NEW_KAL */
#define IPN_NEW_KAL

/* These modifier will be used in hardware and network related structure,
 * to make sure it's packed by one byte, without any byte alignment.
 */
#define __KAL_PACK_BEGIN__      /* _Pragma("pack(1)") */
#define __KAL_PACK_END__        /* _Pragma("pack()") */
#define __KAL_ATTRIB_PACKED__   __attribute__ ((__packed__))
#define __KAL_INLINE__          static __inline__

/************************************************************************
*                E X T E R N A L   R E F E R E N C E S
*************************************************************************
*/

/************************************************************************
*                          C O N S T A N T S
*************************************************************************
*/
#define MAX_WLAN_DEVICES        8    /* Just for Linux OS */

/* If the Ethernet backbone is considered, the RX_SKB_BUF_SZ must be
 * at least 1586 bytes (1544+20+20+2), which includes encryption(20),
 * driver reserved space(20=12 + 8 bubble) and VLAN Tag(2).
 */
#define RX_SKB_BUF_SZ           1600

/* HZ must be larger than 2 (500ms) */
#ifndef HZ
#define HZ                      100  /* 10ms, basic time unit */
#endif

/* Define directory arg to DMA mapping routines. i.e.,
 * kal_pci_map_single() and kal_pci_unmap_single()
 */
#ifndef PCI_DMA_BIDIRECTIONAL
#define PCI_DMA_BIDIRECTIONAL   0
#define PCI_DMA_TODEVICE        1
#define PCI_DMA_FROMDEVICE      2
#define PCI_DMA_NONE            3
#endif

/************************************************************************
*                         D A T A   T Y P E S
*************************************************************************
*/

/************************************************************************
*                        P U B L I C   D A T A
*************************************************************************
*/
#define kal_jiffies             kal_ticks()

/************************************************************************
*                             M A C R O S
*************************************************************************
*/

/************************************************************************
*              F U N C T I O N   D E C L A R A T I O N S
*************************************************************************
*/
/* Pointer to private data in kal_net_device_t */
void *kal_dev_priv(void *dev);
void kal_set_dev_macAddr(void *dev, unsigned char *macAddr);

void kal_netdev_rx(void *skb, void *dev);
int kal_auth_request(void *dev, unsigned char *macAddr);
int kal_assoc_request(void *dev, unsigned char *macAddr,
                unsigned char *infoElem, int infoLen);
void kal_disassoc_indicate(void *dev, unsigned char *macAddr);
void kal_tkip_micFailure(void *dev, unsigned char *macAddr);
void kal_indicate_wds_tx_up(void *dev);
void kal_indicate_wds_tx_down(void *dev);

unsigned long kal_pci_readl(unsigned long membase, int offset);
void kal_pci_writel(unsigned long _value, unsigned long membase, int offset);

/* Non-cache memory allocate/free for descriptors */
void *kal_pci_alloc_sz(void *pci_p, int sz, unsigned long *dma_p);
void kal_pci_free_sz(void *pci_p, int sz, void *m, unsigned long dma);
unsigned long kal_pci_map_single(void *pcidev, void *mem, int size, int dir);
void kal_pci_unmap_single(void *pcidev, unsigned long bus, int size, int dir);

/* Functions acting on socket buffers for packets */
void *kal_alloc_skb(int sz);
void kal_free_skb(void *skb);
unsigned char *kal_skb_data(void *skb);
unsigned int  kal_skb_len(void *skb);
unsigned char *kal_skb_put(void *skb, int len);
unsigned char *kal_skb_push(void *skb, int len);
unsigned char *kal_skb_pull(void *skb, int len);

/* Cache memory allocation/free. */
void *kal_malloc(int sz);
void kal_free(void *cp);

/* Timer related functions */
void kal_udelay(int t);
void kal_mdelay(int t);
unsigned long kal_ticks(void);

/************************************************************************
*           I N L I N E   F U N C T I O N   D E F I N I T I O N S
*************************************************************************
*/

#endif  /* __WLAN_KAL_H */

