/*
** $Id: ant.c 1.5 2004/10/04 18:08:14 cmchang Exp $
*/

/*************************************************************************
* Copyright (c) 2002 Inprocomm, Inc.
*
* All rights reserved. Copying, compilation, modification, distribution
* or any other use whatsoever of this material is strictly prohibited
* except in accordance with a Software License Agreement with
* Inprocomm, Inc.
**************************************************************************/
/*************************************************************************
GENERAL DESCRIPTION
********************
This file contains the antenna-related routines of Windows driver for
INPROCOMM 802.11 Wireless LAN Adapters.
*/

/************************************************************************
*                E X T E R N A L   R E F E R E N C E S
*************************************************************************
*/
#include "precomp.h"

/************************************************************************
*                          C O N S T A N T S
*************************************************************************
*/

/************************************************************************
*                         D A T A   T Y P E S
*************************************************************************
*/

/************************************************************************
*                        P U B L I C   D A T A
*************************************************************************
*/

/************************************************************************
*                       P R I V A T E   D A T A
*************************************************************************
*/
DEBUGFILE("ant.c");

/************************************************************************
*                             M A C R O S
*************************************************************************
*/

/************************************************************************
*              F U N C T I O N   D E C L A R A T I O N S
*************************************************************************
*/

/************************************************************************
*                          F U N C T I O N S
*************************************************************************
*/

/*_________________________________________________________________________
**  antInitAntennaList
**
**  descriptions: This routine is called to initialize the list of NIC
**                antennas in the MIB.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**      eepromAntennaSettings - The antenna settings read from the EEPROM
**      eepromPhyModeSetting - The PHY mode setting read from the EEPROM
**  return:         (none)
**  note:
**__________________________________________________________________________*/
VOID
antInitAntennaList(IN PMP_ADAPTER adapter_p,
#if SUPPORT_802_11A
                   IN UINT_16 eepromAntennaSettings,
                   IN UINT_16 eepromPhyModeSetting)
#else
                   IN UINT_16 eepromAntennaSettings)
#endif
{
    UINT numTxAntennas = 1;
    UINT numRxAntennas = 1;

    DEBUGFUNC("antInitAntennaList");

    switch (eepromAntennaSettings & EEPROM_TX_ANT_PRESENT_MASK) {
        case EEPROM_TX_ANT_0_PRESENT:
             INITLOG(("Tx antennas: 0\n"));
             adapter_p->primaryTxAntenna = 0;
             break;

        case EEPROM_TX_ANT_1_PRESENT:
             INITLOG(("Tx antennas: 1\n"));
             adapter_p->primaryTxAntenna = 1;
             break;

        case EEPROM_TX_ANT_0_PRESENT | EEPROM_TX_ANT_1_PRESENT:
             INITLOG(("Tx antennas: 0 & 1\n"));
             adapter_p->primaryTxAntenna = 0;
             numTxAntennas = 2;
             break;

        default:
             INITLOG(("Tx antennas: (none)\n"));
             adapter_p->primaryTxAntenna = 0;
    }

    switch (eepromAntennaSettings & EEPROM_RX_ANT_PRESENT_MASK) {
        case EEPROM_RX_ANT_0_PRESENT:
             INITLOG(("Rx antennas: 0\n"));
             adapter_p->primaryRxAntenna = 0;
             break;

        case EEPROM_RX_ANT_1_PRESENT:
             INITLOG(("Rx antennas: 1\n"));
             adapter_p->primaryRxAntenna = 1;
             break;

        case EEPROM_RX_ANT_0_PRESENT | EEPROM_RX_ANT_1_PRESENT:
             INITLOG(("Rx antennas: 0 & 1\n"));
#if SUPPORT_802_11A
             if ((eepromPhyModeSetting & EEPROM_PHY_MODE_MASK) ==
                 EEPROM_PHY_MODE_A_G) {
                 adapter_p->primaryRxAntenna = 1;
             } else
#endif
             {
                 adapter_p->primaryRxAntenna = 0;
             }
             numRxAntennas = 2;
             break;

        default:
             INITLOG(("Rx antennas: (none)\n"));
#if SUPPORT_802_11A
             if ((eepromPhyModeSetting & EEPROM_PHY_MODE_MASK) ==
                 EEPROM_PHY_MODE_A_G) {
                 adapter_p->primaryRxAntenna = 1;
             } else
#endif
             {
                 adapter_p->primaryRxAntenna = 0;
             }
    }

    adapter_p->mib.dot11AntennasListTable[MIB_ANTENNA_PRIMARY - 1].dot11SupportedTxAntenna =
        TRUE;

    if (numTxAntennas == 2) {
        adapter_p->mib.dot11AntennasListTable[MIB_ANTENNA_SECONDARY - 1].dot11SupportedTxAntenna =
            TRUE;
    } else {
        adapter_p->mib.dot11AntennasListTable[MIB_ANTENNA_SECONDARY - 1].dot11SupportedTxAntenna =
            FALSE;
    }

    adapter_p->mib.dot11CurrentTxAntenna = MIB_ANTENNA_PRIMARY;
    adapter_p->currentTxAntenna = adapter_p->primaryTxAntenna;

    adapter_p->mib.dot11AntennasListTable[MIB_ANTENNA_PRIMARY - 1].dot11SupportedRxAntenna =
        TRUE;
    adapter_p->mib.dot11AntennasListTable[MIB_ANTENNA_PRIMARY - 1].dot11DiversitySelectionRx =
        TRUE;

    if (numRxAntennas == 2) {
        adapter_p->mib.dot11AntennasListTable[MIB_ANTENNA_SECONDARY - 1].dot11SupportedRxAntenna =
            TRUE;
        adapter_p->mib.dot11AntennasListTable[MIB_ANTENNA_SECONDARY - 1].dot11DiversitySelectionRx =
            TRUE;
    } else {
        adapter_p->mib.dot11AntennasListTable[MIB_ANTENNA_SECONDARY - 1].dot11SupportedRxAntenna =
            FALSE;
        adapter_p->mib.dot11AntennasListTable[MIB_ANTENNA_SECONDARY - 1].dot11DiversitySelectionRx =
            FALSE;
    }

    if (adapter_p->mib.dot11DiversitySupport != MIB_DIVERSITY_NOT_SUPPORTED &&
        adapter_p->mib.dot11AntennasListTable[MIB_ANTENNA_PRIMARY - 1].dot11DiversitySelectionRx &&
        adapter_p->mib.dot11AntennasListTable[MIB_ANTENNA_SECONDARY - 1].dot11DiversitySelectionRx) {
        adapter_p->mib.dot11CurrentRxAntenna = MIB_ANTENNA_DIVERSITY;
    } else {
        adapter_p->mib.dot11CurrentRxAntenna = MIB_ANTENNA_PRIMARY;
    }

    adapter_p->mib.numAntennas = numRxAntennas;
}   /* antInitAntennaList */

/*_________________________________________________________________________
**  antSetRxAntennaMode
**
**  descriptions: This routine is called to set the Rx antenna mode to the
**                Baseband according to the MIB attributes.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**  return:         (none)
**  note:
**__________________________________________________________________________*/
VOID
antSetRxAntennaMode(IN PMP_ADAPTER adapter_p)
{
    UINT_32 rxAntennaMode;

    DEBUGFUNC("antSetRxAntennaMode");

    ASSERT(adapter_p->mib.dot11CurrentRxAntenna == MIB_ANTENNA_PRIMARY ||
        adapter_p->mib.dot11CurrentRxAntenna == MIB_ANTENNA_SECONDARY ||
        adapter_p->mib.dot11CurrentRxAntenna == MIB_ANTENNA_DIVERSITY);

    switch (adapter_p->mib.dot11CurrentRxAntenna) {
        case MIB_ANTENNA_PRIMARY:
             if (adapter_p->primaryRxAntenna == 0) {
                 DBGLOG(MGT, TRACE,
                     ("Set fixed Rx antenna mode: primary (0)\n"));
                 rxAntennaMode = NIC_ANT_MODE_FIXED_0;
             } else {
                 DBGLOG(MGT, TRACE,
                     ("Set fixed Rx antenna mode: primary (1)\n"));
                 rxAntennaMode = NIC_ANT_MODE_FIXED_1;
             }
             break;

        case MIB_ANTENNA_SECONDARY:
             if (adapter_p->primaryRxAntenna == 0) {
                 DBGLOG(MGT, TRACE,
                     ("Set fixed Rx antenna mode: secondary (1)\n"));
                 rxAntennaMode = NIC_ANT_MODE_FIXED_1;
             } else {
                 DBGLOG(MGT, TRACE,
                     ("Set fixed Rx antenna mode: secondary (0)\n"));
                 rxAntennaMode = NIC_ANT_MODE_FIXED_0;
             }
             break;

        default:
             DBGLOG(MGT, TRACE, ("Rx antenna diversity enabled\n"));
             rxAntennaMode = NIC_ANT_MODE_DIVERSITY;
    }

    phySetRxAntenna(adapter_p, rxAntennaMode);
}   /* antSetRxAntennaMode */

/*_________________________________________________________________________
**  antSetTxAntennaSelected
**
**  descriptions: This routine is called to set the antenna used for
**                transmitting.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**      newTxAntenna - The new antenna used for transmitting. The values of
**                     0, 1, and 0xffffffff indicate primary, secondary,
**                     and diversity, respectively.
**  return:
**      NDIS_STATUS_SUCCESS
**      NDIS_STATUS_INVALID_DATA
**  note:
**__________________________________________________________________________*/
NDIS_STATUS
antSetTxAntennaSelected(IN PMP_ADAPTER adapter_p,
                        IN UINT_32 newTxAntenna)
{
    DEBUGFUNC("antSetTxAntennaSelected");

    if (newTxAntenna == 0xffffffff) {
        DBGLOG(REQ, TRACE, ("Diversity not supported\n"));

        return NDIS_STATUS_INVALID_DATA;
    }

    if (newTxAntenna >= adapter_p->mib.numAntennas ||
        !adapter_p->mib.dot11AntennasListTable[newTxAntenna].dot11SupportedTxAntenna) {
        DBGLOG(REQ, TRACE, ("Unsupported Tx antenna %d\n", newTxAntenna));

        return NDIS_STATUS_INVALID_DATA;
    }

    if (newTxAntenna == MIB_ANTENNA_SECONDARY - 1) {
        adapter_p->mib.dot11CurrentTxAntenna = MIB_ANTENNA_SECONDARY;
        if (adapter_p->primaryTxAntenna == 1) {
            adapter_p->currentTxAntenna = 0;
        } else {
            adapter_p->currentTxAntenna = 1;
        }
    } else {
        adapter_p->mib.dot11CurrentTxAntenna = MIB_ANTENNA_PRIMARY;
        adapter_p->currentTxAntenna = adapter_p->primaryTxAntenna;
    }

    return NDIS_STATUS_SUCCESS;
}   /* antSetTxAntennaSelected */

/*_________________________________________________________________________
**  antSetRxAntennaSelected
**
**  descriptions: This routine is called to set the antenna used for
**                receiving.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**      newRxAntenna - The new antenna used for receiving. The values of 0,
**                     1, and 0xffffffff indicate primary, secondary, and
**                     diversity, respectively.
**  return:
**      NDIS_STATUS_SUCCESS
**      NDIS_STATUS_INVALID_DATA
**  note:
**__________________________________________________________________________*/
NDIS_STATUS
antSetRxAntennaSelected(IN PMP_ADAPTER adapter_p,
                        IN UINT_32 newRxAntenna)
{
    DEBUGFUNC("antSetRxAntennaSelected");

    if (newRxAntenna == 0xffffffff) {
        if (adapter_p->mib.dot11DiversitySupport ==
            MIB_DIVERSITY_NOT_SUPPORTED) {
            DBGLOG(REQ, TRACE, ("Diversity not supported\n"));
            return NDIS_STATUS_INVALID_DATA;
        }

        if (adapter_p->mib.numAntennas < 2) {
            DBGLOG(REQ, TRACE,
                ("Diversity not supported due to single antenna\n"));
            return NDIS_STATUS_INVALID_DATA;
        }

        if (!adapter_p->mib.dot11AntennasListTable[MIB_ANTENNA_PRIMARY - 1].dot11DiversitySelectionRx) {
            DBGLOG(REQ, TRACE, ("Primary antenna diversity not supported\n"));
            return NDIS_STATUS_INVALID_DATA;
        }

        if (!adapter_p->mib.dot11AntennasListTable[MIB_ANTENNA_SECONDARY - 1].dot11DiversitySelectionRx) {
            DBGLOG(REQ, TRACE, ("Secondary antenna diversity not supported\n"));
            return NDIS_STATUS_INVALID_DATA;
        }

        adapter_p->mib.dot11CurrentRxAntenna = MIB_ANTENNA_DIVERSITY;
    } else {
        if (newRxAntenna >= adapter_p->mib.numAntennas ||
            !adapter_p->mib.dot11AntennasListTable[newRxAntenna].dot11SupportedRxAntenna) {
            DBGLOG(REQ, TRACE, ("Unsupported Rx antenna %d\n", newRxAntenna));

            return NDIS_STATUS_INVALID_DATA;
        }

        if (newRxAntenna == MIB_ANTENNA_SECONDARY - 1) {
            adapter_p->mib.dot11CurrentRxAntenna = MIB_ANTENNA_SECONDARY;
        } else {
            adapter_p->mib.dot11CurrentRxAntenna = MIB_ANTENNA_PRIMARY;
        }
    }

    antSetRxAntennaMode(adapter_p);

    return NDIS_STATUS_SUCCESS;
}   /* antSetRxAntennaSelected */

