/*
** $Id: nic.c 1.11 2004/09/14 17:45:54 tplee Exp $
*/

/*************************************************************************
* Copyright (c) 2004 Inprocomm, Inc.
*
* All rights reserved. Copying, compilation, modification, distribution
* or any other use whatsoever of this material is strictly prohibited
* except in accordance with a Software License Agreement with
* Inprocomm, Inc.
**************************************************************************/
/*************************************************************************
GENERAL DESCRIPTION
********************
*/

/************************************************************************
*                E X T E R N A L   R E F E R E N C E S
*************************************************************************
*/
#include "precomp.h"


/************************************************************************
*                         D A T A   T Y P E S
*************************************************************************
*/
typedef struct _NIC_TX_POWER_GAIN_GROUP {
    UINT_16     validMask;
    UINT_8      eepromOffset0dBm;
    UINT_8      eepromOffset5dBm;
    UINT_8      eepromOffset10dBm;
    UINT        channelNumListSize;
    UINT_8      channelNumList[8];
} NIC_TX_POWER_GAIN_GROUP, *PNIC_TX_POWER_GAIN_GROUP;

typedef struct _NIC_RF_CAL_GROUP {
    UINT_16     validMask;
    UINT_8      channelBand;
    UINT        channelNumListSize;
    UINT_8      channelNumList[8];
} NIC_RF_CAL_GROUP, *PNIC_RF_CAL_GROUP;


/************************************************************************
*                        P U B L I C   D A T A
*************************************************************************
*/

/************************************************************************
*                       P R I V A T E   D A T A
*************************************************************************
*/
DEBUGFILE("nic.c");


extern NIC_RF_SPECIFIC_CONFIG rfConfigMaxim2825_2827;
#if SUPPORT_802_11A
extern NIC_RF_SPECIFIC_CONFIG rfConfigMaxim2829;
extern NIC_RF_SPECIFIC_CONFIG rfConfigTheta5024;
#endif
extern NIC_RF_SPECIFIC_CONFIG rfConfigRFMD2959;
extern NIC_RF_SPECIFIC_CONFIG rfConfigAiroha2230;
extern NIC_RF_SPECIFIC_CONFIG rfConfigUBEC2451;

const PNIC_RF_SPECIFIC_CONFIG phySupportedRFConfig[] = {
    &rfConfigMaxim2825_2827,
#if SUPPORT_802_11A
    &rfConfigMaxim2829,
#endif
    &rfConfigRFMD2959,
    &rfConfigAiroha2230,
    &rfConfigUBEC2451
};


#if SUPPORT_802_11A
static UINT_8 nicChannelNumMappingTable5[] = {
    8, 12, 16,
    34, 36, 38, 40, 42, 44, 46, 48,
    52, 56, 60, 64,
    100, 104, 108, 112, 116,
    120, 124, 128, 132, 136, 140,
    149, 153, 157, 161,
    240, 244, 248, 252
};

static NIC_TX_POWER_GAIN_GROUP nicTxPowerGainGroup5[] = {
    {
        EEPROM_TX_PWR_MARK_5G_1_VALID_MASK,
        EEPROM_TX_PWR_5G_GROUP_1_0,
        EEPROM_TX_PWR_5G_GROUP_1_5,
        EEPROM_TX_PWR_5G_GROUP_1_10,
        3, { 8, 12, 16, 0, 0, 0, 0, 0 }
    },
    {
        EEPROM_TX_PWR_MARK_5G_2_VALID_MASK,
        EEPROM_TX_PWR_5G_GROUP_2_0,
        EEPROM_TX_PWR_5G_GROUP_2_5,
        EEPROM_TX_PWR_5G_GROUP_2_10,
        8, { 34, 36, 38, 40, 42, 44, 46, 48 }
    },
    {
        EEPROM_TX_PWR_MARK_5G_3_VALID_MASK,
        EEPROM_TX_PWR_5G_GROUP_3_0,
        EEPROM_TX_PWR_5G_GROUP_3_5,
        EEPROM_TX_PWR_5G_GROUP_3_10,
        4, { 52, 56, 60, 64, 0, 0, 0, 0 }
    },
    {
        EEPROM_TX_PWR_MARK_5G_4_VALID_MASK,
        EEPROM_TX_PWR_5G_GROUP_4_0,
        EEPROM_TX_PWR_5G_GROUP_4_5,
        EEPROM_TX_PWR_5G_GROUP_4_10,
        5, { 100, 104, 108, 112, 116, 0, 0, 0 }
    },
    {
        EEPROM_TX_PWR_MARK_5G_5_VALID_MASK,
        EEPROM_TX_PWR_5G_GROUP_5_0,
        EEPROM_TX_PWR_5G_GROUP_5_5,
        EEPROM_TX_PWR_5G_GROUP_5_10,
        6, { 120, 124, 128, 132, 136, 140, 0, 0 }
    },
    {
        EEPROM_TX_PWR_MARK_5G_6_VALID_MASK,
        EEPROM_TX_PWR_5G_GROUP_6_0,
        EEPROM_TX_PWR_5G_GROUP_6_5,
        EEPROM_TX_PWR_5G_GROUP_6_10,
        4, { 149, 153, 157, 161, 0, 0, 0, 0 }
    },
    {
        EEPROM_TX_PWR_MARK_4_9G_VALID_MASK,
        EEPROM_TX_PWR_4_9G_0,
        EEPROM_TX_PWR_4_9G_5,
        EEPROM_TX_PWR_4_9G_10,
        4, { 240, 244, 248, 252, 0, 0, 0, 0 }
    }
};
#endif

static NIC_RF_CAL_GROUP nicRFCalGroup[] = {
    {
        EEPROM_RF_CAL_2_4G_GROUP_1_VALID_MASK,
        CHANNEL_BAND_24,
        3, { 1, 2, 3, 0, 0, 0, 0, 0 }
    },
    {
        EEPROM_RF_CAL_2_4G_GROUP_2_VALID_MASK,
        CHANNEL_BAND_24,
        5, { 4, 5, 6, 7, 8, 0, 0, 0 }
    },
    {
        EEPROM_RF_CAL_2_4G_GROUP_3_VALID_MASK,
        CHANNEL_BAND_24,
        6, { 9, 10, 11, 12, 13, 14, 0, 0 }
    }
#if SUPPORT_802_11A
    ,
    {
        EEPROM_RF_CAL_4_9G_GROUP_VALID_MASK,
        CHANNEL_BAND_5,
        4, { 240, 244, 248, 252, 0, 0, 0, 0 }
    },
    {
        EEPROM_RF_CAL_5G_GROUP_1_VALID_MASK,
        CHANNEL_BAND_5,
        3, { 8, 12, 16, 0, 0, 0, 0, 0 }
    },
    {
        EEPROM_RF_CAL_5G_GROUP_2_VALID_MASK,
        CHANNEL_BAND_5,
        8, { 34, 36, 38, 40, 42, 44, 46, 48 }
    },
    {
        EEPROM_RF_CAL_5G_GROUP_3_VALID_MASK,
        CHANNEL_BAND_5,
        4, { 52, 56, 60, 64, 0, 0, 0, 0 }
    },
    {
        EEPROM_RF_CAL_5G_GROUP_4_VALID_MASK,
        CHANNEL_BAND_5,
        5, { 100, 104, 108, 112, 116, 0, 0, 0 }
    },
    {
        EEPROM_RF_CAL_5G_GROUP_5_VALID_MASK,
        CHANNEL_BAND_5,
        6, { 120, 124, 128, 132, 136, 140, 0, 0 }
    },
    {
        EEPROM_RF_CAL_5G_GROUP_6_VALID_MASK,
        CHANNEL_BAND_5,
        4, { 149, 153, 157, 161, 0, 0, 0, 0 }
    }
#endif
};


/************************************************************************
*                             M A C R O S
*************************************************************************
*/

/************************************************************************
*              F U N C T I O N   D E C L A R A T I O N S
*************************************************************************
*/

/************************************************************************
*                          F U N C T I O N S
*************************************************************************
*/

/*_________________________________________________________________________
**  nicSetupRFSpecificConfig
**
**  descriptions: This routine is called to set up the RF-specific
**                configuration table.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**      eepromPhyModeSetting - The PHY mode setting read from the EEPROM
**  return:       (none)
**  note:
**__________________________________________________________________________*/
VOID
#if SUPPORT_802_11A
nicSetupRFSpecificConfig(IN PMP_ADAPTER adapter_p,
                         IN UINT_16 eepromPhyModeSetting)
#else
nicSetupRFSpecificConfig(IN PMP_ADAPTER adapter_p)
#endif
{
    UINT i;
    UINT_16 txdacState;

    DEBUGFUNC("nicSetupRFSpecificConfig");

    for (i = 0; i < sizeof(phySupportedRFConfig) /
        sizeof(PNIC_RF_SPECIFIC_CONFIG); i++) {
        if (phySupportedRFConfig[i]->rfCtrlMode != adapter_p->rfCtrlMode) {
            continue;
        }

        adapter_p->rfSpecificConfig_p = phySupportedRFConfig[i];

        DBGLOG(INIT, TRACE, ("Set up RF-specific config table for %s\n",
            phySupportedRFConfig[i]->rfName_p));

#if SUPPORT_802_11A
        switch (eepromPhyModeSetting & EEPROM_PHY_MODE_MASK) {
            case EEPROM_PHY_MODE_A:
                 adapter_p->rfSpecificConfig_p->phyRFChannelTable24_p = NULL;
                 adapter_p->rfSpecificConfig_p->phyRFChannelTableLen24 = 0;
                 break;

            case EEPROM_PHY_MODE_G:
                 adapter_p->rfSpecificConfig_p->phyRFChannelTable5_p = NULL;
                 adapter_p->rfSpecificConfig_p->phyRFChannelTableLen5 = 0;
                 break;
        }
#endif

        break;
    }

#if WORKAROUND_TXDAC_POWER_CONSUMPTION
    txdacState = ReadEEProm16(adapter_p, EEPROM_TXDAC_STATE);
    if (txdacState & BIT(0)) {
        adapter_p->rfSpecificConfig_p->txDACon = !adapter_p->rfSpecificConfig_p->txDACon;
    }

#endif
}   /* nicSetupRFSpecificConfig */

#if SUPPORT_802_11A
/*_________________________________________________________________________
**  nicGetPerChannelConfigIndex5
**
**  descriptions: This routine is called to get the index of the 5 GHz band
**                per-channel configuration table by the channel number.
**  parameters:
**  return:
**      TRUE - Get the index successfully.
**      FALSE - Failed to the index.
**  note:
**__________________________________________________________________________*/
BOOLEAN
nicGetPerChannelConfigIndex5(IN UINT_8 channelNum,
                             OUT PUINT index_p)
{
    UINT i;

    DEBUGFUNC("nicGetPerChannelConfigIndex5");

    for (i = 0; i < sizeof(nicChannelNumMappingTable5); i++) {
        if (channelNum != nicChannelNumMappingTable5[i]) {
            continue;
        }

        *index_p = i;
        return TRUE;
    }

    return FALSE;
}   /* nicGetPerChannelConfigIndex5 */
#endif

/*_________________________________________________________________________
**  nicComputeTxPowerGain
**
**  descriptions: This routine is called to compute the transmit power
**                gain for the given transmit power, in units of dBm.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**      gainEntry_p - Pointer to the entry in the tx power gain table for
**                    the calculation of tx power gain
**      txPower - The given tx power, in dBm
**  result:
**      txPowerGain_p - Pointer to the value of the tx power gain
**                      corresponding to the given tx power
**  return:
**      TRUE - The corresponding tx power gain for the given tx power is
**             available.
**      FALSE - The corresponding tx power gain for the given tx power is
**              not available.
**  note:
**__________________________________________________________________________*/
BOOLEAN
nicComputeTxPowerGain(IN PMP_ADAPTER adapter_p,
                      IN PTX_POWER_GAIN gainEntry_p,
                      IN UINT_8 txPower,
                      OUT PUINT_8 txPowerGain_p)
{
    INT_32 tmp32;

    if (adapter_p->rfSpecificConfig_p == NULL) {
        ERRORLOG(("Unsupported RF Type\n"));
        return FALSE;
    }

    if (adapter_p->rfSpecificConfig_p->txPowerGainStepNum == 0 ||
        adapter_p->rfSpecificConfig_p->txPowerGainStepDenom == 0) {
        WARNLOG(("Cannot compute Tx power gain for RF: %s\n",
            adapter_p->rfSpecificConfig_p->rfName_p));
        return FALSE;
    }

    if (!gainEntry_p->tpcEnabled) {
        WARNLOG(("Cannot compute Tx power gain for TPC disabled\n"));
        return FALSE;
    }

    if (txPower > gainEntry_p->maxTxPower_dBm) {
        return FALSE;
    }

    if (txPower == 10) {
        *txPowerGain_p = gainEntry_p->gain10dBm;
    } else {
        tmp32 = (INT_32) txPower - 10;

        tmp32 = gainEntry_p->gain10dBm + (tmp32 *
            adapter_p->rfSpecificConfig_p->txPowerGainStepDenom) /
            adapter_p->rfSpecificConfig_p->txPowerGainStepNum;

        if (tmp32 > 255) {
            tmp32 = 255;
        } else if (tmp32 < 0) {
            tmp32 = 0;
        }

        *txPowerGain_p = (UINT_8) tmp32;
    }

    return TRUE;
}   /* nicComputeTxPowerGain */

/*_________________________________________________________________________
**  nicComputeTxPower
**
**  descriptions: This routine is called to compute the transmit power, in
**                dBm, for the given transmit power gain.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**      gainEntry_p - Pointer to the entry in the tx power gain table for
**                    the calculation of tx power
**      txPowerGain - The given tx power gain
**  result:
**      txPower_p - Pointer to the value of the tx power, in dBm,
**                  corresponding to the given tx power
**  return:
**      TRUE - The corresponding tx power for the given tx power gain is
**             available.
**      FALSE - The corresponding tx power for the given tx power gain is
**              not available.
**  note:
**__________________________________________________________________________*/
BOOLEAN
nicComputeTxPower(IN PMP_ADAPTER adapter_p,
                  IN PTX_POWER_GAIN gainEntry_p,
                  IN UINT_8 txPowerGain,
                  OUT PUINT_8 txPower_p)
{
    INT_32 tmp32;

    if (adapter_p->rfSpecificConfig_p == NULL) {
        ERRORLOG(("Unsupported RF Type\n"));
        return FALSE;
    }

    if (adapter_p->rfSpecificConfig_p->txPowerGainStepNum == 0 ||
        adapter_p->rfSpecificConfig_p->txPowerGainStepDenom == 0) {
        WARNLOG(("Cannot compute Tx power for RF: %s\n",
            adapter_p->rfSpecificConfig_p->rfName_p));
        return FALSE;
    }

    if (!gainEntry_p->tpcEnabled) {
        WARNLOG(("Cannot compute Tx power for TPC disabled\n"));
        return FALSE;
    }

    if (txPowerGain == gainEntry_p->gain10dBm) {
        *txPower_p = 10;
    } else {
        tmp32 = (INT_32) txPowerGain - (INT_32) gainEntry_p->gain10dBm;

        tmp32 = 10 + (tmp32 *
            adapter_p->rfSpecificConfig_p->txPowerGainStepNum) /
            adapter_p->rfSpecificConfig_p->txPowerGainStepDenom;

        if (tmp32 > 255) {
            tmp32 = 255;
        } else if (tmp32 < 0) {
            tmp32 = 0;
        }

        *txPower_p = (UINT_8) tmp32;
    }

    return TRUE;
}   /* nicComputeTxPower */

/*_________________________________________________________________________
**  nicReadMaxTxPowerGain
**
**  descriptions: This routine will read the maximum Tx power gain values
**                stored in the EEPROM.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**  return:
**  note:
**__________________________________________________________________________*/
VOID
nicReadMaxTxPowerGain(IN PMP_ADAPTER adapter_p,
                      IN UINT_8 eepromStartingOffset,
                      IN UINT_8 eepromEndOffset,
#if SUPPORT_802_11A
                      IN UINT_8 channelBand,
#endif
                      IN BOOLEAN forCCK)
{
    UINT i, j, k;
    UINT_16 eepromWordValue;
    UINT_8 tmp8;

    j = 0;

    for (i = eepromStartingOffset; i <= eepromEndOffset; i++) {
        eepromWordValue = ReadEEProm16(adapter_p, (UINT_8) i);

        for (k = 0; k < 2; k++) {
            tmp8 = (UINT_8) (eepromWordValue & 0x00FF);

            if (tmp8 != 0 && tmp8 != 0xFF) {
#if SUPPORT_802_11A
                if (channelBand == CHANNEL_BAND_5) {
                    adapter_p->nicPerChannelConfig5[j].txPowerGainOFDM.gainMax =
                        tmp8;
                } else
#endif
                {
                    if (forCCK) {
                        adapter_p->nicPerChannelConfig24[j].txPowerGainCCK.gainMax =
                            tmp8;
                    } else {
                        adapter_p->nicPerChannelConfig24[j].txPowerGainOFDM.gainMax =
                            tmp8;
                    }
                }
            }

            eepromWordValue >>= 8;
            j++;
        }
    }
}   /* nicReadMaxTxPowerGain */

/*_________________________________________________________________________
**  nicReadTxPowerGain
**
**  descriptions: This routine will read the TX power setting stored in
**                EEPROM.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**  return:
**  note:
**__________________________________________________________________________*/
VOID
nicReadTxPowerGain(IN PMP_ADAPTER adapter_p,
                   IN UINT_8 eepromOffset0dBm,
                   IN UINT_8 eepromOffset5dBm,
                   IN UINT_8 eepromOffset10dBm,
                   OUT PUINT_8 gain0dBm_p,
                   OUT PUINT_8 gain5dBm_p,
                   OUT PUINT_8 gain10dBm_p)
{
    UINT_16 tmp16;

    tmp16 = ReadEEProm16(adapter_p, eepromOffset0dBm);

    if (eepromOffset0dBm == eepromOffset5dBm) {
        *gain0dBm_p = (UINT_8) (tmp16 & 0x00FF);
        *gain5dBm_p = (UINT_8) (tmp16 >> 8);

        tmp16 = ReadEEProm16(adapter_p, eepromOffset10dBm);

        *gain10dBm_p = (UINT_8) (tmp16 & 0x00FF);
    } else {
        *gain0dBm_p = (UINT_8) (tmp16 >> 8);

        ASSERT(eepromOffset5dBm == eepromOffset10dBm);

        tmp16 = ReadEEProm16(adapter_p, eepromOffset5dBm);

        *gain5dBm_p = (UINT_8) (tmp16 & 0x00FF);
        *gain10dBm_p = (UINT_8) (tmp16 >> 8);
    }
}   /* nicReadTxPowerGain */

/*_________________________________________________________________________
**  nicSetupTxPowerCtrlInfo
**
**  descriptions: This routine is called to set up the Tx power control
**                (TPC) information.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**  return:
**  note:
**__________________________________________________________________________*/
VOID
nicSetupTxPowerCtrlInfo(IN PMP_ADAPTER adapter_p,
                        IN PTX_POWER_GAIN gainEntry_p,
                        IN UINT_8 gain0dBm,
                        IN UINT_8 gain5dBm,
                        IN UINT_8 gain10dBm)
{
    gainEntry_p->gain0dBm = gain0dBm;
    gainEntry_p->gain5dBm = gain5dBm;
    gainEntry_p->gain10dBm = gain10dBm;

    if (gainEntry_p->gainMax) {
        if (gain10dBm) {
            gainEntry_p->tpcEnabled = TRUE;

            nicComputeTxPower(adapter_p, gainEntry_p, gainEntry_p->gainMax,
                &gainEntry_p->maxTxPower_dBm);
        } else {
            gainEntry_p->maxTxPower_dBm = 0xFF;
        }
    }
}   /* nicSetupTxPowerCtrlInfo */

/*_________________________________________________________________________
**  nicReadTxPowerSetting
**
**  descriptions: This routine is called to read the TX power setting
**                stored in the EEPROM.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**  return:
**  note:
**__________________________________________________________________________*/
VOID
nicReadTxPowerSetting(IN PMP_ADAPTER adapter_p)
{
    UINT i;
    UINT_16 eepromWordValue;
    UINT_16 flags;
    UINT_8 gain0, gain5, gain10;
    UINT_8 tmp8;
#if DBG
    PTX_POWER_GAIN gainEntry_p;
#endif
#if SUPPORT_802_11A
    PNIC_TX_POWER_GAIN_GROUP group_p;
    UINT j, k;
#endif

    DEBUGFUNC("nicReadTxPowerSetting");


    ASSERT(2 * (EEPROM_2G_CCK_TXPWR_GAIN_END - EEPROM_2G_CCK_TXPWR_GAIN_START +
        1) <= MAX_NUM_2G_RF_CHANNELS);

#if SUPPORT_802_11A
    nicReadMaxTxPowerGain(adapter_p, EEPROM_2G_CCK_TXPWR_GAIN_START,
        EEPROM_2G_CCK_TXPWR_GAIN_END, CHANNEL_BAND_24, TRUE);
#else
    nicReadMaxTxPowerGain(adapter_p, EEPROM_2G_CCK_TXPWR_GAIN_START,
        EEPROM_2G_CCK_TXPWR_GAIN_END, TRUE);
#endif

    ASSERT(2 * (EEPROM_2G_OFDM_TXPWR_GAIN_END -
        EEPROM_2G_OFDM_TXPWR_GAIN_START + 1) <= MAX_NUM_2G_RF_CHANNELS);

#if SUPPORT_802_11A
    nicReadMaxTxPowerGain(adapter_p, EEPROM_2G_OFDM_TXPWR_GAIN_START,
        EEPROM_2G_OFDM_TXPWR_GAIN_END, CHANNEL_BAND_24, FALSE);
#else
    nicReadMaxTxPowerGain(adapter_p, EEPROM_2G_OFDM_TXPWR_GAIN_START,
        EEPROM_2G_OFDM_TXPWR_GAIN_END, FALSE);
#endif

#if SUPPORT_802_11A
    ASSERT(2 * (EEPROM_5G_OFDM_TXPWR_GAIN_END -
        EEPROM_5G_OFDM_TXPWR_GAIN_START + 1) <= MAX_NUM_5G_RF_CHANNELS);

    nicReadMaxTxPowerGain(adapter_p, EEPROM_5G_OFDM_TXPWR_GAIN_START,
        EEPROM_5G_OFDM_TXPWR_GAIN_END, CHANNEL_BAND_5, FALSE);
#endif

    tmp8 = 0;

    for (i = EEPROM_TX_PWR_MARK_START; i <= EEPROM_TX_PWR_MARK_END; i++) {
        eepromWordValue = ReadEEProm16(adapter_p, (UINT_8) i);

        tmp8 += (UINT_8) (eepromWordValue & 0x00FF);
        tmp8 += (UINT_8) (eepromWordValue >> 8);
    }

    if (tmp8 == 0xFF) {
        INITLOG(("0/5/10-dBm tx power gain table present\n"));

        flags = ReadEEProm16(adapter_p, EEPROM_TX_PWR_MARK_VALID_2_4G);

        if (flags & EEPROM_TX_PWR_MARK_2_4G_CCK_VALID_MASK) {
            INITLOG(("2.4 GHz CCK tx power gain table: Valid\n"));

            nicReadTxPowerGain(adapter_p, EEPROM_TX_PWR_2_4G_CCK_0,
                EEPROM_TX_PWR_2_4G_CCK_5, EEPROM_TX_PWR_2_4G_CCK_10,
                &gain0, &gain5, &gain10);

            for (i = 0; i < MAX_NUM_2G_RF_CHANNELS; i++) {
                nicSetupTxPowerCtrlInfo(adapter_p,
                    &adapter_p->nicPerChannelConfig24[i].txPowerGainCCK,
                    gain0, gain5, gain10);
            }
        }
#if DBG
        else {
            INITLOG(("2.4 GHz CCK tx power gain table: Invalid\n"));
        }
#endif

        if (flags & EEPROM_TX_PWR_MARK_2_4G_OFDM_VALID_MASK) {
            INITLOG(("2.4 GHz OFDM tx power gain table: Valid\n"));

            nicReadTxPowerGain(adapter_p, EEPROM_TX_PWR_2_4G_OFDM_0,
                EEPROM_TX_PWR_2_4G_OFDM_5, EEPROM_TX_PWR_2_4G_OFDM_10,
                &gain0, &gain5, &gain10);

            for (i = 0; i < MAX_NUM_2G_RF_CHANNELS; i++) {
                nicSetupTxPowerCtrlInfo(adapter_p,
                    &adapter_p->nicPerChannelConfig24[i].txPowerGainOFDM,
                    gain0, gain5, gain10);
            }
        }
#if DBG
        else {
            INITLOG(("2.4 GHz OFDM tx power gain table: Invalid\n"));
        }
#endif

#if SUPPORT_802_11A
        flags = ReadEEProm16(adapter_p, EEPROM_TX_PWR_MARK_VALID_5G);

        for (i = 0; i < sizeof(nicTxPowerGainGroup5) /
            sizeof(NIC_TX_POWER_GAIN_GROUP); i++) {
            group_p = &nicTxPowerGainGroup5[i];

            if (!(flags & group_p->validMask)) {
                INITLOG(("5 GHz group %d tx power gain table: Invalid\n",
                    i + 1));
                continue;
            }

            INITLOG(("5 GHz group %d tx power gain table: Valid\n", i + 1));

            nicReadTxPowerGain(adapter_p, group_p->eepromOffset0dBm,
                group_p->eepromOffset5dBm, group_p->eepromOffset10dBm,
                &gain0, &gain5, &gain10);

            for (j = 0; j < group_p->channelNumListSize; j++) {
                if (!nicGetPerChannelConfigIndex5(group_p->channelNumList[j],
                    &k)) {
                    continue;
                }

                nicSetupTxPowerCtrlInfo(adapter_p,
                    &adapter_p->nicPerChannelConfig5[k].txPowerGainOFDM,
                    gain0, gain5, gain10);
            }
        }
#endif
    }
#if DBG
    else {
        WARNLOG(("0/5/10-dBm tx power gain table not present or checksum error\n"));
    }
#endif

#if DBG
    DBGLOG(INIT, INFO, ("Tx power gain table for 2.4 GHz:\n"));
    for (i = 0; i < MAX_NUM_2G_RF_CHANNELS; i++) {
        gainEntry_p = &adapter_p->nicPerChannelConfig24[i].txPowerGainCCK;

        DBGLOG(INIT, INFO,
            ("CH%d CCK: max=0x%02x(%d dBm), 10dBm=0x%02x, 5dBm=0x%02x, 0dBm=0x%02x \n",
            i + 1, gainEntry_p->gainMax, gainEntry_p->maxTxPower_dBm,
            gainEntry_p->gain10dBm, gainEntry_p->gain5dBm,
            gainEntry_p->gain0dBm));

        gainEntry_p = &adapter_p->nicPerChannelConfig24[i].txPowerGainOFDM;

        DBGLOG(INIT, INFO,
            ("CH%d OFDM: max=0x%02x(%d dBm), 10dBm=0x%02x, 5dBm=0x%02x, 0dBm=0x%02x\n",
            i + 1, gainEntry_p->gainMax, gainEntry_p->maxTxPower_dBm,
            gainEntry_p->gain10dBm, gainEntry_p->gain5dBm,
            gainEntry_p->gain0dBm));
    }

#if SUPPORT_802_11A
    DBGLOG(INIT, INFO, ("Tx power gain table for 5 GHz:\n"));
    for (i = 0; i < MAX_NUM_5G_RF_CHANNELS; i++) {
        gainEntry_p = &adapter_p->nicPerChannelConfig5[i].txPowerGainOFDM;

        DBGLOG(INIT, INFO,
            ("[%d] CH%d: max=0x%02x(%d dBm), 10dBm=0x%02x, 5dBm=0x%02x, 0dBm=0x%02x\n",
            i, nicChannelNumMappingTable5[i], gainEntry_p->gainMax,
            gainEntry_p->maxTxPower_dBm, gainEntry_p->gain10dBm,
            gainEntry_p->gain5dBm, gainEntry_p->gain0dBm));
    }
#endif
#endif
}   /* nicReadTxPowerSetting */

/*_________________________________________________________________________
**  nicSetTxPowerGain
**
**  descriptions: This routine is called to set the transmit power gain.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**      configEntry_p - Pointer to the per-channel configuration entry
**      channelBand - The current channel band
**      txPowerLimit - The tx power limit, in dBm. The value of 0xFF
**                     indicates the maximum tx power of the NIC is used.
**  return:         (none)
**  note:
**__________________________________________________________________________*/
VOID
nicSetTxPowerGain(IN PMP_ADAPTER adapter_p,
                  IN PNIC_PER_CHANNEL_CONFIG configEntry_p,
#if SUPPORT_802_11A
                  IN UINT_8 channelBand,
#endif
                  IN UINT_8 txPowerLimit)
{
    UINT_8 gainCCK, gainOFDM;
    UINT_8 maxGainCCK, maxGainOFDM;
    UINT_8 maxTxPower_dBm = 0xFF;
    BOOLEAN gainCCKValid, gainOFDMValid;
    BOOLEAN useMaxTxPower = TRUE;
    UINT_32 temp32;

    DEBUGFUNC("nicSetTxPowerGain");

    if (adapter_p->rfSpecificConfig_p == NULL) {
        ERRORLOG(("Unsupported RF Type\n"));
        return;
    }

    maxGainCCK = adapter_p->rfSpecificConfig_p->txPowerGainCCK;

#if SUPPORT_802_11A
    if (channelBand == CHANNEL_BAND_5) {
        maxGainOFDM = adapter_p->rfSpecificConfig_p->txPowerGainOFDM5;
    } else
#endif
    {
        maxGainOFDM = adapter_p->rfSpecificConfig_p->txPowerGainOFDM24;
    }

    if (configEntry_p) {
#if SUPPORT_802_11A
        if (channelBand == CHANNEL_BAND_24)
#endif
        {
            if (configEntry_p->txPowerGainCCK.gainMax) {
                maxGainCCK = configEntry_p->txPowerGainCCK.gainMax;
                maxTxPower_dBm = configEntry_p->txPowerGainCCK.maxTxPower_dBm;
            }
        }

        if (configEntry_p->txPowerGainOFDM.gainMax) {
            maxGainOFDM = configEntry_p->txPowerGainOFDM.gainMax;
            maxTxPower_dBm = configEntry_p->txPowerGainOFDM.maxTxPower_dBm;
        }

        if (txPowerLimit != 0xFF) {
            gainOFDMValid = nicComputeTxPowerGain(adapter_p,
                &configEntry_p->txPowerGainOFDM, txPowerLimit, &gainOFDM);

#if SUPPORT_802_11A
            if (channelBand == CHANNEL_BAND_5) {
                if (gainOFDMValid) {
                    useMaxTxPower = FALSE;
                    gainCCK = maxGainCCK;
                }
            } else
#endif
            {
                gainCCKValid = nicComputeTxPowerGain(adapter_p,
                    &configEntry_p->txPowerGainCCK, txPowerLimit, &gainCCK);

                if (gainOFDMValid && gainCCKValid) {
                    useMaxTxPower = FALSE;
                }
            }
        }
    }

    if (useMaxTxPower) {
        gainCCK = maxGainCCK;
        gainOFDM = maxGainOFDM;

        adapter_p->currentTxPower_dBm = maxTxPower_dBm;

        DBGLOG(HW, TRACE,
            ("Tx power gain: OFDM=0x%02x, CCK=0x%02x (max, %d dBm)\n",
            gainOFDM, gainCCK, adapter_p->currentTxPower_dBm));
    } else {
        adapter_p->currentTxPower_dBm = txPowerLimit;

        DBGLOG(HW, TRACE, ("Tx power gain: OFDM=0x%02x, CCK=0x%02x (%d dBm)\n",
            gainOFDM, gainCCK, adapter_p->currentTxPower_dBm));
    }

    temp32 = ((UINT_32) gainCCK & RFTPCR_CCK_POWER_MASK) |
        (((UINT_32) gainOFDM << 8) & RFTPCR_OFDM_POWER_MASK);

    NIC_MCR_WRITE(adapter_p, MCR_RFTPCR, temp32);

    NIC_MCR_READ(adapter_p, MCR_BGCR0, &temp32);

    temp32 &= ~BGCR0_TRANSMISSION_POWER;

    if ((temp32 & BGCR0_BEACON_TX_RATE) & RATE_CODE_OFDM_MASK) {
        temp32 |= ((UINT_32) gainOFDM << 8) & BGCR0_TRANSMISSION_POWER;
    } else {
        temp32 |= ((UINT_32) gainCCK << 8) & BGCR0_TRANSMISSION_POWER;
    }

    NIC_MCR_WRITE(adapter_p, MCR_BGCR0, temp32);

    adapter_p->txPowerGainForCCK = gainCCK;
    adapter_p->txPowerGainForOFDM = gainOFDM;
}   /* nicSetTxPowerGain */

/*_________________________________________________________________________
**  nicReadRfCalSetting
**
**  descriptions: This routine will read the RF TX/RX calibration setting
**                stored in EEPROM.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**  return:
**  note:
**__________________________________________________________________________*/
VOID
nicReadRfCalSetting(IN PMP_ADAPTER adapter_p)
{
    UINT i, j;
    UINT_16 eepromWordValue;
    UINT_8 checksum;
    UINT_16 calChannelMap;
    PNIC_RF_CAL_GROUP group_p;
    PNIC_PER_CHANNEL_CONFIG configEntry_p;
    UINT_8 tmpBuffer[(EEPROM_RF_CAL_RX_END - EEPROM_RF_CAL_RX_START + 1) * 2];
#if SUPPORT_802_11A
    UINT k;
#endif

    DEBUGFUNC("nicReadTxPowerSetting");


    ASSERT(EEPROM_RF_CAL_TX_END - EEPROM_RF_CAL_TX_START <=
        EEPROM_RF_CAL_RX_END - EEPROM_RF_CAL_RX_START);

    j = 0;
    for (i = EEPROM_RF_CAL_TX_START; i <= EEPROM_RF_CAL_TX_END; i++) {
        eepromWordValue = ReadEEProm16(adapter_p, (UINT_8) i);

        tmpBuffer[j++] = (UINT_8) (eepromWordValue & 0x00FF);
        tmpBuffer[j++] = (UINT_8) (eepromWordValue >> 8);
    }

    ASSERT(j == RF_CAL_TX_CR_NO * 10);

    calChannelMap = ReadEEProm16(adapter_p, EEPROM_RF_CAL_TX_MAP);

    checksum = 0;
    for (i = 0; i < j; i++) {
        checksum += tmpBuffer[i];
    }

    checksum += (UINT_8) (calChannelMap & 0x00FF);
    checksum += (UINT_8) (calChannelMap >> 8);

    eepromWordValue = ReadEEProm16(adapter_p, EEPROM_RF_CAL_TX_TABLE_CHECK_SUM);
    checksum += (UINT_8) (eepromWordValue & 0x00FF);
    checksum += (UINT_8) (eepromWordValue >> 8);

    if (checksum == 0xFF) {
        for (i = 0; i < sizeof(nicRFCalGroup) / sizeof(NIC_RF_CAL_GROUP); i++) {
            group_p = &nicRFCalGroup[i];

            if ((calChannelMap & group_p->validMask) == 0) {
                continue;
            }

            for (j = 0; j < group_p->channelNumListSize; j++) {
#if SUPPORT_802_11A
                if (group_p->channelBand == CHANNEL_BAND_5) {
                    if (!nicGetPerChannelConfigIndex5(group_p->channelNumList[j],
                        &k)) {
                        continue;
                    }

                    configEntry_p = &adapter_p->nicPerChannelConfig5[k];
                } else
#endif
                {
                    ASSERT(group_p->channelNumList[j]);

                    configEntry_p = &adapter_p->nicPerChannelConfig24[group_p->channelNumList[j] - 1];
                }

                configEntry_p->txCalEnabled = TRUE;
                NdisMoveMemory(configEntry_p->txCalSet,
                    &tmpBuffer[i * RF_CAL_TX_CR_NO], RF_CAL_TX_CR_NO);
            }
        }
    }
#if DBG
    else {
        INITLOG(("RF TX Calibration checksum error (0x%02x)\n", checksum));
    }
#endif

    j = 0;
    for (i = EEPROM_RF_CAL_RX_START; i <= EEPROM_RF_CAL_RX_END; i++) {
        eepromWordValue = ReadEEProm16(adapter_p, (UINT_8) i);

        tmpBuffer[j++] = (UINT_8) (eepromWordValue & 0x00FF);
        tmpBuffer[j++] = (UINT_8) (eepromWordValue >> 8);
    }

    ASSERT(j == RF_CAL_RX_CR_NO * 10);

    calChannelMap = ReadEEProm16(adapter_p, EEPROM_RF_CAL_RX_MAP);

    checksum = 0;
    for (i = 0; i < j; i++) {
        checksum += tmpBuffer[i];
    }

    checksum += (UINT_8) (calChannelMap & 0x00FF);
    checksum += (UINT_8) (calChannelMap >> 8);

    eepromWordValue = ReadEEProm16(adapter_p, EEPROM_RF_CAL_RX_TABLE_CHECK_SUM);
    checksum += (UINT_8) (eepromWordValue & 0x00FF);
    checksum += (UINT_8) (eepromWordValue >> 8);

    if (checksum == 0xFF) {
        for (i = 0; i < sizeof(nicRFCalGroup) / sizeof(NIC_RF_CAL_GROUP); i++) {
            group_p = &nicRFCalGroup[i];

            if ((calChannelMap & group_p->validMask) == 0) {
                continue;
            }

            for (j = 0; j < group_p->channelNumListSize; j++) {
#if SUPPORT_802_11A
                if (group_p->channelBand == CHANNEL_BAND_5) {
                    if (!nicGetPerChannelConfigIndex5(group_p->channelNumList[j],
                        &k)) {
                        continue;
                    }

                    configEntry_p = &adapter_p->nicPerChannelConfig5[k];
                } else
#endif
                {
                    ASSERT(group_p->channelNumList[j]);

                    configEntry_p = &adapter_p->nicPerChannelConfig24[group_p->channelNumList[j] - 1];
                }

                configEntry_p->rxCalEnabled = TRUE;
                NdisMoveMemory(configEntry_p->rxCalSet,
                    &tmpBuffer[i * RF_CAL_RX_CR_NO], RF_CAL_RX_CR_NO);
            }
        }
    }
#if DBG
    else {
        INITLOG(("RF RX Calibration check sum error (0x%02x)\n", checksum));
    }
#endif

#if DBG
    DBGLOG(INIT, INFO, ("RF calibration for 2.4 GHz band:\n"));
    for (i = 0; i < MAX_NUM_2G_RF_CHANNELS; i++) {
        configEntry_p = &adapter_p->nicPerChannelConfig24[i];

        if (configEntry_p->txCalEnabled) {
            DBGLOG(INIT, INFO, ("CH %d: Tx calibration enabled\n", i + 1));
            DBGLOG_MEM8(INIT, INFO, configEntry_p->txCalSet, RF_CAL_TX_CR_NO);
        } else {
            DBGLOG(INIT, INFO, ("CH %d: Tx calibration disabled\n", i + 1));
        }

        if (configEntry_p->rxCalEnabled) {
            DBGLOG(INIT, INFO, ("CH %d: Rx calibration enabled\n", i + 1));
            DBGLOG_MEM8(INIT, INFO, configEntry_p->rxCalSet, RF_CAL_RX_CR_NO);
        } else {
            DBGLOG(INIT, INFO, ("CH %d: Rx calibration disabled\n", i + 1));
        }
    }

#if SUPPORT_802_11A
    DBGLOG(INIT, INFO, ("RF calibration for 5 GHz band:\n"));
    for (i = 0; i < sizeof(nicChannelNumMappingTable5); i++) {
        configEntry_p = &adapter_p->nicPerChannelConfig5[i];

        if (configEntry_p->txCalEnabled) {
            DBGLOG(INIT, INFO, ("CH %d: Tx calibration enabled\n",
                nicChannelNumMappingTable5[i]));
            DBGLOG_MEM8(INIT, INFO, configEntry_p->txCalSet, RF_CAL_TX_CR_NO);
        } else {
            DBGLOG(INIT, INFO, ("CH %d: Tx calibration disabled\n",
                nicChannelNumMappingTable5[i]));
        }

        if (configEntry_p->rxCalEnabled) {
            DBGLOG(INIT, INFO, ("CH %d: Rx calibration enabled\n",
                nicChannelNumMappingTable5[i]));
            DBGLOG_MEM8(INIT, INFO, configEntry_p->rxCalSet, RF_CAL_RX_CR_NO);
        } else {
            DBGLOG(INIT, INFO, ("CH %d: Rx calibration disabled\n",
                nicChannelNumMappingTable5[i]));
        }
    }
#endif
#endif
} /* nicReadRfCalSetting */

/*_________________________________________________________________________
**  nicReadPerChannelConfig
**
**  descriptions: This routine is called to read the per-channel
**                configuration information from the EEPROM.
**  parameters:
**      adapter_p - Pointer to the Adapter structure
**  return:
**  note:
**__________________________________________________________________________*/
VOID
nicReadPerChannelConfig(IN PMP_ADAPTER adapter_p)
{
    nicReadTxPowerSetting(adapter_p);

    nicReadRfCalSetting(adapter_p);
}   /* nicReadPerChannelConfig */

